@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -258,7 +257,7 @@ new_catalog_button_clicked_cb (GtkWidget  *widget,
 	GthFileData *selected_parent;
 	GFile       *parent;
 	GFile       *gio_parent;
-	GError      *error;
+	GError      *error = NULL;
 	GFile       *gio_file;
 
 	name = _gtk_request_dialog_run (GTK_WINDOW (data->dialog),
