@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -192,7 +191,7 @@ base_write_to_doc (GthCatalog  *catalog,
 	if (catalog->priv->name != NULL)
 		dom_element_append_child (root, dom_document_create_element_with_text (doc, catalog->priv->name, "name", NULL));
 
-	if (gth_datetime_valid (catalog->priv->date_time)) {
+	if (gth_datetime_valid_date (catalog->priv->date_time)) {
 		char *s;
 
 		s = gth_datetime_to_exif_date (catalog->priv->date_time);
@@ -218,7 +217,7 @@ base_write_to_doc (GthCatalog  *catalog,
 			char  *uri;
 
 			uri = g_file_get_uri (file);
-			dom_element_append_child (DOM_ELEMENT (node), dom_document_create_element (doc, "file", "uri", uri, NULL));
+			dom_element_append_child (node, dom_document_create_element (doc, "file", "uri", uri, NULL));
 
 			g_free (uri);
 		}
@@ -334,7 +333,7 @@ void
 gth_catalog_set_date (GthCatalog  *catalog,
 		      GthDateTime *date_time)
 {
-	if (gth_datetime_valid (date_time))
+	if (g_date_valid (date_time->date))
 		g_date_set_dmy (catalog->priv->date_time->date,
 				g_date_get_day (date_time->date),
 				g_date_get_month (date_time->date),
@@ -375,20 +374,6 @@ gth_catalog_get_order (GthCatalog *catalog,
 }
 
 
-void
-gth_catalog_set_for_date (GthCatalog  *catalog,
-			  GthDateTime *date_time)
-{
-	GFile *catalog_file;
-
-	gth_catalog_set_date (catalog, date_time);
-	catalog_file = gth_catalog_get_file_for_date (date_time);
-	gth_catalog_set_file (catalog, catalog_file);
-
-	g_object_unref (catalog_file);
-}
-
-
 void
 gth_catalog_load_from_data (GthCatalog  *catalog,
 			    const void  *buffer,
@@ -656,7 +641,7 @@ get_display_name (GFile       *file,
 		 g_string_append (display_name, _("Catalogs"));
 	}
 	else {
-		if ((name == NULL) && ! gth_datetime_valid (date_time)) {
+		if ((name == NULL) && ! gth_datetime_valid_date (date_time)) {
 			char *name;
 			char *utf8_name;
 
@@ -671,7 +656,7 @@ get_display_name (GFile       *file,
 			if (name != NULL)
 				g_string_append (display_name, name);
 
-			if (gth_datetime_valid (date_time)) {
+			if (gth_datetime_valid_date (date_time)) {
 				char *formatted;
 
 				formatted = gth_datetime_strftime (date_time, "%x");
@@ -733,7 +718,7 @@ update_standard_attributes (GFile       *file,
 	char *display_name;
 	char *edit_name;
 
-	if (gth_datetime_valid (date_time)) {
+	if (gth_datetime_valid_date (date_time)) {
 		char *sort_order_s;
 		int   sort_order;
 
@@ -779,7 +764,7 @@ gth_catalog_update_metadata (GthCatalog  *catalog,
 
 	/* general::event-date */
 
-	if (gth_datetime_valid (catalog->priv->date_time)) {
+	if (gth_datetime_valid_date (catalog->priv->date_time)) {
 		GObject *metadata;
 		char    *raw;
 		char    *formatted;
@@ -1129,22 +1114,20 @@ gth_catalog_load_from_file_async (GFile         *file,
 
 
 GFile *
-gth_catalog_get_file_for_date (GthDateTime *date_time)
+gth_catalog_get_file_for_date (GthDateTime *date_time,
+			       const char  *extension)
 {
 	char  *year;
 	char  *uri;
-	GFile *base;
 	char  *display_name;
 	GFile *catalog_file;
 
 	year = gth_datetime_strftime (date_time, "%Y");
 	uri = g_strconcat ("catalog:///", year, "/", NULL);
-	base = g_file_new_for_uri (uri);
 	display_name = gth_datetime_strftime (date_time, "%Y-%m-%d");
-	catalog_file = _g_file_new_for_display_name (uri, display_name, ".catalog");
+	catalog_file = _g_file_new_for_display_name (uri, display_name, extension);
 
 	g_free (display_name);
-	g_object_unref (base);
 	g_free (uri);
 	g_free (year);
 
@@ -1152,6 +1135,22 @@ gth_catalog_get_file_for_date (GthDateTime *date_time)
 }
 
 
+GFile *
+gth_catalog_get_file_for_tag (const char *tag,
+			      const char *extension)
+{
+	char  *uri;
+	GFile *catalog_file;
+
+	uri = g_strconcat ("catalog:///", _("Tags"), "/", NULL);
+	catalog_file = _g_file_new_for_display_name (uri, tag, extension);
+
+	g_free (uri);
+
+	return catalog_file;
+}
+
+
 GthCatalog *
 gth_catalog_load_from_file (GFile *file)
 {
