@@ -16,24 +16,21 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
 #include "gth-change-date-task.h"
 
 
-#define HOURS_TO_SECONDS(h) ((h) * 3600)
-
-
 struct _GthChangeDateTaskPrivate {
+	GFile           *location;
 	GList           *files; /* GFile */
 	GList           *file_list; /* GthFileData */
 	GthChangeFields  fields;
 	GthChangeType    change_type;
 	GthDateTime     *date_time;
-	int              timezone_offset;
+	int              time_offset;
 	int              n_files;
 	int              n_current;
 	GList           *current;
@@ -50,6 +47,7 @@ gth_change_date_task_finalize (GObject *object)
 
 	self = GTH_CHANGE_DATE_TASK (object);
 
+	_g_object_unref (self->priv->location);
 	_g_object_list_unref (self->priv->file_list);
 	_g_object_list_unref (self->priv->files);
 	gth_datetime_free (self->priv->date_time);
@@ -85,53 +83,79 @@ set_date_time_from_change_type (GthChangeDateTask *self,
 static void
 update_modification_time (GthChangeDateTask *self)
 {
-	GthDateTime *date_time;
-	GList       *scan;
-	GError      *error = NULL;
 
-	if ((self->priv->fields & GTH_CHANGE_LAST_MODIFIED_DATE) == 0) {
-		gth_task_completed (GTH_TASK (self), NULL);
-		return;
-	}
+	GList      *scan;
+	GError     *error = NULL;
+	GthMonitor *monitor;
+	GList      *files;
 
-	date_time = gth_datetime_new ();
-	for (scan = self->priv->file_list; scan; scan = scan->next) {
-		GthFileData *file_data = scan->data;
-		GTimeVal     timeval;
+	if ((self->priv->fields & GTH_CHANGE_LAST_MODIFIED_DATE) == GTH_CHANGE_LAST_MODIFIED_DATE) {
+		GthDateTime *date_time;
 
-		gth_datetime_clear (date_time);
-		if (self->priv->change_type == GTH_CHANGE_ADJUST_TIMEZONE)
-			set_date_time_from_change_type (self, date_time, GTH_CHANGE_TO_FILE_MODIFIED_DATE, file_data);
-		else
-			set_date_time_from_change_type (self, date_time, self->priv->change_type, file_data);
-		if (! gth_datetime_valid (date_time))
-			continue;
+		date_time = gth_datetime_new ();
+		for (scan = self->priv->file_list; scan; scan = scan->next) {
+			GthFileData *file_data = scan->data;
+			GTimeVal     timeval;
 
-		if (gth_datetime_to_timeval (date_time, &timeval)) {
-			if (self->priv->change_type == GTH_CHANGE_ADJUST_TIMEZONE)
-				timeval.tv_sec += HOURS_TO_SECONDS (self->priv->timezone_offset);
-			if (! _g_file_set_modification_time (file_data->file,
-							     &timeval,
-							     gth_task_get_cancellable (GTH_TASK (self)),
-							     &error))
-			{
-				break;
+			gth_datetime_clear (date_time);
+			if (self->priv->change_type == GTH_CHANGE_ADJUST_TIME)
+				set_date_time_from_change_type (self, date_time, GTH_CHANGE_TO_FILE_MODIFIED_DATE, file_data);
+			else
+				set_date_time_from_change_type (self, date_time, self->priv->change_type, file_data);
+
+			/* Read the time from the attribute if it's not valid in
+			 * date_time. */
+
+			if (! gth_time_valid (date_time->time)) {
+				GTimeVal    *original_modification_time;
+				GthDateTime *original_date_time;
+
+				original_modification_time = gth_file_data_get_modification_time (file_data);
+				original_date_time = gth_datetime_new ();
+				gth_datetime_from_timeval (original_date_time, original_modification_time);
+				*date_time->time = *original_date_time->time;
+
+				gth_datetime_free (original_date_time);
+			}
+
+			if (! gth_datetime_valid (date_time))
+				continue;
+
+			if (gth_datetime_to_timeval (date_time, &timeval)) {
+				if (self->priv->change_type == GTH_CHANGE_ADJUST_TIME)
+					timeval.tv_sec += self->priv->time_offset;
+				if (! _g_file_set_modification_time (file_data->file,
+								     &timeval,
+								     gth_task_get_cancellable (GTH_TASK (self)),
+								     &error))
+				{
+					break;
+				}
 			}
 		}
+
+		gth_datetime_free (date_time);
 	}
+
+	monitor = gth_main_get_default_monitor ();
+	files = gth_file_data_list_to_file_list (self->priv->file_list);
+	gth_monitor_folder_changed (monitor, self->priv->location, files, GTH_MONITOR_EVENT_CHANGED);
+
 	gth_task_completed (GTH_TASK (self), error);
 
-	gth_datetime_free (date_time);
+	_g_object_list_unref (files);
 }
 
 
 static void
-write_metadata_reasy_cb (GError   *error,
-			 gpointer  user_data)
+write_metadata_ready_cb (GObject      *source_object,
+                	 GAsyncResult *result,
+                	 gpointer      user_data)
 {
 	GthChangeDateTask *self = user_data;
+	GError            *error = NULL;
 
-	if (error != NULL) {
+	if (! _g_write_metadata_finish (result, &error)) {
 		gth_task_completed (GTH_TASK (self), error);
 		return;
 	}
@@ -149,41 +173,61 @@ static void
 set_date_metadata (GthFileData *file_data,
 		   const char  *attribute,
 		   GthDateTime *date_time,
-		   int          timezone_offset)
+		   int          time_offset)
 {
-	char    *raw;
-	char    *formatted;
-	GObject *metadata;
+	GthDateTime *new_date_time;
 
-	if (timezone_offset != 0) {
-		GTimeVal     timeval;
-		GthDateTime *adjusted_date;
+	new_date_time = gth_datetime_new ();
+	gth_datetime_copy (date_time, new_date_time);
 
-		gth_datetime_to_timeval (date_time, &timeval);
-		timeval.tv_sec += HOURS_TO_SECONDS (timezone_offset);
-		adjusted_date = gth_datetime_new ();
-		gth_datetime_from_timeval (adjusted_date, &timeval);
-		raw = gth_datetime_to_exif_date (adjusted_date);
-		formatted = gth_datetime_strftime (adjusted_date, "%x");
+	if (time_offset != 0) {
+		GTimeVal timeval;
 
-		gth_datetime_free (adjusted_date);
+		gth_datetime_to_timeval (new_date_time, &timeval);
+		timeval.tv_sec += time_offset;
+		gth_datetime_from_timeval (new_date_time, &timeval);
 	}
 	else {
-		raw = gth_datetime_to_exif_date (date_time);
-		formatted = gth_datetime_strftime (date_time, "%x");
+		/* Read the time from the attribute if it's not valid in
+		 * date_time. */
+
+		if (! gth_time_valid (new_date_time->time)) {
+			GthMetadata *original_date;
+
+			original_date = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, attribute);
+			if (original_date != NULL) {
+				GthDateTime *original_date_time;
+
+				original_date_time = gth_datetime_new ();
+				gth_datetime_from_exif_date (original_date_time, gth_metadata_get_raw (original_date));
+				*new_date_time->time = *original_date_time->time;
+
+				gth_datetime_free (original_date_time);
+			}
+		}
 	}
 
-	metadata = (GObject *) gth_metadata_new ();
-	g_object_set (metadata,
-		      "id", attribute,
-		      "raw", raw,
-		      "formatted", formatted,
-		      NULL);
-	g_file_info_set_attribute_object (file_data->info, attribute, metadata);
+	if (gth_datetime_valid (new_date_time)) {
+		char    *raw;
+		char    *formatted;
+		GObject *metadata;
 
-	g_object_unref (metadata);
-	g_free (formatted);
-	g_free (raw);
+		raw = gth_datetime_to_exif_date (new_date_time);
+		formatted = gth_datetime_strftime (new_date_time, "%x");
+		metadata = (GObject *) gth_metadata_new ();
+		g_object_set (metadata,
+			      "id", attribute,
+			      "raw", raw,
+			      "formatted", formatted,
+			      NULL);
+		g_file_info_set_attribute_object (file_data->info, attribute, metadata);
+
+		g_object_unref (metadata);
+		g_free (formatted);
+		g_free (raw);
+	}
+
+	gth_datetime_free (new_date_time);
 }
 
 
@@ -240,24 +284,24 @@ info_ready_cb (GList    *files,
 	for (scan = self->priv->file_list; scan; scan = scan->next) {
 		GthFileData *file_data = scan->data;
 
-		if (self->priv->change_type == GTH_CHANGE_ADJUST_TIMEZONE) {
+		if (self->priv->change_type == GTH_CHANGE_ADJUST_TIME) {
 			if (self->priv->fields & GTH_CHANGE_COMMENT_DATE) {
 				gth_datetime_clear (date_time);
 				set_date_time_from_field (self, date_time, GTH_CHANGE_COMMENT_DATE, file_data);
 				if (gth_datetime_valid (date_time))
-					set_date_metadata (file_data, "general::datetime", date_time, self->priv->timezone_offset);
+					set_date_metadata (file_data, "general::datetime", date_time, self->priv->time_offset);
 			}
 			if (self->priv->fields & GTH_CHANGE_EXIF_DATETIMEORIGINAL_TAG) {
 				gth_datetime_clear (date_time);
 				set_date_time_from_field (self, date_time, GTH_CHANGE_EXIF_DATETIMEORIGINAL_TAG, file_data);
 				if (gth_datetime_valid (date_time))
-					set_date_metadata (file_data, "Exif::Photo::DateTimeOriginal", date_time, self->priv->timezone_offset);
+					set_date_metadata (file_data, "Exif::Photo::DateTimeOriginal", date_time, self->priv->time_offset);
 			}
 		}
 		else {
 			gth_datetime_clear (date_time);
 			set_date_time_from_change_type (self, date_time, self->priv->change_type, file_data);
-			if (gth_datetime_valid (date_time)) {
+			if (g_date_valid (date_time->date)) {
 				if (self->priv->fields & GTH_CHANGE_COMMENT_DATE) {
 					set_date_metadata (file_data, "general::datetime", date_time, 0);
 				}
@@ -278,9 +322,10 @@ info_ready_cb (GList    *files,
 
 		attributes = _g_string_array_join (attribute_v, ",");
 		_g_write_metadata_async (self->priv->file_list,
+					 (self->priv->fields & GTH_CHANGE_EXIF_DATETIMEORIGINAL_TAG) ? GTH_METADATA_WRITE_FORCE_EMBEDDED : GTH_METADATA_WRITE_DEFAULT,
 					 attributes,
 					 gth_task_get_cancellable (GTH_TASK (self)),
-					 write_metadata_reasy_cb,
+					 write_metadata_ready_cb,
 					 self);
 
 		g_free (attributes);
@@ -299,8 +344,7 @@ gth_change_date_task_exec (GthTask *task)
 	GthChangeDateTask *self = GTH_CHANGE_DATE_TASK (task);
 
 	_g_query_all_metadata_async (self->priv->files,
-				     FALSE,
-				     TRUE,
+				     GTH_LIST_DEFAULT,
 				     "*",
 				     gth_task_get_cancellable (task),
 				     info_ready_cb,
@@ -362,21 +406,23 @@ gth_change_date_task_get_type (void)
 
 
 GthTask *
-gth_change_date_task_new (GList             *files, /* GthFileData */
+gth_change_date_task_new (GFile             *location,
+			  GList             *files, /* GthFileData */
 			  GthChangeFields    fields,
 			  GthChangeType      change_type,
 			  GthDateTime       *date_time,
-			  int                timezone_offset)
+			  int                time_offset)
 {
 	GthChangeDateTask *self;
 
 	self = GTH_CHANGE_DATE_TASK (g_object_new (GTH_TYPE_CHANGE_DATE_TASK, NULL));
+	self->priv->location = g_file_dup (location);
 	self->priv->files = gth_file_data_list_to_file_list (files);
 	self->priv->fields = fields;
 	self->priv->change_type = change_type;
 	if (date_time != NULL)
 		gth_datetime_copy (date_time, self->priv->date_time);
-	self->priv->timezone_offset = timezone_offset;
+	self->priv->time_offset = time_offset;
 
 	return (GthTask *) self;
 }
