@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -27,7 +26,6 @@
 #include "dlg-comments-preferences.h"
 #include "gth-comment.h"
 #include "gth-metadata-provider-comment.h"
-#include "gth-test-category.h"
 #include "preferences.h"
 
 
@@ -38,12 +36,12 @@ GthMetadataCategory comments_metadata_category[] = {
 
 
 GthMetadataInfo comments_metadata_info[] = {
-	{ "comment::caption", "", "comment", 1, NULL, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::note", "", "comment", 2, NULL, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::place", "", "comment", 3, NULL, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::time", "", "comment", 4, NULL, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::categories", "", "comment", 5, NULL, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::rating", "", "comment", 6, NULL, GTH_METADATA_ALLOW_NOWHERE },
+	{ "comment::caption", N_("Title"), "comment", 1, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
+	{ "comment::note", N_("Description"), "comment", 2, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
+	{ "comment::place", N_("Place"), "comment", 3, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
+	{ "comment::time", N_("Date"), "comment", 4, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
+	{ "comment::categories", N_("Tags"), "comment", 5, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
+	{ "comment::rating", N_("Rating"), "comment", 6, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
 	{ NULL, NULL, NULL, 0, 0 }
 };
 
@@ -119,6 +117,8 @@ comments__read_metadata_ready_cb (GthFileData *file_data,
 		for (scan = gth_string_list_get_list (comment_categories); scan; scan = scan->next)
 			gth_comment_add_category (comment, (char *) scan->data);
 
+	gth_comment_set_rating (comment, g_file_info_get_attribute_int32 (file_data->info, "comment::rating"));
+
 	/* sync embedded data and .comment data if required */
 
 	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "general::description");
@@ -221,8 +221,10 @@ gthumb_extension_activate (void)
 	gth_main_register_object (GTH_TYPE_TEST,
 				  "comment::category",
 				  GTH_TYPE_TEST_CATEGORY,
+				  "attributes", "comment::categories",
 				  "display-name", _("Tag"),
 				  NULL);
+
 	gth_hook_add_callback ("add-sidecars", 10, G_CALLBACK (comments__add_sidecars_cb), NULL);
 	gth_hook_add_callback ("read-metadata-ready", 10, G_CALLBACK (comments__read_metadata_ready_cb), NULL);
 }
