@@ -170,6 +170,16 @@ const char *_KEYWORDS_TAG_NAMES[] = {
 	NULL
 };
 
+/* Some evil camera fill in the ImageDescription or UserComment fields
+   with useless fluff. Try to filter these out, so they do not show up
+   as comments */
+const char *stupid_comment_filter[] = {
+	"OLYMPUS DIGITAL CAMERA",
+	"SONY DSC",
+	"KONICA MINOLTA DIGITAL CAMERA",
+	"MINOLTA DIGITAL CAMERA",
+	NULL };
+
 
 inline static char *
 exiv2_key_from_attribute (const char *attribute)
@@ -458,9 +468,38 @@ set_string_list_attribute_from_tagset (GFileInfo  *info,
 }
 
 
+static void
+clear_studip_comments_from_tagset (GFileInfo  *info,
+				   const char *tagset[])
+{
+	int i;
+
+	for (i = 0; tagset[i] != NULL; i++) {
+		GObject    *metadata;
+		const char *value;
+		int         j;
+
+		metadata = g_file_info_get_attribute_object (info, tagset[i]);
+		if ((metadata == NULL) || ! GTH_IS_METADATA (metadata))
+			continue;
+
+		value = gth_metadata_get_formatted (GTH_METADATA (metadata));
+		for (j = 0; stupid_comment_filter[j] != NULL; j++) {
+			if (strstr (value, stupid_comment_filter[j]) == value) {
+				g_file_info_remove_attribute (info, tagset[i]);
+				break;
+			}
+		}
+	}
+}
+
+
 static void
 set_attributes_from_tagsets (GFileInfo *info)
 {
+	clear_studip_comments_from_tagset (info, _DESCRIPTION_TAG_NAMES);
+	clear_studip_comments_from_tagset (info, _TITLE_TAG_NAMES);
+
 	set_attribute_from_tagset (info, "general::datetime", _LAST_DATE_TAG_NAMES);
 	if (g_file_info_get_attribute_object (info, "general::datetime") == NULL)
 		set_attribute_from_tagset (info, "general::datetime", _ORIGINAL_DATE_TAG_NAMES);
@@ -497,19 +536,19 @@ set_attributes_from_tagsets (GFileInfo *info)
 static const char *
 get_exif_default_category (const Exiv2::Exifdatum &md)
 {
+#if EXIV2_TEST_VERSION(0, 21, 0)
+	if (Exiv2::ExifTags::isMakerGroup(md.groupName()))
+#else
 	if (Exiv2::ExifTags::isMakerIfd(md.ifdId()))
+#endif
 		return "Exif::MakerNotes";
 
-	switch (md.ifdId()) {
-	case Exiv2::ifd1Id:
+	if (md.groupName().compare("Thumbnail") == 0)
 		return "Exif::Thumbnail";
-	case Exiv2::gpsIfdId:
+	else if (md.groupName().compare("GPSInfo") == 0)
 		return "Exif::GPS";
-	case Exiv2::iopIfdId:
+	else if (md.groupName().compare("Iop") == 0)
 		return "Exif::Versions";
-	default:
-		break;
-	}
 
 	return "Exif::Other";
 }
@@ -531,7 +570,11 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 			stringstream description;
 			if (! md->tagLabel().empty())
 				description << md->tagLabel();
-			else if (md->ifdId () > Exiv2::ifd1Id)
+#if EXIV2_TEST_VERSION(0, 21, 0)
+			else if (Exiv2::ExifTags::isMakerGroup(md->groupName()))
+#else
+			else if (Exiv2::ExifTags::isMakerIfd(md->ifdId()))
+#endif
 				// Must be a MakerNote - include group name
 				description << md->groupName() << "." << md->tagName();
 			else
@@ -1135,9 +1178,9 @@ exiv2_generate_thumbnail (const char *uri,
 
 		Exiv2::ExifData &ed = image->exifData();
 
-		long orientation = ed["Exif.Image.Orientation"].toLong();
-		long image_width = ed["Exif.Photo.PixelXDimension"].toLong();
-		long image_height = ed["Exif.Photo.PixelYDimension"].toLong();
+		long orientation = (ed["Exif.Image.Orientation"].count() > 0) ? ed["Exif.Image.Orientation"].toLong() : 1;
+		long image_width = (ed["Exif.Photo.PixelXDimension"].count() > 0) ? ed["Exif.Photo.PixelXDimension"].toLong() : -1;
+		long image_height = (ed["Exif.Photo.PixelYDimension"].count() > 0) ? ed["Exif.Photo.PixelYDimension"].toLong() : -1;
 
 		if ((orientation != 1) || (image_width <= 0) || (image_height <= 0))
 			return NULL;
