@@ -187,7 +187,7 @@ export_dialog_response_cb (GtkDialog *dialog,
 
 	switch (response_id) {
 	case GTK_RESPONSE_HELP:
-		show_help_dialog (GTK_WINDOW (data->browser), "export-to-facebook");
+		show_help_dialog (GTK_WINDOW (data->browser), "gthumb-export-social");
 		break;
 
 	case GTK_RESPONSE_DELETE_EVENT:
@@ -549,7 +549,7 @@ dlg_export_to_facebook (GthBrowser *browser,
 
 	/* Set the widget data */
 
-	data->list_view = gth_file_list_new (GTH_FILE_LIST_TYPE_NO_SELECTION, FALSE);
+	data->list_view = gth_file_list_new (gth_icon_view_new (), GTH_FILE_LIST_TYPE_NO_SELECTION, FALSE);
 	gth_file_list_set_thumb_size (GTH_FILE_LIST (data->list_view), 112);
 	gth_file_view_set_spacing (GTH_FILE_VIEW (gth_file_list_get_view (GTH_FILE_LIST (data->list_view))), 0);
 	gth_file_list_enable_thumbs (GTH_FILE_LIST (data->list_view), TRUE);
