@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -27,16 +26,16 @@
 #include "gth-copy-task.h"
 #include "gth-delete-task.h"
 #include "gth-duplicate-task.h"
+#include "preferences.h"
 
 
 static void
 _gth_browser_create_new_folder (GthBrowser *browser,
 				GFile      *parent)
 {
-	char          *folder_name;
-	GthFileSource *file_source;
-	GError        *error = NULL;
-	GFile         *file;
+	char   *folder_name;
+	GError *error = NULL;
+	GFile  *new_folder;
 
 	folder_name = _gtk_request_dialog_run (GTK_WINDOW (browser),
 					       GTK_DIALOG_MODAL,
@@ -48,24 +47,30 @@ _gth_browser_create_new_folder (GthBrowser *browser,
 	if (folder_name == NULL)
 		return;
 
-	file_source = gth_main_get_file_source (parent);
-	file = _g_directory_create_unique (parent, folder_name, "", &error);
-	if (file != NULL) {
-		GList *list;
+	new_folder = g_file_get_child_for_display_name (parent, folder_name, &error);
+	if ((new_folder != NULL) && (g_file_make_directory (new_folder, NULL, &error))) {
+		GList       *list;
+		GtkWidget   *folder_tree;
+		GtkTreePath *path;
 
-		list = g_list_prepend (NULL, file);
+		list = g_list_prepend (NULL, new_folder);
 		gth_monitor_folder_changed (gth_main_get_default_monitor (),
 					    parent,
 					    list,
 					    GTH_MONITOR_EVENT_CREATED);
 
+		folder_tree = gth_browser_get_folder_tree (browser);
+		path = gth_folder_tree_get_path (GTH_FOLDER_TREE (folder_tree), parent);
+		gth_folder_tree_expand_row (GTH_FOLDER_TREE (folder_tree), path, FALSE);
+
+		gtk_tree_path_free (path);
 		g_list_free (list);
 	}
 	else
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (browser), _("Could not create the folder"), &error);
 
-	_g_object_unref (file);
-	g_object_unref (file_source);
+	_g_object_unref (new_folder);
+	g_free (folder_name);
 }
 
 
@@ -138,11 +143,16 @@ clipboard_get_cb (GtkClipboard     *clipboard,
 		  gpointer          user_data_or_owner)
 {
 	ClipboardData *clipboard_data = user_data_or_owner;
+	GdkAtom        targets[1];
+	int            n_targets;
 
-	if (gtk_targets_include_uri (&selection_data->target, 1)) {
+	targets[0] = gtk_selection_data_get_target (selection_data);
+	n_targets = 1;
+
+	if (gtk_targets_include_uri (targets, n_targets)) {
 		gtk_selection_data_set_uris (selection_data, clipboard_data->uris);
 	}
-	else if (gtk_targets_include_text (&selection_data->target, 1)) {
+	else if (gtk_targets_include_text (targets, n_targets)) {
 		char  *str;
 		gsize  len;
 
@@ -150,7 +160,7 @@ clipboard_get_cb (GtkClipboard     *clipboard,
 		gtk_selection_data_set_text (selection_data, str, len);
 		g_free (str);
 	}
-	else if (selection_data->target == GNOME_COPIED_FILES) {
+	else if (gtk_selection_data_get_target (selection_data) == GNOME_COPIED_FILES) {
 		char  *str;
 		gsize  len;
 
@@ -215,8 +225,13 @@ void
 gth_browser_activate_action_edit_cut_files (GtkAction  *action,
 					    GthBrowser *browser)
 {
-	GList *items;
-	GList *file_list;
+	GtkWidget *focused_widget;
+	GList     *items;
+	GList     *file_list;
+
+	focused_widget = gtk_window_get_focus (GTK_WINDOW (browser));
+	if ((focused_widget != NULL) && GTK_IS_EDITABLE (focused_widget))
+		return;
 
 	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
 	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
@@ -231,8 +246,13 @@ void
 gth_browser_activate_action_edit_copy_files (GtkAction  *action,
 					     GthBrowser *browser)
 {
-	GList *items;
-	GList *file_list;
+	GtkWidget *focused_widget;
+	GList     *items;
+	GList     *file_list;
+
+	focused_widget = gtk_window_get_focus (GTK_WINDOW (browser));
+	if ((focused_widget != NULL) && GTK_IS_EDITABLE (focused_widget))
+		return;
 
 	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
 	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
@@ -273,12 +293,19 @@ clipboard_received_cb (GtkClipboard     *clipboard,
 {
 	PasteData   *paste_data = user_data;
 	GthBrowser  *browser = paste_data->browser;
+	const char  *raw_data;
 	char       **clipboard_data;
 	int          i;
 	GthTask     *task;
 
-	clipboard_data = g_strsplit_set ((const char *) gtk_selection_data_get_data (selection_data), "\n\r", -1);
-	if (clipboard_data[0] == NULL) {
+	raw_data = (const char *) gtk_selection_data_get_data (selection_data);
+	if (raw_data == NULL) {
+		paste_data_free (paste_data);
+		return;
+	}
+
+	clipboard_data = g_strsplit_set (raw_data, "\n\r", -1);
+	if ((clipboard_data == NULL) || (clipboard_data[0] == NULL)) {
 		g_strfreev (clipboard_data);
 		paste_data_free (paste_data);
 		return;
@@ -292,7 +319,7 @@ clipboard_received_cb (GtkClipboard     *clipboard,
 	paste_data->files = g_list_reverse (paste_data->files);
 	paste_data->file_source = gth_main_get_file_source (paste_data->destination->file);
 
-	if (paste_data->cut && ! gth_file_source_can_cut (paste_data->file_source)) {
+	if (paste_data->cut && ! gth_file_source_can_cut (paste_data->file_source, paste_data->files->data)) {
 		GtkWidget *dialog;
 		int        response;
 
@@ -330,8 +357,13 @@ void
 gth_browser_activate_action_edit_paste (GtkAction  *action,
 					GthBrowser *browser)
 {
+	GtkWidget *focused_widget;
 	PasteData *paste_data;
 
+	focused_widget = gtk_window_get_focus (GTK_WINDOW (browser));
+	if ((focused_widget != NULL) && GTK_IS_EDITABLE (focused_widget))
+		return;
+
 	paste_data = g_new0 (PasteData, 1);
 	paste_data->browser = g_object_ref (browser);
 	paste_data->destination = g_object_ref (gth_browser_get_location_data (browser));
@@ -410,18 +442,13 @@ delete_permanently_response_cb (GtkDialog *dialog,
 }
 
 
-void
-gth_browser_activate_action_edit_trash (GtkAction  *action,
-					GthBrowser *browser)
+static void
+trash_files (GtkWindow *window,
+	     GList     *file_list)
 {
-	GList    *items;
-	GList    *file_list = NULL;
 	GList    *scan;
-	GError   *error = NULL;
 	gboolean  moved_to_trash = TRUE;
-
-	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
-	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
+	GError   *error = NULL;
 
 	for (scan = file_list; scan; scan = scan->next) {
 		GthFileData *file_data = scan->data;
@@ -433,7 +460,7 @@ gth_browser_activate_action_edit_trash (GtkAction  *action,
 
 				g_clear_error (&error);
 
-				d = _gtk_yesno_dialog_new (GTK_WINDOW (browser),
+				d = _gtk_yesno_dialog_new (window,
 							   GTK_DIALOG_MODAL,
 							   _("The files cannot be moved to the Trash. Do you want to delete them permanently?"),
 							   GTK_STOCK_CANCEL,
@@ -446,7 +473,7 @@ gth_browser_activate_action_edit_trash (GtkAction  *action,
 
 				break;
 			}
-			_gtk_error_dialog_from_gerror_show (GTK_WINDOW (browser), _("Could not move the files to the Trash"), &error);
+			_gtk_error_dialog_from_gerror_show (window, _("Could not move the files to the Trash"), &error);
 			break;
 		}
 	}
@@ -455,33 +482,89 @@ gth_browser_activate_action_edit_trash (GtkAction  *action,
 		GList  *files;
 
 		files = gth_file_data_list_to_file_list (file_list);
-		notify_files_delete (GTK_WINDOW (browser), files);
+		notify_files_delete (window, files);
 
 		_g_object_list_unref (files);
 	}
+}
 
+
+static void
+trash_files_response_cb (GtkDialog *dialog,
+			 int        response_id,
+			 gpointer   user_data)
+{
+	GList *file_list = user_data;
+
+	if (response_id == GTK_RESPONSE_YES)
+		trash_files (gtk_window_get_transient_for (GTK_WINDOW (dialog)), file_list);
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 	_g_object_list_unref (file_list);
+}
+
+
+void
+gth_browser_activate_action_edit_trash (GtkAction  *action,
+					GthBrowser *browser)
+{
+	GList *items;
+	GList *file_list = NULL;
+
+	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
+	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
+
+	if (eel_gconf_get_boolean (PREF_MSG_CONFIRM_DELETION, DEFAULT_MSG_CONFIRM_DELETION)) {
+		int        file_count;
+		char      *prompt;
+		GtkWidget *d;
+
+		file_count = g_list_length (file_list);
+		if (file_count == 1) {
+			GthFileData *file_data = file_list->data;
+			prompt = g_strdup_printf (_("Are you sure you want to move \"%s\" to trash?"), g_file_info_get_display_name (file_data->info));
+		}
+		else
+			prompt = g_strdup_printf (ngettext("Are you sure you want to move to trash "
+							   "the %'d selected file?",
+							   "Are you sure you want to move to trash "
+							   "the %'d selected files?", file_count),
+						  file_count);
+
+		d = _gtk_message_dialog_new (GTK_WINDOW (browser),
+					     GTK_DIALOG_MODAL,
+					     GTK_STOCK_DIALOG_QUESTION,
+					     prompt,
+					     NULL,
+					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					     _("Mo_ve to Trash"), GTK_RESPONSE_YES,
+					     NULL);
+		g_signal_connect (d, "response", G_CALLBACK (trash_files_response_cb), file_list);
+		gtk_widget_show (d);
+
+		g_free (prompt);
+	}
+	else {
+		trash_files (GTK_WINDOW (browser), file_list);
+		_g_object_list_unref (file_list);
+	}
+
 	_gtk_tree_path_list_free (items);
 }
 
 
 void
-gth_browser_activate_action_edit_delete (GtkAction  *action,
-					 GthBrowser *browser)
+gth_file_mananger_delete_files (GtkWindow *window,
+				GList     *file_list /* GthFileData list */)
 {
-	GList     *items;
-	GList     *file_list;
 	int        file_count;
 	char      *prompt;
 	GtkWidget *d;
 
-	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
-	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
-
+	file_list = _g_object_list_ref (file_list);
 	file_count = g_list_length (file_list);
 	if (file_count == 1) {
 		GthFileData *file_data = file_list->data;
-
 		prompt = g_strdup_printf (_("Are you sure you want to permanently delete \"%s\"?"), g_file_info_get_display_name (file_data->info));
 	}
 	else
@@ -491,7 +574,7 @@ gth_browser_activate_action_edit_delete (GtkAction  *action,
 					  	   "the %'d selected files?", file_count),
 					  file_count);
 
-	d = _gtk_message_dialog_new (GTK_WINDOW (browser),
+	d = _gtk_message_dialog_new (window,
 				     GTK_DIALOG_MODAL,
 				     GTK_STOCK_DIALOG_QUESTION,
 				     prompt,
@@ -503,6 +586,22 @@ gth_browser_activate_action_edit_delete (GtkAction  *action,
 	gtk_widget_show (d);
 
 	g_free (prompt);
+}
+
+
+void
+gth_browser_activate_action_edit_delete (GtkAction  *action,
+					 GthBrowser *browser)
+{
+	GList *items;
+	GList *file_list;
+
+
+	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
+	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
+	gth_file_mananger_delete_files (GTK_WINDOW (browser), file_list);
+
+	_g_object_list_unref (file_list);
 	_gtk_tree_path_list_free (items);
 }
 
@@ -515,7 +614,7 @@ gth_browser_activate_action_folder_open_in_file_manager (GtkAction  *action,
 	char        *uri;
 	GError      *error = NULL;
 
-	file_data = gth_folder_tree_get_selected (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)));
+	file_data = gth_browser_get_folder_popup_file_data (browser);
 	if (file_data == NULL)
 		return;
 
@@ -539,13 +638,11 @@ gth_browser_activate_action_folder_create (GtkAction  *action,
 {
 	GthFileData *parent;
 
-	parent = gth_folder_tree_get_selected (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)));
-	if (parent == NULL)
-		return;
-
-	_gth_browser_create_new_folder (browser, parent->file);
-
-	g_object_unref (parent);
+	parent = gth_browser_get_folder_popup_file_data (browser);
+	if (parent != NULL) {
+		_gth_browser_create_new_folder (browser, parent->file);
+		g_object_unref (parent);
+	}
 }
 
 
@@ -555,7 +652,7 @@ gth_browser_activate_action_folder_rename (GtkAction  *action,
 {
 	GthFileData *file_data;
 
-	file_data = gth_folder_tree_get_selected (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)));
+	file_data = gth_browser_get_folder_popup_file_data (browser);
 	if (file_data == NULL)
 		return;
 
@@ -572,7 +669,7 @@ gth_browser_activate_action_folder_cut (GtkAction  *action,
 	GthFileData *file_data;
 	GList       *file_list;
 
-	file_data = gth_folder_tree_get_selected (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)));
+	file_data = gth_browser_get_folder_popup_file_data (browser);
 	if (file_data == NULL)
 		return;
 
@@ -590,7 +687,7 @@ gth_browser_activate_action_folder_copy (GtkAction  *action,
 	GthFileData *file_data;
 	GList       *file_list;
 
-	file_data = gth_folder_tree_get_selected (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)));
+	file_data = gth_browser_get_folder_popup_file_data (browser);
 	if (file_data == NULL)
 		return;
 
@@ -608,7 +705,7 @@ gth_browser_activate_action_folder_paste (GtkAction  *action,
 	GthFileData *file_data;
 	PasteData   *paste_data;
 
-	file_data = gth_folder_tree_get_selected (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)));
+	file_data = gth_browser_get_folder_popup_file_data (browser);
 	if (file_data == NULL)
 		return;
 
@@ -717,7 +814,7 @@ gth_browser_activate_action_folder_trash (GtkAction  *action,
 	GthFileData *file_data;
 	GError      *error = NULL;
 
-	file_data = gth_folder_tree_get_selected (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)));
+	file_data = gth_browser_get_folder_popup_file_data (browser);
 	if (file_data == NULL)
 		return;
 
@@ -771,7 +868,7 @@ gth_browser_activate_action_folder_delete (GtkAction  *action,
 	DeleteFolderData *delete_data;
 	GtkWidget        *d;
 
-	file_data = gth_folder_tree_get_selected (GTH_FOLDER_TREE (gth_browser_get_folder_tree (browser)));
+	file_data = gth_browser_get_folder_popup_file_data (browser);
 	if (file_data == NULL)
 		return;
 
@@ -794,3 +891,152 @@ gth_browser_activate_action_folder_delete (GtkAction  *action,
 
 	g_free (prompt);
 }
+
+
+/* Copy/Move to folder */
+
+
+typedef struct {
+	GthBrowser *browser;
+	gboolean    move;
+	GFile      *destination;
+	gboolean    view_destination;
+} CopyToFolderData;
+
+
+static void
+copy_to_folder_data_free (CopyToFolderData *data)
+{
+	g_object_unref (data->destination);
+	g_object_unref (data->browser);
+	g_free (data);
+}
+
+
+static void
+copy_complete_cb (GthTask  *task,
+		  GError   *error,
+		  gpointer  user_data)
+{
+	CopyToFolderData *data = user_data;
+
+	if ((error == NULL) && (data->view_destination))
+		gth_browser_load_location (data->browser, data->destination);
+
+	g_object_unref (task);
+	copy_to_folder_data_free (data);
+}
+
+
+static void
+copy_files_to_folder (GthBrowser *browser,
+		      gboolean    move,
+		      char       *destination_uri,
+		      gboolean    view_destination)
+{
+	GthFileData      *destination_data;
+	GthFileSource    *file_source;
+	GList            *items;
+	GList            *file_list;
+	GList            *files;
+	CopyToFolderData *data;
+	GthTask          *task;
+
+	destination_data = gth_file_data_new_for_uri (destination_uri, NULL);
+	file_source = gth_main_get_file_source (destination_data->file);
+
+	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
+	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
+	files = gth_file_data_list_to_file_list (file_list);
+
+	data = g_new0 (CopyToFolderData, 1);
+	data->browser = g_object_ref (browser);
+	data->move = move;
+	data->destination = g_file_dup (destination_data->file);
+	data->view_destination = view_destination;
+
+	task = gth_copy_task_new (file_source, destination_data, move, files);
+	g_signal_connect (task,
+			  "completed",
+			  G_CALLBACK (copy_complete_cb),
+			  data);
+	gth_browser_exec_task (browser, task, FALSE);
+
+	_g_object_list_unref (files);
+	_g_object_list_unref (file_list);
+	_gtk_tree_path_list_free (items);
+	g_object_unref (file_source);
+}
+
+
+static void
+copy_to_folder_dialog (GthBrowser *browser,
+		       gboolean    move)
+{
+	GtkWidget *dialog;
+	char      *start_uri;
+	GtkWidget *box;
+	GtkWidget *view_destination_button;
+
+	dialog = gtk_file_chooser_dialog_new (move ? _("Move To") : _("Copy To"),
+					      NULL,
+					      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      (move ? _("Move") : _("Copy")), GTK_RESPONSE_ACCEPT,
+					      NULL);
+
+	start_uri = eel_gconf_get_string (PREF_FILE_MANAGER_COPY_LAST_FOLDER, get_home_uri ());
+	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dialog), start_uri);
+	g_free(start_uri);
+
+	box = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (box), 6);
+	gtk_widget_show (box);
+
+	view_destination_button = gtk_check_button_new_with_mnemonic (_("_View the destination"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view_destination_button),
+				      eel_gconf_get_boolean (PREF_FILE_MANAGER_COPY_VIEW_DESTINATION, TRUE));
+	gtk_widget_show (view_destination_button);
+	gtk_box_pack_start (GTK_BOX (box), view_destination_button, FALSE, FALSE, 0);
+
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), box, FALSE, FALSE, 0);
+
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+		char *destination_uri;
+
+		destination_uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+		if (destination_uri != NULL) {
+			gboolean view_destination;
+
+			/* save the options */
+
+			view_destination = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view_destination_button));
+			eel_gconf_set_boolean (PREF_FILE_MANAGER_COPY_VIEW_DESTINATION, view_destination);
+			eel_gconf_set_string (PREF_FILE_MANAGER_COPY_LAST_FOLDER, destination_uri);
+
+			/* copy / move the files */
+
+			copy_files_to_folder (browser, move, destination_uri, view_destination);
+		}
+
+		g_free (destination_uri);
+	}
+
+	gtk_widget_destroy (dialog);
+}
+
+
+void
+gth_browser_activate_action_tool_copy_to_folder (GtkAction  *action,
+						 GthBrowser *browser)
+{
+	copy_to_folder_dialog (browser, FALSE);
+}
+
+
+void
+gth_browser_activate_action_tool_move_to_folder (GtkAction  *action,
+						 GthBrowser *browser)
+{
+	copy_to_folder_dialog (browser, TRUE);
+}
