@@ -51,6 +51,10 @@ static const char *fixed_ui_info =
 "      <menuitem action='Edit_CopyFiles'/>"
 "      <menuitem action='Edit_PasteInFolder'/>"
 "    </placeholder>"
+"    <placeholder name='Folder_Actions'>"
+"      <menuitem action='Tool_CopyToFolder'/>"
+"      <menuitem action='Tool_MoveToFolder'/>"
+"    </placeholder>"
 "  </popup>"
 "  <popup name='FilePopup'>"
 "    <placeholder name='File_Actions'>"
@@ -59,6 +63,8 @@ static const char *fixed_ui_info =
 "      <menuitem action='Edit_PasteInFolder'/>"
 "    </placeholder>"
 "    <placeholder name='Folder_Actions'>"
+"      <menuitem action='Tool_CopyToFolder'/>"
+"      <menuitem action='Tool_MoveToFolder'/>"
 "      <menuitem action='Edit_Trash'/>"
 "      <menuitem action='Edit_Delete'/>"
 "    </placeholder>"
@@ -204,6 +210,14 @@ static GtkActionEntry action_entries[] = {
 	  N_("_Delete"), NULL,
 	  NULL,
 	  G_CALLBACK (gth_browser_activate_action_folder_delete) },
+	{ "Tool_CopyToFolder", NULL,
+	  N_("Copy to..."), NULL,
+	  N_("Copy the selected files to another folder"),
+	  G_CALLBACK (gth_browser_activate_action_tool_copy_to_folder) },
+	{ "Tool_MoveToFolder", NULL,
+	  N_("Move to..."), NULL,
+	  N_("Move the selected files to another folder"),
+	  G_CALLBACK (gth_browser_activate_action_tool_move_to_folder) }
 };
 
 
@@ -266,12 +280,20 @@ gth_file_list_drag_data_received (GtkWidget        *file_view,
 	*/
 
 	if ((context->suggested_action == GDK_ACTION_COPY)
-	    || (context->suggested_action == GDK_ACTION_MOVE))
+	    || (context->suggested_action == GDK_ACTION_MOVE)
+	    || (context->suggested_action == GDK_ACTION_ASK))
 	{
 		success = TRUE;
 	}
 
+	if (context->suggested_action == GDK_ACTION_ASK) {
+		context->action = _gtk_menu_ask_drag_drop_action (file_view, context->actions, time);
+		success = context->action != 0;
+	}
+
 	gtk_drag_finish (context, success, FALSE, time);
+	if (! success)
+		return;
 
 	uris = gtk_selection_data_get_uris (selection_data);
 	selected_files = _g_file_list_new_from_uriv (uris);
@@ -303,9 +325,9 @@ gth_file_list_drag_data_received (GtkWidget        *file_view,
 			gboolean       move;
 
 			file_source = gth_browser_get_location_source (browser);
-			/*move = gdk_drag_context_get_suggested_action (context) == GDK_ACTION_MOVE;*/
-			move = context->suggested_action == GDK_ACTION_MOVE;
-			if (move && ! gth_file_source_can_cut (file_source)) {
+			/*move = gdk_drag_context_get_action (context) == GDK_ACTION_MOVE;*/
+			move = context->action == GDK_ACTION_MOVE;
+			if (move && ! gth_file_source_can_cut (file_source, (GFile *) selected_files->data)) {
 				GtkWidget *dialog;
 				int        response;
 
@@ -327,15 +349,18 @@ gth_file_list_drag_data_received (GtkWidget        *file_view,
 			}
 
 			if (! cancel) {
-				GthTask *task;
+				GthFileSource *location_source;
+				GthTask       *task;
 
-				task = gth_copy_task_new (file_source,
+				location_source = gth_main_get_file_source (gth_browser_get_location (browser));
+				task = gth_copy_task_new (location_source,
 							  gth_browser_get_location_data (browser),
 							  move,
 							  selected_files);
 				gth_browser_exec_task (browser, task, FALSE);
 
 				g_object_unref (task);
+				g_object_unref (location_source);
 			}
 		}
 	}
@@ -432,6 +457,8 @@ gth_file_list_drag_motion (GtkWidget      *file_view,
 			data->scroll_event = 0;
 		}
 	}
+	else if (context->suggested_action == GDK_ACTION_ASK)
+		gdk_drag_status (context, GDK_ACTION_ASK, time);
 	else
 		gdk_drag_status (context, GDK_ACTION_COPY, time);
 
@@ -473,7 +500,7 @@ void
 fm__gth_browser_construct_cb (GthBrowser *browser)
 {
 	BrowserData *data;
-	GError      *error;
+	GError      *error = NULL;
 	GtkWidget   *file_view;
 
 	g_return_if_fail (GTH_IS_BROWSER (browser));
@@ -709,7 +736,7 @@ fm__gth_browser_folder_tree_drag_data_received_cb (GthBrowser    *browser,
 	if (file_source == NULL)
 		return;
 
-	if ((action == GDK_ACTION_MOVE) && ! gth_file_source_can_cut (file_source)) {
+	if ((action == GDK_ACTION_MOVE) && ! gth_file_source_can_cut (file_source, (GFile *) file_list->data)) {
 		GtkWidget *dialog;
 		int        response;
 
@@ -795,6 +822,7 @@ fm__gth_browser_update_sensitivity_cb (GthBrowser *browser)
 	BrowserData   *data;
 	GthFileSource *file_source;
 	int            n_selected;
+	GthFileData   *location_data;
 	gboolean       sensitive;
 	GthFileData   *folder;
 
@@ -804,7 +832,8 @@ fm__gth_browser_update_sensitivity_cb (GthBrowser *browser)
 	file_source = gth_browser_get_location_source (browser);
 	n_selected = gth_file_selection_get_n_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
 
-	sensitive = (n_selected > 0) && (file_source != NULL) && gth_file_source_can_cut (file_source);
+	location_data = gth_browser_get_location_data (browser);
+	sensitive = (n_selected > 0) && (file_source != NULL) && (location_data != NULL) && gth_file_source_can_cut (file_source, location_data->file);
 	set_action_sensitive (data, "Edit_CutFiles", sensitive);
 
 	sensitive = (n_selected > 0) && (file_source != NULL);
@@ -812,6 +841,8 @@ fm__gth_browser_update_sensitivity_cb (GthBrowser *browser)
 	set_action_sensitive (data, "Edit_Trash", sensitive);
 	set_action_sensitive (data, "Edit_Delete", sensitive);
 	set_action_sensitive (data, "Edit_Duplicate", sensitive);
+	set_action_sensitive (data, "Tool_MoveToFolder", sensitive);
+	set_action_sensitive (data, "Tool_CopyToFolder", sensitive);
 
 	folder = gth_browser_get_folder_popup_file_data (browser);
 	set_action_sensitive (data, "Folder_Create", (folder != NULL) && g_file_info_get_attribute_boolean (folder->info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE));
@@ -838,13 +869,14 @@ static void
 activate_open_with_application_item (GtkMenuItem *menuitem,
 				     gpointer     data)
 {
-	GthBrowser *browser = data;
-	GList      *items;
-	GList      *file_list;
-	GList      *uris;
-	GList      *scan;
-	GAppInfo   *appinfo;
-	GError     *error = NULL;
+	GthBrowser          *browser = data;
+	GList               *items;
+	GList               *file_list;
+	GList               *uris;
+	GList               *scan;
+	GAppInfo            *appinfo;
+	GdkAppLaunchContext *context;
+	GError              *error = NULL;
 
 	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
 	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
@@ -859,11 +891,16 @@ activate_open_with_application_item (GtkMenuItem *menuitem,
 	appinfo = g_object_get_data (G_OBJECT (menuitem), "appinfo");
 	g_return_if_fail (G_IS_APP_INFO (appinfo));
 
-	if (! g_app_info_launch_uris (appinfo, uris, NULL, &error))
+	context = gdk_app_launch_context_new ();
+	gdk_app_launch_context_set_screen (context, gtk_widget_get_screen (GTK_WIDGET (browser)));
+	gdk_app_launch_context_set_timestamp (context, 0);
+	gdk_app_launch_context_set_icon (context, g_app_info_get_icon (appinfo));
+	if (! g_app_info_launch_uris (appinfo, uris, G_APP_LAUNCH_CONTEXT (context), &error))
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (browser),
 						    _("Could not perform the operation"),
 						    &error);
 
+	g_object_unref (context);
 	g_list_free (uris);
 	_g_object_list_unref (file_list);
 	_gtk_tree_path_list_free (items);
@@ -1029,7 +1066,7 @@ fm__gth_browser_file_list_key_press_cb (GthBrowser  *browser,
 			items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
 			file_data_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
 			file_list = gth_file_data_list_to_file_list (file_data_list);
-			_g_launch_command (GTK_WIDGET (browser), "gimp", "Gimp", file_list);
+			_g_launch_command (GTK_WIDGET (browser), "gimp %U", "Gimp", file_list);
 
 			_g_object_list_unref (file_list);
 			_g_object_list_unref (file_data_list);
