@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -92,7 +91,7 @@ import_dialog_destroy_cb (GtkWidget  *widget,
 }
 
 
-GList *
+static GList *
 get_files_to_download (DialogData *data)
 {
 	GthFileView *file_view;
@@ -126,7 +125,7 @@ import_dialog_response_cb (GtkDialog *dialog,
 
 	case GTK_RESPONSE_DELETE_EVENT:
 	case GTK_RESPONSE_CANCEL:
-		gtk_widget_destroy (data->dialog);
+		gth_file_list_cancel (GTH_FILE_LIST (data->file_list), (DataFunc) gtk_widget_destroy, data->dialog);
 		break;
 
 	case GTK_RESPONSE_OK:
@@ -151,8 +150,6 @@ import_dialog_response_cb (GtkDialog *dialog,
 				GthSubfolderType     subfolder_type;
 				GthSubfolderFormat   subfolder_format;
 				char                *custom_format;
-				gboolean             overwrite_files;
-				gboolean             adjust_orientation;
 				GthTask             *task;
 
 				destination = gth_import_preferences_get_destination ();
@@ -160,8 +157,6 @@ import_dialog_response_cb (GtkDialog *dialog,
 				subfolder_format = eel_gconf_get_enum (PREF_IMPORT_SUBFOLDER_FORMAT, GTH_TYPE_SUBFOLDER_FORMAT, GTH_SUBFOLDER_FORMAT_YYYYMMDD);
 				single_subfolder = eel_gconf_get_boolean (PREF_IMPORT_SUBFOLDER_SINGLE, FALSE);
 				custom_format = eel_gconf_get_string (PREF_IMPORT_SUBFOLDER_CUSTOM_FORMAT, "");
-				overwrite_files = eel_gconf_get_boolean (PREF_IMPORT_OVERWRITE, FALSE);
-				adjust_orientation = eel_gconf_get_boolean (PREF_IMPORT_ADJUST_ORIENTATION, FALSE);
 
 				task = gth_import_task_new (data->browser,
 							    file_list,
@@ -173,8 +168,8 @@ import_dialog_response_cb (GtkDialog *dialog,
 							    (photoset->title != NULL ? photoset->title : ""),
 							    NULL,
 							    FALSE,
-							    overwrite_files,
-							    adjust_orientation);
+							    FALSE,
+							    FALSE);
 				gth_browser_exec_task (data->browser, task, FALSE);
 				gtk_widget_destroy (data->dialog);
 
@@ -235,7 +230,7 @@ photoset_list_ready_cb (GObject      *source_object,
 	data->photosets = flickr_service_list_photosets_finish (FLICKR_SERVICE (source_object), res, &error);
 	if (error != NULL) {
 		if (data->conn != NULL)
-			gth_task_dialog (GTH_TASK (data->conn), TRUE);
+			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->browser), _("Could not connect to the server"), &error);
 		gtk_widget_destroy (data->dialog);
 		return;
@@ -260,7 +255,7 @@ photoset_list_ready_cb (GObject      *source_object,
 		g_free (n_photos);
 	}
 
-	gth_task_dialog (GTH_TASK (data->conn), TRUE);
+	gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (data->browser));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
@@ -351,12 +346,13 @@ list_photos_ready_cb (GObject      *source_object,
 	GList      *list;
 	GList      *scan;
 
-	gth_task_dialog (GTH_TASK (data->conn), TRUE);
+	gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
+
 	_g_object_list_unref (data->photos);
 	data->photos = flickr_service_list_photos_finish (data->service, result, &error);
 	if (error != NULL) {
 		if (data->conn != NULL)
-			gth_task_dialog (GTH_TASK (data->conn), TRUE);
+			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not get the photo list"), &error);
 		gtk_widget_destroy (data->dialog);
 		return;
@@ -365,9 +361,24 @@ list_photos_ready_cb (GObject      *source_object,
 	list = NULL;
 	for (scan = data->photos; scan; scan = scan->next) {
 		FlickrPhoto *photo = scan->data;
+		const char  *url;
 		GthFileData *file_data;
 
-		file_data = gth_file_data_new_for_uri (photo->url_o, (photo->mime_type != NULL) ? photo->mime_type : "image/jpeg");
+		url = NULL;
+		if (photo->url_s != NULL)
+			url = photo->url_s;
+		else if (photo->url_m != NULL)
+			url = photo->url_m;
+		else if (photo->url_o != NULL)
+			url = photo->url_o;
+
+		if (url == NULL)
+			url = photo->url_sq;
+
+		if (url == NULL)
+			continue;
+
+		file_data = gth_file_data_new_for_uri (url, (photo->mime_type != NULL) ? photo->mime_type : "image/jpeg");
 		g_file_info_set_file_type (file_data->info, G_FILE_TYPE_REGULAR);
 		g_file_info_set_size (file_data->info, FAKE_SIZE); /* set a fake size to make the progress dialog work correctly */
 		g_file_info_set_attribute_object (file_data->info, "flickr::object", G_OBJECT (photo));
@@ -400,7 +411,9 @@ photoset_combobox_changed_cb (GtkComboBox *widget,
 			    PHOTOSET_DATA_COLUMN, &data->photoset,
 			    -1);
 
-	gth_task_dialog (GTH_TASK (data->conn), FALSE);
+	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog), data->photoset->title);
+
+	gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
 	flickr_service_list_photos (data->service,
 				    data->photoset,
 				    "original_format, url_sq, url_t, url_s, url_m, url_o",
@@ -413,13 +426,16 @@ photoset_combobox_changed_cb (GtkComboBox *widget,
 
 
 static GdkPixbufAnimation *
-flickr_thumbnail_loader (GthFileData  *file_data,
-		         GError      **error,
-		         gpointer      data)
+flickr_thumbnail_loader (GthFileData   *file_data,
+			 int            requested_size,
+			 int           *original_width,
+			 int           *original_height,
+			 gpointer       user_data,
+			 GCancellable  *cancellable,
+		         GError       **error)
 {
 	GdkPixbufAnimation *animation = NULL;
-	GthThumbLoader     *thumb_loader = data;
-	int                 requested_size;
+	GthThumbLoader     *thumb_loader = user_data;
 	FlickrPhoto        *photo;
 	const char         *uri = NULL;
 
@@ -448,7 +464,7 @@ flickr_thumbnail_loader (GthFileData  *file_data,
 			GdkPixbuf    *pixbuf;
 
 			stream = g_memory_input_stream_new_from_data (buffer, size, g_free);
-			pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, error);
+			pixbuf = gdk_pixbuf_new_from_stream (stream, cancellable, error);
 			if (pixbuf != NULL) {
 				GdkPixbuf *rotated;
 
@@ -492,23 +508,13 @@ flickr_photo_position_func (GthFileData *a,
 
 
 static void
-file_list_selection_changed_cb (GtkIconView *iconview,
+file_list_selection_changed_cb (GthFileView *file_view,
 				gpointer     user_data)
 {
 	update_selection_status ((DialogData *) user_data);
 }
 
 
-static void
-preferences_button_clicked_cb (GtkWidget  *widget,
-			       DialogData *data)
-{
-	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog),
-						 (data->photoset != NULL) ? data->photoset->title : "");
-	gtk_window_present (GTK_WINDOW (data->preferences_dialog));
-}
-
-
 void
 dlg_import_from_flickr (FlickrServer *server,
 		        GthBrowser   *browser)
@@ -552,13 +558,14 @@ dlg_import_from_flickr (FlickrServer *server,
 
 	/* Set the widget data */
 
-	data->file_list = gth_file_list_new (GTH_FILE_LIST_TYPE_NORMAL);
+	data->file_list = gth_file_list_new (gth_icon_view_new (), GTH_FILE_LIST_TYPE_NORMAL, FALSE);
 	thumb_loader = gth_file_list_get_thumb_loader (GTH_FILE_LIST (data->file_list));
-	gth_thumb_loader_use_cache (thumb_loader, FALSE);
-	gth_thumb_loader_set_loader (thumb_loader, flickr_thumbnail_loader);
+	gth_thumb_loader_set_use_cache (thumb_loader, FALSE);
+	gth_thumb_loader_set_loader_func (thumb_loader, flickr_thumbnail_loader);
 	gth_file_list_set_thumb_size (GTH_FILE_LIST (data->file_list), FLICKR_SIZE_THUMBNAIL);
 	gth_file_view_set_spacing (GTH_FILE_VIEW (gth_file_list_get_view (GTH_FILE_LIST (data->file_list))), 0);
 	gth_file_list_enable_thumbs (GTH_FILE_LIST (data->file_list), TRUE);
+	gth_file_list_set_ignore_hidden (GTH_FILE_LIST (data->file_list), TRUE);
 	gth_file_list_set_caption (GTH_FILE_LIST (data->file_list), "none");
 	gth_file_list_set_sort_func (GTH_FILE_LIST (data->file_list), flickr_photo_position_func, FALSE);
 	gth_file_list_clear (GTH_FILE_LIST (data->file_list), _("No album selected"));
@@ -572,16 +579,29 @@ dlg_import_from_flickr (FlickrServer *server,
 	data->preferences_dialog = gth_import_preferences_dialog_new ();
 	gtk_window_set_transient_for (GTK_WINDOW (data->preferences_dialog), GTK_WINDOW (data->dialog));
 
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("destination_button_box")),
+			    gth_import_destination_button_new (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog)),
+			    TRUE,
+			    TRUE,
+			    0);
+	gtk_widget_show_all (GET_WIDGET ("destination_button_box"));
+
 	title = g_strdup_printf (_("Import from %s"), data->server->name);
 	gtk_window_set_title (GTK_WINDOW (data->dialog), title);
 	g_free (title);
 
+	_gtk_window_resize_to_fit_screen_height (data->dialog, 500);
+
 	/* Set the signals handlers. */
 
-	g_signal_connect (G_OBJECT (data->dialog),
+	g_signal_connect (data->dialog,
 			  "destroy",
 			  G_CALLBACK (import_dialog_destroy_cb),
 			  data);
+	g_signal_connect (data->dialog,
+			  "delete-event",
+			  G_CALLBACK (gtk_true),
+			  NULL);
 	g_signal_connect (data->dialog,
 			  "response",
 			  G_CALLBACK (import_dialog_response_cb),
@@ -598,16 +618,13 @@ dlg_import_from_flickr (FlickrServer *server,
 			  "changed",
 			  G_CALLBACK (photoset_combobox_changed_cb),
 			  data);
-	g_signal_connect (G_OBJECT (gth_file_list_get_view (GTH_FILE_LIST (data->file_list))),
-			  "selection_changed",
+	g_signal_connect (gth_file_list_get_view (GTH_FILE_LIST (data->file_list)),
+			  "file-selection-changed",
 			  G_CALLBACK (file_list_selection_changed_cb),
 			  data);
-	g_signal_connect (GET_WIDGET ("preferences_button"),
-			  "clicked",
-			  G_CALLBACK (preferences_button_clicked_cb),
-			  data);
 
 	update_selection_status (data);
+	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog), "");
 
 	data->conn = flickr_connection_new (data->server);
 	data->service = flickr_service_new (data->conn);
