@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -25,26 +24,29 @@
 #include <gdk/gdkkeysyms.h>
 #include <gthumb.h>
 #include "gth-image-viewer-page.h"
-
+#include "preferences.h"
 
 #define GTH_IMAGE_VIEWER_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTH_TYPE_IMAGE_VIEWER_PAGE, GthImageViewerPagePrivate))
-#define GCONF_NOTIFICATIONS 7
+#define GCONF_NOTIFICATIONS 8
 
 
 struct _GthImageViewerPagePrivate {
 	GthBrowser        *browser;
-	GtkWidget         *nav_window;
+	GtkWidget         *image_navigator;
 	GtkWidget         *viewer;
 	GthImagePreloader *preloader;
 	GtkActionGroup    *actions;
 	guint              merge_id;
 	GthImageHistory   *history;
 	GthFileData       *file_data;
-	gulong             preloader_sig_id;
+	gulong             requested_ready_id;
+	gulong             original_size_ready_id;
 	guint              cnxn_id[GCONF_NOTIFICATIONS];
 	guint              hide_mouse_timeout;
 	guint              motion_signal;
 	gboolean           pixbuf_changed;
+	gboolean           shrink_wrap;
+	GFile             *last_loaded;
 };
 
 static gpointer gth_image_viewer_page_parent_class = NULL;
@@ -151,7 +153,7 @@ static void
 image_viewer_activate_action_view_shrink_wrap (GtkAction          *action,
 					       GthImageViewerPage *self)
 {
-	gth_image_viewer_page_shrink_wrap (self);
+	eel_gconf_set_boolean (PREF_VIEWER_SHRINK_WRAP, gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
 
@@ -190,23 +192,31 @@ static GtkActionEntry image_viewer_action_entries[] = {
 	  N_("Width"), "",
 	  N_("Zoom to fit width"),
 	  G_CALLBACK (image_viewer_activate_action_view_zoom_fit_width) },
+};
 
+
+static GtkToggleActionEntry image_viewer_toggle_action_entries[] = {
 	{ "ImageViewer_View_ShrinkWrap", NULL,
 	  N_("_Fit Window to Image"), "<control>e",
 	  N_("Resize the window to the size of the image"),
-	  G_CALLBACK (image_viewer_activate_action_view_shrink_wrap) },
+	  G_CALLBACK (image_viewer_activate_action_view_shrink_wrap),
+	  FALSE }
 };
 
 
 static void
-viewer_image_ready_cb (GtkWidget          *widget,
-		       GthImageViewerPage *self)
+gth_image_viewer_page_file_loaded (GthImageViewerPage *self,
+				   gboolean            success)
 {
-	gth_image_history_clear (self->priv->history);
-	gth_image_history_add_image (self->priv->history,
-				     gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer)),
-				     FALSE);
-	gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), TRUE);
+	if (_g_file_equal (self->priv->last_loaded, self->priv->file_data->file))
+		return;
+
+	_g_object_unref (self->priv->last_loaded);
+	self->priv->last_loaded = g_object_ref (self->priv->file_data->file);
+
+	gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self),
+				     self->priv->file_data,
+				     success);
 }
 
 
@@ -238,6 +248,15 @@ viewer_button_press_event_cb (GtkWidget          *widget,
 }
 
 
+static gboolean
+viewer_popup_menu_cb (GtkWidget          *widget,
+		      GthImageViewerPage *self)
+{
+	gth_browser_file_menu_popup (self->priv->browser, NULL);
+	return TRUE;
+}
+
+
 static gboolean
 viewer_scroll_event_cb (GtkWidget 	   *widget,
 		        GdkEventScroll      *event,
@@ -268,25 +287,59 @@ viewer_key_press_cb (GtkWidget          *widget,
 
 static void
 image_preloader_requested_ready_cb (GthImagePreloader  *preloader,
+				    GthFileData        *requested,
+				    GdkPixbufAnimation *animation,
+				    int                 original_width,
+				    int                 original_height,
 				    GError             *error,
 				    GthImageViewerPage *self)
 {
-	GthImageLoader *image_loader;
+	if (! _g_file_equal (requested->file, self->priv->file_data->file))
+		return;
 
 	if (error != NULL) {
-		gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), FALSE);
-		return;
-	}
-
-	image_loader = gth_image_preloader_get_loader (self->priv->preloader, gth_image_preloader_get_requested (self->priv->preloader));
-	if (image_loader == NULL) {
-		gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), FALSE);
+		gth_image_viewer_page_file_loaded (self, FALSE);
 		return;
 	}
 
 	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
+	gth_image_viewer_set_animation (GTH_IMAGE_VIEWER (self->priv->viewer),
+					animation,
+					original_width,
+					original_height);
+	if (self->priv->shrink_wrap)
+		gth_image_viewer_page_shrink_wrap (self, TRUE);
+	gth_image_history_clear (self->priv->history);
+	gth_image_history_add_image (self->priv->history,
+				     gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer)),
+				     FALSE);
+	gth_image_viewer_page_file_loaded (self, TRUE);
+}
 
-	gth_image_viewer_load_from_image_loader (GTH_IMAGE_VIEWER (self->priv->viewer), image_loader);
+
+static void
+image_preloader_original_size_ready_cb (GthImagePreloader  *preloader,
+				        GthFileData        *requested,
+				        GdkPixbufAnimation *animation,
+				        int                 original_width,
+				        int                 original_height,
+				        GError             *error,
+				        GthImageViewerPage *self)
+{
+	if (! _g_file_equal (requested->file, self->priv->file_data->file))
+		return;
+
+	if (error != NULL)
+		return;
+
+	gth_image_viewer_set_better_quality (GTH_IMAGE_VIEWER (self->priv->viewer),
+					     animation,
+					     original_width,
+					     original_height);
+	gth_image_history_clear (self->priv->history);
+	gth_image_history_add_image (self->priv->history,
+				     gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer)),
+				     FALSE);
 }
 
 
@@ -386,11 +439,171 @@ pref_reset_scrollbars_changed (GConfClient *client,
 }
 
 
+static void
+pref_viewer_shrink_wrap_changed (GConfClient *client,
+				 guint        cnxn_id,
+				 GConfEntry  *entry,
+				 gpointer     user_data)
+{
+	GthImageViewerPage *self = user_data;
+
+	gth_image_viewer_page_shrink_wrap (self, eel_gconf_get_boolean (PREF_VIEWER_SHRINK_WRAP, FALSE));
+}
+
+
+static void
+paint_comment_over_image_func (GthImageViewer *image_viewer,
+			       GdkEventExpose *event,
+			       cairo_t        *cr,
+			       gpointer        user_data)
+{
+	GthImageViewerPage *self = user_data;
+	GthFileData        *file_data = self->priv->file_data;
+	GString            *file_info;
+	char               *comment;
+	const char         *file_date;
+	const char         *file_size;
+	int                 current_position;
+	int                 n_visibles;
+	int                 width;
+	int                 height;
+	GthMetadata        *metadata;
+	PangoLayout        *layout;
+	PangoAttrList      *attr_list = NULL;
+	GError             *error = NULL;
+	char               *text;
+	static GdkPixbuf   *icon = NULL;
+	int                 icon_width;
+	int                 icon_height;
+	int                 image_width;
+	int                 image_height;
+	const int           x_padding = 10;
+	const int           y_padding = 10;
+	int                 max_text_width;
+	PangoRectangle      bounds;
+	int                 text_x;
+	int                 text_y;
+	int                 icon_x;
+	int                 icon_y;
+
+	file_info = g_string_new ("");
+
+	comment = gth_file_data_get_attribute_as_string (file_data, "general::description");
+	if (comment != NULL) {
+		g_string_append_printf (file_info, "<b>%s</b>\n\n", comment);
+		g_free (comment);
+	}
+
+	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "general::datetime");
+	if (metadata != NULL)
+		file_date = gth_metadata_get_formatted (metadata);
+	else
+		file_date = g_file_info_get_attribute_string (file_data->info, "gth::file::display-mtime");
+	file_size = g_file_info_get_attribute_string (file_data->info, "gth::file::display-size");
+
+	gth_browser_get_file_list_info (self->priv->browser, &current_position, &n_visibles);
+	gth_image_viewer_get_original_size (GTH_IMAGE_VIEWER (self->priv->viewer), &width, &height);
+
+	g_string_append_printf (file_info,
+			        "<small><i>%s - %dx%d (%d%%) - %s</i>\n<tt>%d/%d - %s</tt></small>",
+			        file_date,
+			        width,
+			        height,
+			        (int) (gth_image_viewer_get_zoom (GTH_IMAGE_VIEWER (self->priv->viewer)) * 100),
+			        file_size,
+			        current_position + 1,
+			        n_visibles,
+				g_file_info_get_attribute_string (file_data->info, "standard::display-name"));
+
+	layout = gtk_widget_create_pango_layout (GTK_WIDGET (self->priv->viewer), NULL);
+	pango_layout_set_wrap (layout, PANGO_WRAP_WORD);
+	pango_layout_set_font_description (layout, GTK_WIDGET (self->priv->viewer)->style->font_desc);
+	pango_layout_set_alignment (layout, PANGO_ALIGN_LEFT);
+
+	if (! pango_parse_markup (file_info->str,
+			          -1,
+				  0,
+				  &attr_list,
+				  &text,
+				  NULL,
+				  &error))
+	{
+		g_warning ("Failed to set text from markup due to error parsing markup: %s\nThis is the text that caused the error: %s",  error->message, file_info->str);
+		g_error_free (error);
+		g_object_unref (layout);
+		g_string_free (file_info, TRUE);
+		return;
+	}
+
+	pango_layout_set_attributes (layout, attr_list);
+        pango_layout_set_text (layout, text, strlen (text));
+
+        if (icon == NULL) {
+        	GIcon *gicon;
+
+        	gicon = g_themed_icon_new (GTK_STOCK_PROPERTIES);
+        	icon = _g_icon_get_pixbuf (gicon, 24, NULL);
+
+        	g_object_unref (gicon);
+        }
+	icon_width = gdk_pixbuf_get_width (icon);
+	icon_height = gdk_pixbuf_get_height (icon);
+
+	gdk_drawable_get_size (gtk_widget_get_window (self->priv->viewer), &image_width, &image_height);
+	max_text_width = ((image_width * 3 / 4) - icon_width - (x_padding * 3) - (x_padding * 2));
+
+	pango_layout_set_width (layout, max_text_width * PANGO_SCALE);
+	pango_layout_get_pixel_extents (layout, NULL, &bounds);
+
+	bounds.width += (2 * x_padding) + (icon_width + x_padding);
+	bounds.height = MIN (image_height - icon_height - (y_padding * 2), bounds.height + (2 * y_padding));
+	bounds.x = MAX ((image_width - bounds.width) / 2, 0);
+	bounds.y = MAX (image_height - bounds.height - (y_padding * 3), 0);
+
+	text_x = bounds.x + x_padding + icon_width + x_padding;
+	text_y = bounds.y + y_padding;
+	icon_x = bounds.x + x_padding;
+	icon_y = bounds.y + (bounds.height - icon_height) / 2;
+
+	cairo_save (cr);
+
+	/* background */
+
+	_cairo_draw_rounded_box (cr, bounds.x, bounds.y, bounds.width, bounds.height, 8.0);
+	cairo_set_source_rgba (cr, 0.94, 0.94, 0.94, 0.81);
+	cairo_fill (cr);
+	cairo_set_line_width (cr, 1.0);
+	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+	cairo_stroke (cr);
+
+	/* icon */
+
+	gdk_cairo_set_source_pixbuf (cr, icon, icon_x, icon_y);
+	cairo_rectangle (cr, icon_x, icon_y, icon_width, icon_height);
+	cairo_fill (cr);
+
+	/* text */
+
+	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+	pango_cairo_update_layout (cr, layout);
+	cairo_move_to (cr, text_x, text_y);
+	pango_cairo_show_layout (cr, layout);
+
+	cairo_restore (cr);
+
+        g_free (text);
+        pango_attr_list_unref (attr_list);
+	g_object_unref (layout);
+	g_string_free (file_info, TRUE);
+}
+
+
 static void
 gth_image_viewer_page_real_activate (GthViewerPage *base,
 				     GthBrowser    *browser)
 {
 	GthImageViewerPage *self;
+	GtkAction          *action;
 	int                 i;
 
 	self = (GthImageViewerPage*) base;
@@ -403,13 +616,21 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 				      image_viewer_action_entries,
 				      G_N_ELEMENTS (image_viewer_action_entries),
 				      self);
+	gtk_action_group_add_toggle_actions (self->priv->actions,
+					     image_viewer_toggle_action_entries,
+					     G_N_ELEMENTS (image_viewer_toggle_action_entries),
+					     self);
 	gtk_ui_manager_insert_action_group (gth_browser_get_ui_manager (browser), self->priv->actions, 0);
 
 	self->priv->preloader = gth_browser_get_image_preloader (browser);
-	self->priv->preloader_sig_id = g_signal_connect (G_OBJECT (self->priv->preloader),
-							 "requested_ready",
-							 G_CALLBACK (image_preloader_requested_ready_cb),
-							 self);
+	self->priv->requested_ready_id = g_signal_connect (G_OBJECT (self->priv->preloader),
+							   "requested_ready",
+							   G_CALLBACK (image_preloader_requested_ready_cb),
+							   self);
+	self->priv->original_size_ready_id = g_signal_connect (G_OBJECT (self->priv->preloader),
+							       "original_size_ready",
+							       G_CALLBACK (image_preloader_original_size_ready_cb),
+							       self);
 
 	self->priv->viewer = gth_image_viewer_new ();
 	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (self->priv->viewer), eel_gconf_get_enum (PREF_ZOOM_QUALITY, GTH_TYPE_ZOOM_QUALITY, GTH_ZOOM_QUALITY_HIGH));
@@ -420,16 +641,21 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 	gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), eel_gconf_get_boolean (PREF_BLACK_BACKGROUND, FALSE));
 	gth_image_viewer_set_reset_scrollbars (GTH_IMAGE_VIEWER (self->priv->viewer), eel_gconf_get_boolean (PREF_RESET_SCROLLBARS, TRUE));
 
+	self->priv->shrink_wrap = eel_gconf_get_boolean (PREF_VIEWER_SHRINK_WRAP, FALSE);
+	action = gtk_action_group_get_action (self->priv->actions, "ImageViewer_View_ShrinkWrap");
+	if (action != NULL)
+		g_object_set (action, "active", self->priv->shrink_wrap, NULL);
+
 	gtk_widget_show (self->priv->viewer);
 
-	g_signal_connect (G_OBJECT (self->priv->viewer),
-			  "image_ready",
-			  G_CALLBACK (viewer_image_ready_cb),
-			  self);
 	g_signal_connect (G_OBJECT (self->priv->viewer),
 			  "zoom_changed",
 			  G_CALLBACK (viewer_zoom_changed_cb),
 			  self);
+	g_signal_connect (G_OBJECT (self->priv->viewer),
+			  "popup-menu",
+			  G_CALLBACK (viewer_popup_menu_cb),
+			  self);
 	g_signal_connect_after (G_OBJECT (self->priv->viewer),
 				"button_press_event",
 				G_CALLBACK (viewer_button_press_event_cb),
@@ -447,10 +673,10 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 			  G_CALLBACK (viewer_key_press_cb),
 			  self);
 
-	self->priv->nav_window = gth_nav_window_new (GTH_IMAGE_VIEWER (self->priv->viewer));
-	gtk_widget_show (self->priv->nav_window);
+	self->priv->image_navigator = gth_image_navigator_new (GTH_IMAGE_VIEWER (self->priv->viewer));
+	gtk_widget_show (self->priv->image_navigator);
 
-	gth_browser_set_viewer_widget (browser, self->priv->nav_window);
+	gth_browser_set_viewer_widget (browser, self->priv->image_navigator);
 	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
 
 	/* gconf notifications */
@@ -493,6 +719,11 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 					   PREF_RESET_SCROLLBARS,
 					   pref_reset_scrollbars_changed,
 					   self);
+
+	self->priv->cnxn_id[i++] = eel_gconf_notification_add (
+					   PREF_VIEWER_SHRINK_WRAP,
+					   pref_viewer_shrink_wrap_changed,
+					   self);
 }
 
 
@@ -512,8 +743,15 @@ gth_image_viewer_page_real_deactivate (GthViewerPage *base)
 
 	/**/
 
-	g_signal_handler_disconnect (self->priv->preloader, self->priv->preloader_sig_id);
-	self->priv->preloader_sig_id = 0;
+	gtk_ui_manager_remove_action_group (gth_browser_get_ui_manager (self->priv->browser), self->priv->actions);
+	g_object_unref (self->priv->actions);
+	self->priv->actions = NULL;
+
+	g_signal_handler_disconnect (self->priv->preloader, self->priv->requested_ready_id);
+	g_signal_handler_disconnect (self->priv->preloader, self->priv->original_size_ready_id);
+	self->priv->requested_ready_id = 0;
+	self->priv->original_size_ready_id = 0;
+
 	g_object_unref (self->priv->preloader);
 	self->priv->preloader = NULL;
 
@@ -577,18 +815,24 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 	GthFileStore       *file_store;
 	GtkTreeIter         iter;
 	GthFileData        *next_file_data = NULL;
+	GthFileData        *next2_file_data = NULL;
 	GthFileData        *prev_file_data = NULL;
+	int                 window_width;
+	int                 window_height;
 
 	self = (GthImageViewerPage*) base;
 	g_return_if_fail (file_data != NULL);
 
 	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
 
+	_g_clear_object (&self->priv->last_loaded);
+
 	if ((self->priv->file_data != NULL)
 	    && g_file_equal (file_data->file, self->priv->file_data->file)
 	    && (gth_file_data_get_mtime (file_data) == gth_file_data_get_mtime (self->priv->file_data))
 	    && ! self->priv->pixbuf_changed)
 	{
+		gth_image_viewer_page_file_loaded (self, TRUE);
 		return;
 	}
 
@@ -596,22 +840,34 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 	self->priv->file_data = gth_file_data_dup (file_data);
 
 	file_store = gth_browser_get_file_store (self->priv->browser);
-	if (gth_file_store_find_visible (file_store, file_data->file, &iter)) {
+	if (gth_file_store_find_visible (file_store, self->priv->file_data->file, &iter)) {
 		GtkTreeIter iter2;
+		GtkTreeIter iter3;
 
 		iter2 = iter;
 		if (gth_file_store_get_next_visible (file_store, &iter2))
 			next_file_data = gth_file_store_get_file (file_store, &iter2);
 
+		iter3 = iter2;
+		if (gth_file_store_get_next_visible (file_store, &iter3))
+			next2_file_data = gth_file_store_get_file (file_store, &iter3);
+
 		iter2 = iter;
 		if (gth_file_store_get_prev_visible (file_store, &iter2))
 			prev_file_data = gth_file_store_get_file (file_store, &iter2);
 	}
 
+	gtk_window_get_size (GTK_WINDOW (self->priv->browser),
+			     &window_width,
+			     &window_height);
+
 	gth_image_preloader_load (self->priv->preloader,
-				  file_data,
+				  self->priv->file_data,
+				  (gth_image_prelaoder_get_load_policy (self->priv->preloader) == GTH_LOAD_POLICY_TWO_STEPS) ? MAX (window_width, window_height) : -1,
 				  next_file_data,
-				  prev_file_data);
+				  next2_file_data,
+				  prev_file_data,
+				  NULL);
 }
 
 
@@ -621,7 +877,7 @@ gth_image_viewer_page_real_focus (GthViewerPage *base)
 	GtkWidget *widget;
 
 	widget = GTH_IMAGE_VIEWER_PAGE (base)->priv->viewer;
-	if (GTK_WIDGET_REALIZED (widget) && GTK_WIDGET_MAPPED (widget))
+	if (gtk_widget_get_realized (widget) && gtk_widget_get_mapped (widget))
 		gtk_widget_grab_focus (widget);
 }
 
@@ -633,12 +889,13 @@ gth_image_viewer_page_real_fullscreen (GthViewerPage *base,
 	GthImageViewerPage *self;
 
 	self = (GthImageViewerPage *) base;
+
 	if (active) {
-		gth_nav_window_set_scrollbars_visible (GTH_NAV_WINDOW (self->priv->nav_window), FALSE);
+		gth_image_navigator_set_scrollbars_visible (GTH_IMAGE_NAVIGATOR (self->priv->image_navigator), FALSE);
 		gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), TRUE);
 	}
 	else {
-		gth_nav_window_set_scrollbars_visible (GTH_NAV_WINDOW (self->priv->nav_window), TRUE);
+		gth_image_navigator_set_scrollbars_visible (GTH_IMAGE_NAVIGATOR (self->priv->image_navigator), TRUE);
 		gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), eel_gconf_get_boolean (PREF_BLACK_BACKGROUND, FALSE));
 	}
 }
@@ -911,7 +1168,7 @@ _gth_image_viewer_page_set_pixbuf (GthImageViewerPage *self,
 	int          height;
 	char        *size;
 
-	gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer), pixbuf);
+	gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer), pixbuf, -1, -1);
 
 	file_data = gth_browser_get_current_file (GTH_BROWSER (self->priv->browser));
 
@@ -922,7 +1179,7 @@ _gth_image_viewer_page_set_pixbuf (GthImageViewerPage *self,
 	g_file_info_set_attribute_int32 (file_data->info, "image::width", width);
 	g_file_info_set_attribute_int32 (file_data->info, "image::height", height);
 
-	size = g_strdup_printf ("%d × %d", width, height);
+	size = g_strdup_printf (_("%d × %d"), width, height);
 	g_file_info_set_attribute_string (file_data->info, "general::dimensions", size);
 
 	gth_monitor_metadata_changed (gth_main_get_default_monitor (), file_data);
@@ -945,6 +1202,40 @@ gth_image_viewer_page_real_revert (GthViewerPage *base)
 }
 
 
+static void
+gth_image_viewer_page_real_update_info (GthViewerPage *base,
+					GthFileData   *file_data)
+{
+	GthImageViewerPage *self = GTH_IMAGE_VIEWER_PAGE (base);
+
+	if (! _g_file_equal (self->priv->file_data->file, file_data->file))
+		return;
+	_g_object_unref (self->priv->file_data);
+	self->priv->file_data = gth_file_data_dup (file_data);
+
+	if (self->priv->viewer == NULL)
+		return;
+
+	gth_image_viewer_update_view (GTH_IMAGE_VIEWER (self->priv->viewer));
+}
+
+
+static void
+gth_image_viewer_page_real_show_properties (GthViewerPage *base,
+					    gboolean       show)
+{
+	GthImageViewerPage *self;
+
+	self = GTH_IMAGE_VIEWER_PAGE (base);
+
+	if (show)
+		gth_image_viewer_add_painter (GTH_IMAGE_VIEWER (self->priv->viewer), paint_comment_over_image_func, self);
+	else
+		gth_image_viewer_remove_painter (GTH_IMAGE_VIEWER (self->priv->viewer), paint_comment_over_image_func, self);
+	gth_image_viewer_update_view (GTH_IMAGE_VIEWER (self->priv->viewer));
+}
+
+
 static void
 gth_image_viewer_page_finalize (GObject *obj)
 {
@@ -954,6 +1245,7 @@ gth_image_viewer_page_finalize (GObject *obj)
 
 	g_object_unref (self->priv->history);
 	_g_object_unref (self->priv->file_data);
+	_g_object_unref (self->priv->last_loaded);
 
 	G_OBJECT_CLASS (gth_image_viewer_page_parent_class)->finalize (obj);
 }
@@ -986,6 +1278,8 @@ gth_viewer_page_interface_init (GthViewerPageIface *iface)
 	iface->save = gth_image_viewer_page_real_save;
 	iface->save_as = gth_image_viewer_page_real_save_as;
 	iface->revert = gth_image_viewer_page_real_revert;
+	iface->update_info = gth_image_viewer_page_real_update_info;
+	iface->show_properties = gth_image_viewer_page_real_show_properties;
 }
 
 
@@ -994,6 +1288,8 @@ gth_image_viewer_page_instance_init (GthImageViewerPage *self)
 {
 	self->priv = GTH_IMAGE_VIEWER_PAGE_GET_PRIVATE (self);
 	self->priv->history = gth_image_history_new ();
+	self->priv->shrink_wrap = FALSE;
+	self->priv->last_loaded = NULL;
 }
 
 
@@ -1093,14 +1389,32 @@ gth_image_viewer_page_reset (GthImageViewerPage *self)
 }
 
 
+static int
+add_non_content_width (GthImageViewerPage *self,
+		       GtkWidget          *non_content)
+{
+	int width = 0;
+
+	if ((non_content != NULL) && gtk_widget_get_visible (non_content)) {
+		GtkAllocation allocation;
+
+		gtk_widget_get_allocation (non_content, &allocation);
+		width = allocation.width;
+	}
+
+	return width;
+}
+
+
 static int
 add_non_content_height (GthImageViewerPage *self,
 			GtkWidget          *non_content)
 {
 	int height = 0;
 
-	if ((non_content != NULL) && GTK_WIDGET_VISIBLE (non_content)) {
+	if ((non_content != NULL) && gtk_widget_get_visible (non_content)) {
 		GtkAllocation allocation;
+
 		gtk_widget_get_allocation (non_content, &allocation);
 		height = allocation.height;
 	}
@@ -1110,24 +1424,47 @@ add_non_content_height (GthImageViewerPage *self,
 
 
 void
-gth_image_viewer_page_shrink_wrap (GthImageViewerPage *self)
+gth_image_viewer_page_shrink_wrap (GthImageViewerPage *self,
+				   gboolean            activate)
 {
-	GdkPixbuf *pixbuf;
-	int        width;
-	int        height;
-	double     ratio;
-	int        other_width;
-	int        other_height;
-	GdkScreen *screen;
-	int        max_width;
-	int        max_height;
+	GthFileData *file_data;
+	int          width;
+	int          height;
+	double       ratio;
+	int          other_width;
+	int          other_height;
+	GdkScreen   *screen;
+	int          max_width;
+	int          max_height;
 
-	pixbuf = gth_image_viewer_page_get_pixbuf (self);
-	if (pixbuf == NULL)
+	self->priv->shrink_wrap = activate;
+	if (! self->priv->shrink_wrap) {
+		int width;
+		int height;
+
+		if (gth_window_get_page_size (GTH_WINDOW (self->priv->browser),
+					      GTH_BROWSER_PAGE_BROWSER,
+					      &width,
+					      &height))
+		{
+			gth_window_save_page_size (GTH_WINDOW (self->priv->browser), GTH_BROWSER_PAGE_VIEWER, width, height);
+			gth_window_apply_saved_size (GTH_WINDOW (self->priv->browser), GTH_BROWSER_PAGE_VIEWER);
+		}
+		else
+			gth_window_clear_saved_size (GTH_WINDOW (self->priv->browser), GTH_BROWSER_PAGE_VIEWER);
+		gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_FIT_SIZE_IF_LARGER);
+
+		return;
+	}
+
+	file_data = gth_browser_get_current_file (self->priv->browser);
+	if (file_data == NULL)
+		return;
+
+	gth_image_viewer_get_original_size (GTH_IMAGE_VIEWER (self->priv->viewer), &width, &height);
+	if ((width <= 0) || (height <= 0))
 		return;
 
-	width = gdk_pixbuf_get_width (pixbuf);
-	height = gdk_pixbuf_get_height (pixbuf);
 	ratio = (double) width / height;
 
 	other_width = 0;
@@ -1136,23 +1473,33 @@ gth_image_viewer_page_shrink_wrap (GthImageViewerPage *self)
 	other_height += add_non_content_height (self, gth_window_get_area (GTH_WINDOW (self->priv->browser), GTH_WINDOW_TOOLBAR));
 	other_height += add_non_content_height (self, gth_window_get_area (GTH_WINDOW (self->priv->browser), GTH_WINDOW_STATUSBAR));
 	other_height += add_non_content_height (self, gth_browser_get_viewer_toolbar (self->priv->browser));
+	if (eel_gconf_get_enum (PREF_UI_VIEWER_THUMBNAILS_ORIENT, GTK_TYPE_ORIENTATION, GTK_ORIENTATION_HORIZONTAL) == GTK_ORIENTATION_HORIZONTAL)
+		other_height += add_non_content_height (self, gth_browser_get_thumbnail_list (self->priv->browser));
+	else
+		other_width += add_non_content_width (self, gth_browser_get_thumbnail_list (self->priv->browser));
+	other_width += add_non_content_width (self, gth_browser_get_viewer_sidebar (self->priv->browser));
 	other_width += 2;
 	other_height += 2;
 
 	screen = gtk_widget_get_screen (GTK_WIDGET (self->priv->browser));
-	max_width = gdk_screen_get_width (screen) * 9 / 10;
-	max_height = gdk_screen_get_height (screen) * 8 / 10;
+	max_width = round ((double) gdk_screen_get_width (screen) * 8.5 / 10.0);
+	max_height = round ((double) gdk_screen_get_height (screen) * 8.5 / 10.0);
 
 	if (width + other_width > max_width) {
-		width = max_width;
+		width = max_width - other_width;
 		height = width / ratio;
 	}
 
 	if (height + other_height > max_height) {
-		height = max_height;
+		height = max_height - other_height;
 		width = height * ratio;
 	}
 
-	gtk_window_resize (GTK_WINDOW (self->priv->browser), width + other_width, height + other_height);
-	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_FIT_SIZE);
+	gth_window_save_page_size (GTH_WINDOW (self->priv->browser),
+				   GTH_BROWSER_PAGE_VIEWER,
+				   width + other_width,
+				   height + other_height);
+	if (gth_window_get_current_page (GTH_WINDOW (self->priv->browser)) == GTH_BROWSER_PAGE_VIEWER)
+		gth_window_apply_saved_size (GTH_WINDOW (self->priv->browser), GTH_BROWSER_PAGE_VIEWER);
+	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_FIT_SIZE_IF_LARGER);
 }
