@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -36,6 +35,7 @@ struct _GthImportTaskPrivate {
 	GthBrowser          *browser;
 	GList               *files;
 	GFile               *destination;
+	GHashTable          *destinations;
 	GthSubfolderType     subfolder_type;
 	GthSubfolderFormat   subfolder_format;
 	gboolean             single_subfolder;
@@ -56,6 +56,9 @@ struct _GthImportTaskPrivate {
 	GFile               *imported_catalog;
 	gboolean             delete_not_supported;
 	int                  n_imported;
+	GthOverwriteResponse default_response;
+	void                *buffer;
+	gsize                buffer_size;
 };
 
 
@@ -72,6 +75,8 @@ gth_import_task_finalize (GObject *object)
 	if (ImportPhotos)
 		gtk_window_present (GTK_WINDOW (self->priv->browser));
 
+	g_free (self->priv->buffer);
+	g_hash_table_unref (self->priv->destinations);
 	_g_object_list_unref (self->priv->files);
 	g_object_unref (self->priv->destination);
 	_g_object_unref (self->priv->destination_file);
@@ -152,11 +157,12 @@ catalog_imported_file (GthImportTask *self)
 		date_time = gth_datetime_new ();
 		gth_datetime_from_timeval (date_time, &timeval);
 
-		catalog_file = gth_catalog_get_file_for_date (date_time);
+		catalog_file = gth_catalog_get_file_for_date (date_time, ".catalog");
 		catalog = gth_catalog_load_from_file (catalog_file);
 		if (catalog == NULL)
 			catalog = gth_catalog_new ();
-		gth_catalog_set_for_date (catalog, date_time);
+		gth_catalog_set_date (catalog, date_time);
+		gth_catalog_set_file (catalog, catalog_file);
 
 		g_hash_table_insert (self->priv->catalogs, g_strdup (key), catalog);
 
@@ -176,10 +182,19 @@ catalog_imported_file (GthImportTask *self)
 
 
 static void
-write_metadata_ready_func (GError   *error,
-			   gpointer  user_data)
+write_metadata_ready_func (GObject      *source_object,
+	 	 	   GAsyncResult *result,
+	 	 	   gpointer      user_data)
 {
 	GthImportTask *self = user_data;
+	GError        *error = NULL;
+
+	if (! _g_write_metadata_finish (result, &error)
+	    && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+	{
+		gth_task_completed (GTH_TASK (self), error);
+		return;
+	}
 
 	if (error != NULL)
 		g_clear_error (&error);
@@ -196,6 +211,11 @@ transformation_ready_cb (GError   *error,
 	GthStringList *tag_list;
 	GList         *file_list;
 
+	if ((error != NULL) && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+		gth_task_completed (GTH_TASK (self), error);
+		return;
+	}
+
 	if ((self->priv->tags == NULL) || (self->priv->tags[0] == NULL)) {
 		catalog_imported_file (self);
 		return;
@@ -205,6 +225,7 @@ transformation_ready_cb (GError   *error,
 	g_file_info_set_attribute_object (self->priv->destination_file->info, "comment::categories", G_OBJECT (tag_list));
 	file_list = g_list_prepend (NULL, self->priv->destination_file);
 	_g_write_metadata_async (file_list,
+				 GTH_METADATA_WRITE_DEFAULT,
 				 "comment::categories",
 				 gth_task_get_cancellable (GTH_TASK (self)),
 				 write_metadata_ready_func,
@@ -216,21 +237,151 @@ transformation_ready_cb (GError   *error,
 
 
 static void
-write_buffer_ready_cb (void     **buffer,
-		       gsize      count,
-		       GError    *error,
-		       gpointer   user_data)
+write_file_to_destination (GthImportTask *self,
+			   GFile         *destination_file,
+			   void          *buffer,
+			   gsize          count,
+			   gboolean       overwrite);
+
+
+static void
+overwrite_dialog_response_cb (GtkDialog *dialog,
+                              gint       response_id,
+                              gpointer   user_data)
 {
 	GthImportTask *self = user_data;
-	GthFileData   *file_data;
-	gboolean       appling_tranformation = FALSE;
+
+	if (response_id != GTK_RESPONSE_OK)
+		self->priv->default_response = GTH_OVERWRITE_RESPONSE_CANCEL;
+	else
+		self->priv->default_response = gth_overwrite_dialog_get_response (GTH_OVERWRITE_DIALOG (dialog));
+
+	gtk_widget_hide (GTK_WIDGET (dialog));
+	gth_task_dialog (GTH_TASK (self), FALSE, NULL);
+
+	switch (self->priv->default_response) {
+	case GTH_OVERWRITE_RESPONSE_NO:
+	case GTH_OVERWRITE_RESPONSE_ALWAYS_NO:
+	case GTH_OVERWRITE_RESPONSE_UNSPECIFIED:
+		import_next_file (self);
+		break;
+
+	case GTH_OVERWRITE_RESPONSE_YES:
+	case GTH_OVERWRITE_RESPONSE_ALWAYS_YES:
+		write_file_to_destination (self,
+					   self->priv->destination_file->file,
+					   self->priv->buffer,
+					   self->priv->buffer_size,
+					   TRUE);
+		break;
+
+	case GTH_OVERWRITE_RESPONSE_RENAME:
+		{
+			GthFileData *file_data;
+			GFile       *destination_folder;
+			GFile       *new_destination;
+
+			file_data = self->priv->current->data;
+			destination_folder = gth_import_utils_get_file_destination (file_data,
+										    self->priv->destination,
+										    self->priv->subfolder_type,
+										    self->priv->subfolder_format,
+										    self->priv->single_subfolder,
+										    self->priv->custom_format,
+										    self->priv->event_name,
+										    self->priv->import_start_time);
+			new_destination = g_file_get_child_for_display_name (destination_folder, gth_overwrite_dialog_get_filename (GTH_OVERWRITE_DIALOG (dialog)), NULL);
+			write_file_to_destination (self,
+						   new_destination,
+						   self->priv->buffer,
+						   self->priv->buffer_size,
+						   FALSE);
+
+			g_object_unref (new_destination);
+			g_object_unref (destination_folder);
+		}
+		break;
+
+	case GTH_OVERWRITE_RESPONSE_CANCEL:
+		{
+			GError *error;
+
+			error = g_error_new_literal (GTH_TASK_ERROR, GTH_TASK_ERROR_CANCELLED, "");
+			gth_task_completed (GTH_TASK (self), error);
+		}
+		break;
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+
+static void
+after_saving_to_destination (GthImportTask  *self,
+			     void          **buffer,
+			     gsize           count,
+			     GError         *error)
+{
+	GthFileData *file_data;
+	gboolean     appling_tranformation = FALSE;
+
+	file_data = self->priv->current->data;
 
 	if (error != NULL) {
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+			if (self->priv->default_response != GTH_OVERWRITE_RESPONSE_ALWAYS_NO) {
+				GInputStream *stream;
+				GdkPixbuf    *pixbuf;
+				GtkWidget    *dialog;
+
+				/* take ownership of the buffer */
+
+				if (buffer != NULL) {
+					self->priv->buffer = *buffer;
+					self->priv->buffer_size = count;
+					*buffer = NULL;
+				}
+				else {
+					self->priv->buffer = NULL;
+					self->priv->buffer_size = 0;
+				}
+
+				/* show the overwrite dialog */
+
+				if (self->priv->buffer != NULL) {
+					stream = g_memory_input_stream_new_from_data (self->priv->buffer, self->priv->buffer_size, NULL);
+					pixbuf = gdk_pixbuf_new_from_stream_at_scale (stream, 128, 128, TRUE, NULL, NULL);
+				}
+				else {
+					stream = NULL;
+					pixbuf = NULL;
+				}
+
+				dialog = gth_overwrite_dialog_new (file_data->file,
+								   pixbuf,
+								   self->priv->destination_file->file,
+								   self->priv->default_response,
+								   self->priv->files->next == NULL);
+				g_signal_connect (dialog,
+						  "response",
+						  G_CALLBACK (overwrite_dialog_response_cb),
+						  self);
+				gtk_widget_show (dialog);
+				gth_task_dialog (GTH_TASK (self), TRUE, dialog);
+
+				_g_object_unref (pixbuf);
+				_g_object_unref (stream);
+			}
+			else
+				import_next_file (self);
+
+			return;
+		}
+
 		gth_task_completed (GTH_TASK (self), error);
 		return;
 	}
 
-	file_data = self->priv->current->data;
 	if (self->priv->delete_imported) {
 		GError *local_error = NULL;
 
@@ -251,28 +402,13 @@ write_buffer_ready_cb (void     **buffer,
 	}
 
 	if (self->priv->adjust_orientation && gth_main_extension_is_active ("image_rotation")) {
-		GthMetadata *metadata;
-
-		metadata = (GthMetadata *) g_file_info_get_attribute_object (self->priv->destination_file->info, "Embedded::Image::Orientation");
-		if (metadata != NULL) {
-			const char *value;
-
-			value = gth_metadata_get_raw (metadata);
-			if (value != NULL) {
-				GthTransform transform;
-
-				transform = (GthTransform) strtol (value, (char **) NULL, 10);
-				if (transform != 1) {
-					apply_transformation_async (self->priv->destination_file,
-								    transform,
-								    JPEG_MCU_ACTION_ABORT,
-								    gth_task_get_cancellable (GTH_TASK (self)),
-								    transformation_ready_cb,
-								    self);
-					appling_tranformation = TRUE;
-				}
-			}
-		}
+		apply_transformation_async (self->priv->destination_file,
+					    GTH_TRANSFORM_NONE,
+					    JPEG_MCU_ACTION_ABORT,
+					    gth_task_get_cancellable (GTH_TASK (self)),
+					    transformation_ready_cb,
+					    self);
+		appling_tranformation = TRUE;
 	}
 
 	if (! appling_tranformation)
@@ -281,24 +417,99 @@ write_buffer_ready_cb (void     **buffer,
 
 
 static void
-file_buffer_ready_cb (void     **buffer,
-		      gsize      count,
-		      GError    *error,
-		      gpointer   user_data)
+copy_non_image_ready_cb (GObject      *source_object,
+			 GAsyncResult *res,
+			 gpointer      user_data)
+{
+	GError *error = NULL;
+
+	g_file_copy_finish (G_FILE (source_object), res, &error);
+	after_saving_to_destination (GTH_IMPORT_TASK (user_data), NULL, 0, error);
+}
+
+
+static void
+copy_non_image_progress_cb (goffset  current_num_bytes,
+			    goffset  total_num_bytes,
+			    gpointer user_data)
 {
 	GthImportTask *self = user_data;
 	GthFileData   *file_data;
-	GFile         *destination;
-	GFile         *destination_file;
-
-	if (error != NULL) {
-		gth_task_completed (GTH_TASK (self), error);
-		return;
-	}
 
 	file_data = self->priv->current->data;
-	if (gth_main_extension_is_active ("exiv2_tools"))
-		exiv2_read_metadata_from_buffer (*buffer, count, file_data->info, NULL);
+
+	gth_task_progress (GTH_TASK (self),
+			   _("Importing files"),
+			   g_file_info_get_display_name (file_data->info),
+			   FALSE,
+			   (double) (self->priv->copied_size + current_num_bytes) / self->priv->tot_size);
+}
+
+
+static void
+write_buffer_ready_cb (void     **buffer,
+		       gsize      count,
+		       GError    *error,
+		       gpointer   user_data)
+{
+	after_saving_to_destination (GTH_IMPORT_TASK (user_data), buffer, count, error);
+}
+
+
+static void
+write_file_to_destination (GthImportTask *self,
+			   GFile         *destination_file,
+			   void          *buffer,
+			   gsize          count,
+			   gboolean       replace)
+{
+	GthFileData *file_data;
+
+	file_data = self->priv->current->data;
+
+	if ((self->priv->destination_file == NULL) || (destination_file != self->priv->destination_file->file)) {
+		_g_object_unref (self->priv->destination_file);
+		self->priv->destination_file = gth_file_data_new (destination_file, file_data->info);
+	}
+
+	if (buffer != NULL) {
+		gth_task_progress (GTH_TASK (self),
+				   _("Importing files"),
+				   g_file_info_get_display_name (file_data->info),
+				   FALSE,
+				   (double) (self->priv->copied_size + ((double) self->priv->current_file_size / 3.0 * 2.0)) / self->priv->tot_size);
+
+		g_write_file_async (self->priv->destination_file->file,
+				    buffer,
+				    count,
+				    replace,
+				    G_PRIORITY_DEFAULT,
+				    gth_task_get_cancellable (GTH_TASK (self)),
+				    write_buffer_ready_cb,
+				    self);
+
+		self->priv->buffer = NULL; /* the buffer will be deallocated in g_write_file_async */
+	}
+	else
+		g_file_copy_async (file_data->file,
+				   self->priv->destination_file->file,
+				   (replace ? G_FILE_COPY_OVERWRITE : G_FILE_COPY_NONE),
+				   G_PRIORITY_DEFAULT,
+				   gth_task_get_cancellable (GTH_TASK (self)),
+				   copy_non_image_progress_cb,
+				   self,
+				   copy_non_image_ready_cb,
+				   self);
+}
+
+
+static GFile *
+get_destination_file (GthImportTask *self,
+		      GthFileData   *file_data)
+{
+	GError *error = NULL;
+	GFile  *destination;
+	GFile  *destination_file;
 
 	destination = gth_import_utils_get_file_destination (file_data,
 							     self->priv->destination,
@@ -311,36 +522,59 @@ file_buffer_ready_cb (void     **buffer,
 	if (! g_file_make_directory_with_parents (destination, gth_task_get_cancellable (GTH_TASK (self)), &error)) {
 		if (! g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
 			gth_task_completed (GTH_TASK (self), error);
-			return;
+			return NULL;
 		}
 	}
 
+	/* Get the destination file avoiding to overwrite an already imported
+	 * file. */
+
 	destination_file = _g_file_get_destination (file_data->file, NULL, destination);
-	if (self->priv->overwrite_files || ! g_file_query_exists (destination_file, NULL)) {
-		_g_object_unref (self->priv->destination_file);
-		self->priv->destination_file = gth_file_data_new (destination_file, file_data->info);
-
-		gth_task_progress (GTH_TASK (self),
-				   _("Importing files"),
-				   g_file_info_get_display_name (file_data->info),
-				   FALSE,
-				   (self->priv->copied_size + ((double) self->priv->current_file_size / 3.0 * 2.0)) / self->priv->tot_size);
-
-		g_write_file_async (self->priv->destination_file->file,
-				    *buffer,
-				    count,
-				    TRUE,
-				    G_PRIORITY_DEFAULT,
-				    gth_task_get_cancellable (GTH_TASK (self)),
-				    write_buffer_ready_cb,
-				    self);
-		*buffer = NULL; /* g_write_file_async takes ownership of the buffer */
+	while (g_hash_table_lookup (self->priv->destinations, destination_file) != NULL) {
+		GFile *tmp = destination_file;
+		destination_file = _g_file_get_duplicated (tmp);
+		g_object_unref (tmp);
 	}
-	else
-		call_when_idle ((DataFunc) import_next_file, self);
+	g_hash_table_insert (self->priv->destinations, g_object_ref (destination_file), GINT_TO_POINTER (1));
+
+	g_object_unref (destination);
+
+	return destination_file;
+}
+
+
+static void
+file_buffer_ready_cb (void     **buffer,
+		      gsize      count,
+		      GError    *error,
+		      gpointer   user_data)
+{
+	GthImportTask *self = user_data;
+	GthFileData   *file_data;
+	GFile         *destination_file;
+
+	if (error != NULL) {
+		gth_task_completed (GTH_TASK (self), error);
+		return;
+	}
+
+	file_data = self->priv->current->data;
+
+	if (gth_main_extension_is_active ("exiv2_tools"))
+		exiv2_read_metadata_from_buffer (*buffer, count, file_data->info, NULL);
+
+	destination_file = get_destination_file (self, file_data);
+	if (destination_file == NULL)
+		return;
+
+	write_file_to_destination (self,
+				   destination_file,
+				   *buffer,
+				   count,
+				   self->priv->default_response == GTH_OVERWRITE_RESPONSE_ALWAYS_YES);
+	*buffer = NULL; /* g_write_file_async takes ownership of the buffer */
 
 	g_object_unref (destination_file);
-	g_object_unref (destination);
 }
 
 
@@ -349,6 +583,9 @@ import_current_file (GthImportTask *self)
 {
 	GthFileData *file_data;
 
+	g_free (self->priv->buffer);
+	self->priv->buffer = NULL;
+
 	if (self->priv->current == NULL) {
 		save_catalogs (self);
 
@@ -399,17 +636,34 @@ import_current_file (GthImportTask *self)
 	file_data = self->priv->current->data;
 	self->priv->current_file_size = g_file_info_get_size (file_data->info);
 
-	gth_task_progress (GTH_TASK (self),
-			   _("Importing files"),
-			   g_file_info_get_display_name (file_data->info),
-			   FALSE,
-			   (self->priv->copied_size + ((double) self->priv->current_file_size / 3.0)) / self->priv->tot_size);
+	if (_g_mime_type_is_image (gth_file_data_get_mime_type (file_data))
+	    && (self->priv->subfolder_type == GTH_SUBFOLDER_TYPE_FILE_DATE))
+	{
+		gth_task_progress (GTH_TASK (self),
+				   _("Importing files"),
+				   g_file_info_get_display_name (file_data->info),
+				   FALSE,
+				   (double) (self->priv->copied_size + ((double) self->priv->current_file_size / 3.0)) / self->priv->tot_size);
 
-	g_load_file_async (file_data->file,
-			   G_PRIORITY_DEFAULT,
-			   gth_task_get_cancellable (GTH_TASK (self)),
-			   file_buffer_ready_cb,
-			   self);
+		g_load_file_async (file_data->file,
+				   G_PRIORITY_DEFAULT,
+				   gth_task_get_cancellable (GTH_TASK (self)),
+				   file_buffer_ready_cb,
+				   self);
+	}
+	else {
+		GFile *destination_file;
+
+		destination_file = get_destination_file (self, file_data);
+		if (destination_file != NULL) {
+			write_file_to_destination (self,
+						   destination_file,
+						   NULL,
+						   0,
+						   self->priv->default_response == GTH_OVERWRITE_RESPONSE_ALWAYS_YES);
+			g_object_unref (destination_file);
+		}
+	}
 }
 
 
@@ -428,6 +682,7 @@ gth_import_task_exec (GthTask *base)
 	}
 	g_get_current_time (&timeval);
 	self->priv->import_start_time = timeval;
+	self->priv->default_response = GTH_OVERWRITE_RESPONSE_UNSPECIFIED;
 
 	/* create the imported files catalog */
 
@@ -464,6 +719,7 @@ gth_import_task_exec (GthTask *base)
 		gth_datetime_free (date_time);
 	}
 
+	self->priv->buffer = NULL;
 	self->priv->current = self->priv->files;
 	import_current_file (self);
 }
@@ -492,6 +748,11 @@ gth_import_task_init (GthImportTask *self)
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_IMPORT_TASK, GthImportTaskPrivate);
 	self->priv->catalogs = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 	self->priv->delete_not_supported = FALSE;
+	self->priv->destinations = g_hash_table_new_full (g_file_hash,
+							  (GEqualFunc) g_file_equal,
+							  g_object_unref,
+							  NULL);
+	self->priv->buffer = NULL;
 }
 
 
@@ -554,6 +815,7 @@ gth_import_task_new (GthBrowser         *browser,
 	self->priv->tags = g_strdupv (tags);
 	self->priv->delete_imported = delete_imported;
 	self->priv->overwrite_files = overwrite_files;
+	self->priv->default_response = overwrite_files ? GTH_OVERWRITE_RESPONSE_ALWAYS_YES : GTH_OVERWRITE_RESPONSE_UNSPECIFIED;
 	self->priv->adjust_orientation = adjust_orientation;
 
 	return (GthTask *) self;
