@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -155,6 +154,10 @@ export_completed_with_success (DialogData *data)
 	builder = _gtk_builder_new_from_file ("photobucket-export-completed.ui", "photobucket");
 	dialog = _gtk_builder_get_widget (builder, "completed_messagedialog");
 	g_object_set_data_full (G_OBJECT (dialog), "builder", builder, g_object_unref);
+	g_signal_connect (dialog,
+			  "delete-event",
+			  G_CALLBACK (gtk_true),
+			  NULL);
 	g_signal_connect (dialog,
 			  "response",
 			  G_CALLBACK (completed_messagedialog_response_cb),
@@ -193,7 +196,7 @@ export_dialog_response_cb (GtkDialog *dialog,
 
 	switch (response_id) {
 	case GTK_RESPONSE_HELP:
-		show_help_dialog (GTK_WINDOW (data->browser), "export-to-photobucket");
+		show_help_dialog (GTK_WINDOW (data->browser), "gthumb-export-social");
 		break;
 
 	case GTK_RESPONSE_DELETE_EVENT:
@@ -515,6 +518,10 @@ add_album_button_clicked_cb (GtkButton *button,
 
 	dialog = photobucket_album_properties_dialog_new (g_file_info_get_edit_name (data->location->info),
 							  data->albums);
+	g_signal_connect (dialog,
+			  "delete-event",
+			  G_CALLBACK (gtk_true),
+			  NULL);
 	g_signal_connect (dialog,
 			  "response",
 			  G_CALLBACK (new_album_dialog_response_cb),
@@ -615,10 +622,11 @@ dlg_export_to_photobucket (GthBrowser *browser,
 
 	/* Set the widget data */
 
-	list_view = gth_file_list_new (GTH_FILE_LIST_TYPE_NO_SELECTION, FALSE);
+	list_view = gth_file_list_new (gth_icon_view_new (), GTH_FILE_LIST_TYPE_NO_SELECTION, FALSE);
 	gth_file_list_set_thumb_size (GTH_FILE_LIST (list_view), 112);
 	gth_file_view_set_spacing (GTH_FILE_VIEW (gth_file_list_get_view (GTH_FILE_LIST (list_view))), 0);
 	gth_file_list_enable_thumbs (GTH_FILE_LIST (list_view), TRUE);
+	gth_file_list_set_ignore_hidden (GTH_FILE_LIST (list_view), TRUE);
 	gth_file_list_set_caption (GTH_FILE_LIST (list_view), "none");
 	gth_file_list_set_sort_func (GTH_FILE_LIST (list_view), gth_main_get_sort_type ("file::name")->cmp_func, FALSE);
 	gtk_widget_show (list_view);
@@ -637,6 +645,10 @@ dlg_export_to_photobucket (GthBrowser *browser,
 			  "destroy",
 			  G_CALLBACK (export_dialog_destroy_cb),
 			  data);
+	g_signal_connect (data->dialog,
+			  "delete-event",
+			  G_CALLBACK (gtk_true),
+			  NULL);
 	g_signal_connect (data->dialog,
 			  "response",
 			  G_CALLBACK (export_dialog_response_cb),
