@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -34,7 +33,8 @@
 #include "gth-slideshow.h"
 #include "gth-transition.h"
 
-#define HIDE_CURSOR_DELAY 1000
+#define HIDE_CURSOR_DELAY 1
+#define HIDE_PAUSED_SIGN_DELAY 1
 #define DEFAULT_DELAY 2000
 
 
@@ -60,7 +60,9 @@ struct _GthSlideshowPrivate {
 	ClutterTimeline       *timeline;
 	ClutterActor          *image1;
 	ClutterActor          *image2;
+	ClutterActor          *paused_actor;
 #endif
+	GdkPixbuf             *current_pixbuf;
 	GtkWidget             *viewer;
 	guint                  next_event;
 	guint                  delay;
@@ -74,8 +76,12 @@ struct _GthSlideshowPrivate {
 	int                    current_audio_file;
 	GstElement            *playbin;
 #endif
+	GdkPixbuf             *pause_pixbuf;
 	gboolean               paused;
+	gboolean               paint_paused;
+	guint                  hide_paused_sign;
 	gboolean               animating;
+	gboolean               random_order;
 };
 
 
@@ -89,7 +95,7 @@ _gth_slideshow_close (GthSlideshow *self)
 	GthBrowser *browser;
 
 	browser = self->priv->browser;
-	close_browser = ! GTK_WIDGET_VISIBLE (browser);
+	close_browser = ! gtk_widget_get_visible (GTK_WIDGET (browser));
 	gtk_widget_destroy (GTK_WIDGET (self));
 
 	if (close_browser)
@@ -97,12 +103,37 @@ _gth_slideshow_close (GthSlideshow *self)
 }
 
 
+static int
+shuffle_func (gconstpointer a,
+	      gconstpointer b)
+{
+        return g_random_int_range (-1, 2);
+}
+
+
+
+static void
+_gth_slideshow_reset_current (GthSlideshow *self)
+{
+	if (self->priv->random_order)
+		self->priv->file_list = g_list_sort (self->priv->file_list, shuffle_func);
+
+	if (self->priv->direction == GTH_SLIDESHOW_DIRECTION_FORWARD)
+		self->priv->current = g_list_first (self->priv->file_list);
+	else
+		self->priv->current = g_list_last (self->priv->file_list);
+}
+
+
 static void
 _gth_slideshow_load_current_image (GthSlideshow *self)
 {
 	GthFileData *requested_file;
 	GthFileData *next_file;
 	GthFileData *prev_file;
+	int          screen_width;
+	int          screen_height;
+	GdkScreen   *screen;
 
 	if (self->priv->next_event != 0) {
 		g_source_remove (self->priv->next_event);
@@ -114,11 +145,7 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 			_gth_slideshow_close (self);
 			return;
 		}
-
-		if (self->priv->direction == GTH_SLIDESHOW_DIRECTION_FORWARD)
-			self->priv->current = g_list_first (self->priv->file_list);
-		else
-			self->priv->current = g_list_last (self->priv->file_list);
+		_gth_slideshow_reset_current (self);
 	}
 
 	requested_file = (GthFileData *) self->priv->current->data;
@@ -130,10 +157,21 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 		prev_file = (GthFileData *) self->priv->current->prev->data;
 	else
 		prev_file = NULL;
+
+	screen_width = -1;
+	screen_height = -1;
+	screen = gtk_widget_get_screen (GTK_WIDGET (self));
+	if (screen != NULL) {
+		screen_width = gdk_screen_get_width (screen);
+		screen_height = gdk_screen_get_height (screen);
+	}
+
 	gth_image_preloader_load (self->priv->preloader,
 				  requested_file,
+				  MAX (screen_width, screen_height),
 				  next_file,
-				  prev_file);
+				  prev_file,
+				  NULL);
 }
 
 
@@ -192,13 +230,16 @@ view_next_image_automatically (GthSlideshow *self)
 
 
 static void
-image_preloader_requested_ready_cb (GthImagePreloader *preloader,
-				    GError            *error,
-				    gpointer           user_data)
+image_preloader_requested_ready_cb (GthImagePreloader  *preloader,
+				    GthFileData        *requested,
+				    GdkPixbufAnimation *animation,
+				    int                 original_width,
+				    int                 original_height,
+				    GError             *error,
+				    gpointer            user_data)
 {
-	GthSlideshow   *self = user_data;
-	GthImageLoader *image_loader;
-	GdkPixbuf      *pixbuf;
+	GthSlideshow *self = user_data;
+	GdkPixbuf    *static_image;
 
 	if (error != NULL) {
 		g_clear_error (&error);
@@ -206,20 +247,21 @@ image_preloader_requested_ready_cb (GthImagePreloader *preloader,
 		return;
 	}
 
-	image_loader = gth_image_preloader_get_loader (self->priv->preloader, (GthFileData *) self->priv->current->data);
-	if (image_loader == NULL) {
-		_gth_slideshow_load_next_image (self);
-		return;
-	}
+	_g_object_unref (self->priv->current_pixbuf);
 
-	pixbuf = gth_image_loader_get_pixbuf (image_loader);
-	if (pixbuf == NULL) {
+	static_image = gdk_pixbuf_animation_get_static_image (animation);
+	if (static_image != NULL)
+		self->priv->current_pixbuf = gdk_pixbuf_copy (static_image);
+	else
+		self->priv->current_pixbuf = NULL;
+
+	if (self->priv->current_pixbuf == NULL) {
 		_gth_slideshow_load_next_image (self);
 		return;
 	}
 
 	self->priv->one_loaded = TRUE;
-	self->priv->projector->image_ready (self, pixbuf);
+	self->priv->projector->image_ready (self, self->priv->current_pixbuf);
 }
 
 
@@ -240,8 +282,10 @@ gth_slideshow_init (GthSlideshow *self)
 	self->priv->paused = FALSE;
 	self->priv->animating = FALSE;
 	self->priv->direction = GTH_SLIDESHOW_DIRECTION_FORWARD;
+	self->priv->random_order = FALSE;
+	self->priv->current_pixbuf = NULL;
 
-	self->priv->preloader = gth_image_preloader_new ();
+	self->priv->preloader = gth_image_preloader_new (GTH_LOAD_POLICY_ONE_STEP, 3);
 	g_signal_connect (self->priv->preloader,
 			  "requested_ready",
 			  G_CALLBACK (image_preloader_requested_ready_cb),
@@ -259,6 +303,8 @@ gth_slideshow_finalize (GObject *object)
 	if (self->priv->hide_cursor_event != 0)
 		g_source_remove (self->priv->hide_cursor_event);
 
+	_g_object_unref (self->priv->pause_pixbuf);
+	_g_object_unref (self->priv->current_pixbuf);
 	_g_object_list_unref (self->priv->file_list);
 	_g_object_unref (self->priv->browser);
 	_g_object_unref (self->priv->preloader);
@@ -384,28 +430,31 @@ gth_slideshow_show_cb (GtkWidget    *widget,
 	if (! self->priv->first_show)
 		return;
 
-#if HAVE_GSTREAMER
-	if (gstreamer_init ()) {
-		if ((self->priv->audio_files != NULL) && (self->priv->audio_files[0] != NULL)) {
-			self->priv->current_audio_file = 0;
-			if (self->priv->playbin == NULL) {
-				GstBus *bus;
+	self->priv->first_show = FALSE;
 
-				self->priv->playbin = gst_element_factory_make ("playbin", "playbin");
-				bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->playbin));
-				gst_bus_add_signal_watch (bus);
-				g_signal_connect (bus, "message", G_CALLBACK (bus_message_cb), self);
-			}
-			else
-				gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
-			g_object_set (G_OBJECT (self->priv->playbin), "uri", self->priv->audio_files[self->priv->current_audio_file], NULL);
-			gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+#if HAVE_GSTREAMER
+	if ((self->priv->audio_files != NULL)
+	    && (self->priv->audio_files[0] != NULL)
+	    && gstreamer_init ())
+	{
+		self->priv->current_audio_file = 0;
+		if (self->priv->playbin == NULL) {
+			GstBus *bus;
+
+			self->priv->playbin = gst_element_factory_make ("playbin", "playbin");
+			bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->playbin));
+			gst_bus_add_signal_watch (bus);
+			g_signal_connect (bus, "message", G_CALLBACK (bus_message_cb), self);
 		}
+		else
+			gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
+		g_object_set (G_OBJECT (self->priv->playbin), "uri", self->priv->audio_files[self->priv->current_audio_file], NULL);
+		gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
 	}
 #endif
 
+	_gth_slideshow_reset_current (self);
 	_gth_slideshow_load_current_image (self);
-	self->priv->first_show = FALSE;
 }
 
 
@@ -418,8 +467,18 @@ _gth_slideshow_construct (GthSlideshow *self,
 	self->priv->projector = projector;
 	self->priv->browser = _g_object_ref (browser);
 	self->priv->file_list = _g_object_list_ref (file_list);
-	self->priv->current = self->priv->file_list;
 	self->priv->one_loaded = FALSE;
+	self->priv->pause_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+							     "slideshow-pause",
+							     100,
+							     0,
+							     NULL);
+	if (self->priv->pause_pixbuf == NULL)
+		self->priv->pause_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+								     GTK_STOCK_MEDIA_PAUSE,
+								     100,
+								     0,
+								     NULL);
 
 	self->priv->projector->construct (self);
 
@@ -486,6 +545,14 @@ gth_slideshow_set_playlist (GthSlideshow  *self,
 }
 
 
+void
+gth_slideshow_set_random_order (GthSlideshow *self,
+				gboolean      random)
+{
+	self->priv->random_order = random;
+}
+
+
 /* -- default projector -- */
 
 
@@ -507,7 +574,7 @@ static void
 default_projector_image_ready (GthSlideshow *self,
 			       GdkPixbuf    *pixbuf)
 {
-	gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer), pixbuf);
+	gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer), pixbuf, -1, -1);
 	view_next_image_automatically (self);
 }
 
@@ -515,7 +582,10 @@ default_projector_image_ready (GthSlideshow *self,
 static void
 default_projector_finalize (GthSlideshow *self)
 {
-	/* void */
+	if (self->priv->hide_paused_sign != 0) {
+		g_source_remove (self->priv->hide_paused_sign);
+		self->priv->hide_paused_sign = 0;
+	}
 }
 
 
@@ -529,7 +599,12 @@ default_projector_hide_cursor (GthSlideshow *self)
 static void
 default_projector_paused (GthSlideshow *self)
 {
-	/* void */
+	if (self->priv->hide_paused_sign != 0) {
+		g_source_remove (self->priv->hide_paused_sign);
+		self->priv->hide_paused_sign = 0;
+	}
+	self->priv->paint_paused = TRUE;
+	gtk_widget_queue_draw (self->priv->viewer);
 }
 
 
@@ -542,7 +617,7 @@ viewer_event_cb (GtkWidget    *widget,
 		gth_image_viewer_show_cursor (GTH_IMAGE_VIEWER (self->priv->viewer));
 		if (self->priv->hide_cursor_event != 0)
 			g_source_remove (self->priv->hide_cursor_event);
-		self->priv->hide_cursor_event = g_timeout_add (HIDE_CURSOR_DELAY, hide_cursor_cb, self);
+		self->priv->hide_cursor_event = g_timeout_add_seconds (HIDE_CURSOR_DELAY, hide_cursor_cb, self);
 	}
 	else if (event->type == GDK_BUTTON_PRESS) {
 		switch (((GdkEventButton *) event)->button) {
@@ -556,22 +631,34 @@ viewer_event_cb (GtkWidget    *widget,
 			break;
 		}
 	}
+	else if (event->type == GDK_KEY_PRESS) {
+		switch (((GdkEventKey *) event)->keyval) {
+		case GDK_F5:
+			_gth_slideshow_close (self);
+			break;
+		}
+	}
 	else if (event->type == GDK_KEY_RELEASE) {
 		switch (((GdkEventKey *) event)->keyval) {
 		case GDK_Escape:
+		case GDK_q:
 			_gth_slideshow_close (self);
 			break;
-
-		case GDK_space:
+		case GDK_p:
 			_gth_slideshow_toggle_pause (self);
 			break;
 
+		case GDK_space:
 		case GDK_Down:
 		case GDK_Right:
 		case GDK_Page_Down:
-			_gth_slideshow_load_next_image (self);
+			if (self->priv->paused)
+				_gth_slideshow_toggle_pause (self);
+			else
+				_gth_slideshow_load_next_image (self);
 			break;
 
+		case GDK_BackSpace:
 		case GDK_Up:
 		case GDK_Left:
 		case GDK_Page_Up:
@@ -582,9 +669,57 @@ viewer_event_cb (GtkWidget    *widget,
 }
 
 
+static gboolean
+hide_paused_sign_cb (gpointer user_data)
+{
+	GthSlideshow *self = user_data;
+
+	g_source_remove (self->priv->hide_paused_sign);
+	self->priv->hide_paused_sign = 0;
+
+	self->priv->paint_paused = FALSE;
+	gtk_widget_queue_draw (self->priv->viewer);
+
+	return FALSE;
+}
+
+
+static void
+default_projector_pause_painter (GthImageViewer *image_viewer,
+				 GdkEventExpose *event,
+				 cairo_t        *cr,
+				 gpointer        user_data)
+{
+	GthSlideshow *self = user_data;
+	GdkScreen    *screen;
+	double        dest_x;
+	double        dest_y;
+
+	if (! self->priv->paused || ! self->priv->paint_paused || (self->priv->pause_pixbuf == NULL))
+		return;
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (image_viewer));
+	if (screen == NULL)
+		return;
+
+	dest_x = (gdk_screen_get_width (screen) - gdk_pixbuf_get_width (self->priv->pause_pixbuf)) / 2.0;
+	dest_y = (gdk_screen_get_height (screen) - gdk_pixbuf_get_height (self->priv->pause_pixbuf)) / 2.0;
+	gdk_cairo_set_source_pixbuf (cr, self->priv->pause_pixbuf, dest_x, dest_y);
+	cairo_rectangle (cr, dest_x, dest_y, gdk_pixbuf_get_width (self->priv->pause_pixbuf), gdk_pixbuf_get_height (self->priv->pause_pixbuf));
+	cairo_fill (cr);
+
+	if (self->priv->hide_paused_sign != 0)
+		g_source_remove (self->priv->hide_paused_sign);
+	self->priv->hide_paused_sign = g_timeout_add_seconds (HIDE_PAUSED_SIGN_DELAY, hide_paused_sign_cb, self);
+}
+
+
 static void
 default_projector_construct (GthSlideshow *self)
 {
+	self->priv->hide_paused_sign = 0;
+	self->priv->paint_paused = FALSE;
+
 	self->priv->viewer = gth_image_viewer_new ();
 	gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), TRUE);
 	gth_image_viewer_hide_frame (GTH_IMAGE_VIEWER (self->priv->viewer));
@@ -592,9 +727,11 @@ default_projector_construct (GthSlideshow *self)
 	gth_image_viewer_set_zoom_change (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_ZOOM_CHANGE_FIT_SIZE);
 	gth_image_viewer_set_transp_type (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_TRANSP_TYPE_BLACK);
 	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_ZOOM_QUALITY_LOW);
+	gth_image_viewer_add_painter (GTH_IMAGE_VIEWER (self->priv->viewer), default_projector_pause_painter, self);
 
 	g_signal_connect (self->priv->viewer, "button-press-event", G_CALLBACK (viewer_event_cb), self);
 	g_signal_connect (self->priv->viewer, "motion-notify-event", G_CALLBACK (viewer_event_cb), self);
+	g_signal_connect (self->priv->viewer, "key-press-event", G_CALLBACK (viewer_event_cb), self);
 	g_signal_connect (self->priv->viewer, "key-release-event", G_CALLBACK (viewer_event_cb), self);
 
 	gtk_widget_show (self->priv->viewer);
@@ -854,10 +991,28 @@ clutter_projector_hide_cursor (GthSlideshow *self)
 static void
 clutter_projector_paused (GthSlideshow *self)
 {
+	float stage_w;
+	float stage_h;
+
 	if (self->priv->animating) {
 		clutter_timeline_pause (self->priv->timeline);
 		_gth_slideshow_animation_completed (self);
 	}
+
+	clutter_actor_get_size (self->stage, &stage_w, &stage_h);
+	clutter_actor_set_position (self->priv->paused_actor, stage_w / 2.0, stage_h / 2.0);
+	clutter_actor_set_anchor_point_from_gravity (self->priv->paused_actor, CLUTTER_GRAVITY_CENTER);
+	clutter_actor_set_scale (self->priv->paused_actor, 1.0, 1.0);
+	clutter_actor_set_opacity (self->priv->paused_actor, 255);
+	clutter_actor_raise_top (self->priv->paused_actor);
+	clutter_actor_show (self->priv->paused_actor);
+
+	clutter_actor_animate (self->priv->paused_actor,
+			       CLUTTER_LINEAR, 500,
+	                       "opacity", 0,
+	                       "scale-x", 3.0,
+	                       "scale-y", 3.0,
+	                       NULL);
 }
 
 
@@ -884,25 +1039,37 @@ stage_input_cb (ClutterStage *stage,
 			break;
 		}
 	}
+	else if (event->type == CLUTTER_KEY_PRESS) {
+		switch (clutter_event_get_key_symbol (event)) {
+		case CLUTTER_F5:
+			_gth_slideshow_close (self);
+			break;
+		}
+	}
 	else if (event->type == CLUTTER_KEY_RELEASE) {
 		switch (clutter_event_get_key_symbol (event)) {
 		case CLUTTER_Escape:
+		case CLUTTER_q:
 			_gth_slideshow_close (self);
 			break;
-
-		case CLUTTER_space:
+		case CLUTTER_p:
 			_gth_slideshow_toggle_pause (self);
 			break;
 
-		case CLUTTER_Up:
+		case CLUTTER_space:
+		case CLUTTER_Down:
 		case CLUTTER_Right:
-		case CLUTTER_Page_Up:
-			_gth_slideshow_load_next_image (self);
+		case CLUTTER_Page_Down:
+			if (self->priv->paused)
+				_gth_slideshow_toggle_pause (self);
+			else
+				_gth_slideshow_load_next_image (self);
 			break;
 
-		case CLUTTER_Down:
+		case CLUTTER_BackSpace:
+		case CLUTTER_Up:
 		case CLUTTER_Left:
-		case CLUTTER_Page_Down:
+		case CLUTTER_Page_Up:
 			_gth_slideshow_load_prev_image (self);
 			break;
 		}
@@ -911,14 +1078,9 @@ stage_input_cb (ClutterStage *stage,
 
 
 static void
-gth_slideshow_size_allocate_cb (GtkWidget     *widget,
-				GtkAllocation *allocation,
-				gpointer       user_data)
+adapt_image_size_to_stage_size (GthSlideshow *self)
 {
-	GthSlideshow   *self = user_data;
 	gfloat          stage_w, stage_h;
-	GthImageLoader *image_loader;
-	GdkPixbuf      *pixbuf;
 	GdkPixbuf      *image;
 	int             pixbuf_w, pixbuf_h;
 	int             pixbuf_x, pixbuf_y;
@@ -931,28 +1093,23 @@ gth_slideshow_size_allocate_cb (GtkWidget     *widget,
 	if ((stage_w == 0) || (stage_h == 0))
 		return;
 
-	image_loader = gth_image_preloader_get_loader (self->priv->preloader, (GthFileData *) self->priv->current->data);
-	if (image_loader == NULL)
+	if (self->priv->current_pixbuf == NULL)
 		return;
 
-	pixbuf = gth_image_loader_get_pixbuf (image_loader);
-	if (pixbuf == NULL)
-		return;
-
-	image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pixbuf),
+	image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (self->priv->current_pixbuf),
 				FALSE,
-				gdk_pixbuf_get_bits_per_sample (pixbuf),
+				gdk_pixbuf_get_bits_per_sample (self->priv->current_pixbuf),
 				stage_w,
 				stage_h);
 	gdk_pixbuf_fill (image, 0x000000ff);
 
-	pixbuf_w = gdk_pixbuf_get_width (pixbuf);
-	pixbuf_h = gdk_pixbuf_get_height (pixbuf);
+	pixbuf_w = gdk_pixbuf_get_width (self->priv->current_pixbuf);
+	pixbuf_h = gdk_pixbuf_get_height (self->priv->current_pixbuf);
 	scale_keeping_ratio (&pixbuf_w, &pixbuf_h, (int) stage_w, (int) stage_h, TRUE);
 	pixbuf_x = (stage_w - pixbuf_w) / 2;
 	pixbuf_y = (stage_h - pixbuf_h) / 2;
 
-	gdk_pixbuf_composite (pixbuf,
+	gdk_pixbuf_composite (self->priv->current_pixbuf,
 			      image,
 			      pixbuf_x,
 			      pixbuf_y,
@@ -960,8 +1117,8 @@ gth_slideshow_size_allocate_cb (GtkWidget     *widget,
 			      pixbuf_h,
 			      pixbuf_x,
 			      pixbuf_y,
-			      (double) pixbuf_w / gdk_pixbuf_get_width (pixbuf),
-			      (double) pixbuf_h / gdk_pixbuf_get_height (pixbuf),
+			      (double) pixbuf_w / gdk_pixbuf_get_width (self->priv->current_pixbuf),
+			      (double) pixbuf_h / gdk_pixbuf_get_height (self->priv->current_pixbuf),
 			      GDK_INTERP_BILINEAR,
 			      255);
 
@@ -981,6 +1138,15 @@ gth_slideshow_size_allocate_cb (GtkWidget     *widget,
 }
 
 
+static void
+gth_slideshow_size_allocate_cb (GtkWidget     *widget,
+				GtkAllocation *allocation,
+				gpointer       user_data)
+{
+	adapt_image_size_to_stage_size (GTH_SLIDESHOW (user_data));
+}
+
+
 static void
 clutter_projector_construct (GthSlideshow *self)
 {
@@ -993,11 +1159,9 @@ clutter_projector_construct (GthSlideshow *self)
 	clutter_stage_set_color (CLUTTER_STAGE (self->stage), &background_color);
 	g_signal_connect (self->stage, "button-press-event", G_CALLBACK (stage_input_cb), self);
 	g_signal_connect (self->stage, "motion-event", G_CALLBACK (stage_input_cb), self);
+	g_signal_connect (self->stage, "key-press-event", G_CALLBACK (stage_input_cb), self);
 	g_signal_connect (self->stage, "key-release-event", G_CALLBACK (stage_input_cb), self);
 
-	gtk_widget_show (embed);
-	gtk_container_add (GTK_CONTAINER (self), embed);
-
 	self->priv->image1 = clutter_texture_new ();
 	clutter_actor_hide (self->priv->image1);
 	clutter_container_add_actor (CLUTTER_CONTAINER (self->stage), self->priv->image1);
@@ -1014,7 +1178,19 @@ clutter_projector_construct (GthSlideshow *self)
 	g_signal_connect (self->priv->timeline, "new-frame", G_CALLBACK (animation_frame_cb), self);
 	g_signal_connect (self->priv->timeline, "started", G_CALLBACK (animation_started_cb), self);
 
+	if (self->priv->pause_pixbuf != NULL)
+		self->priv->paused_actor = gtk_clutter_texture_new_from_pixbuf (self->priv->pause_pixbuf);
+	else
+		self->priv->paused_actor = gtk_clutter_texture_new_from_stock (GTK_WIDGET (self),
+									       GTK_STOCK_MEDIA_PAUSE,
+									       GTK_ICON_SIZE_DIALOG);
+	clutter_actor_hide (self->priv->paused_actor);
+	clutter_container_add_actor (CLUTTER_CONTAINER (self->stage), self->priv->paused_actor);
+
 	g_signal_connect (self, "size-allocate", G_CALLBACK (gth_slideshow_size_allocate_cb), self);
+
+	gtk_widget_show (embed);
+	gtk_container_add (GTK_CONTAINER (self), embed);
 }
 
 
