@@ -811,7 +811,7 @@ gfile_get_relative_path (GFile *file,
 /* construct a GFile for a GthWebExporter */
 
 
-GFile *
+static GFile *
 get_album_file (GthWebExporter *self,
 		GFile          *target_dir,
 		const char     *subdir,
@@ -824,7 +824,7 @@ get_album_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_html_index_dir (GthWebExporter *self,
 		    const int       page,
 		    GFile          *target_dir)
@@ -836,7 +836,7 @@ get_html_index_dir (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_html_image_dir (GthWebExporter *self,
 		    GFile          *target_dir)
 {
@@ -844,7 +844,7 @@ get_html_image_dir (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_theme_file (GthWebExporter *self,
 		GFile          *target_dir,
 		const char     *filename)
@@ -853,7 +853,7 @@ get_theme_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_html_index_file (GthWebExporter *self,
 		     const int       page,
 		     GFile          *target_dir)
@@ -876,7 +876,7 @@ get_html_index_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_html_image_file (GthWebExporter *self,
 		     ImageData      *image_data,
 		     GFile          *target_dir)
@@ -893,7 +893,7 @@ get_html_image_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_thumbnail_file (GthWebExporter *self,
 		    ImageData      *image_data,
 		    GFile          *target_dir)
@@ -910,7 +910,7 @@ get_thumbnail_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_image_file (GthWebExporter *self,
 		ImageData      *image_data,
 		GFile          *target_dir)
@@ -926,7 +926,7 @@ get_image_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_preview_file (GthWebExporter *self,
 		  ImageData      *image_data,
 		  GFile          *target_dir)
@@ -1945,6 +1945,7 @@ save_other_files_ready_cb (GError   *error,
 				     self->priv->target_dir,
 				     FALSE,
 				     G_FILE_COPY_NONE,
+				     GTH_OVERWRITE_RESPONSE_UNSPECIFIED,
 				     G_PRIORITY_DEFAULT,
 				     gth_task_get_cancellable (GTH_TASK (self)),
 				     save_files_progress_cb,
@@ -2019,6 +2020,7 @@ save_other_files (GthWebExporter *self)
 				     theme_dir,
 				     FALSE,
 				     G_FILE_COPY_NONE,
+				     GTH_OVERWRITE_RESPONSE_UNSPECIFIED,
 				     G_PRIORITY_DEFAULT,
 				     gth_task_get_cancellable (GTH_TASK (self)),
 				     save_files_progress_cb,
@@ -2365,6 +2367,31 @@ save_resized_image_ready_cd (GthFileData *file_data,
 }
 
 
+static const char *
+get_format_description (const char *mime_type)
+{
+	const char *description = NULL;
+	GSList     *formats;
+	GSList     *scan;
+
+	formats = gdk_pixbuf_get_formats ();
+	for (scan = formats; ! description && scan; scan = scan->next) {
+		GdkPixbufFormat  *format = scan->data;
+		char            **mime_types;
+		int               i;
+
+		mime_types = gdk_pixbuf_format_get_mime_types (format);
+		for (i = 0; ! description && mime_types[i] != NULL; i++)
+			if (g_strcmp0 (mime_types[i], mime_type) == 0)
+				description = gdk_pixbuf_format_get_description (format);
+	}
+
+	g_slist_free (formats);
+
+	return description;
+}
+
+
 static gboolean
 save_resized_image (gpointer data)
 {
@@ -2378,6 +2405,8 @@ save_resized_image (gpointer data)
 
 	image_data = self->priv->current_file->data;
 	if (self->priv->copy_images && (image_data->image != NULL)) {
+		char        *filename_no_ext;
+		char        *size;
 		GFile       *destination;
 		GthFileData *file_data;
 
@@ -2387,6 +2416,29 @@ save_resized_image (gpointer data)
 				   FALSE,
 				   (double) (self->priv->image + 1) / (self->priv->n_images + 1));
 
+		/* change the file extension to jpeg */
+
+		filename_no_ext = _g_uri_remove_extension (image_data->dest_filename);
+		g_free (image_data->dest_filename);
+		image_data->dest_filename = g_strconcat(filename_no_ext, ".jpeg", NULL);
+		g_free (filename_no_ext);
+
+		/* change the file type */
+
+		gth_file_data_set_mime_type (image_data->file_data, "image/jpeg");
+		g_file_info_set_attribute_string (image_data->file_data->info, "general::format", get_format_description ("image/jpeg"));
+
+		/* change the image dimensions info */
+
+		g_file_info_set_attribute_int32 (image_data->file_data->info, "image::width", image_data->image_width);
+		g_file_info_set_attribute_int32 (image_data->file_data->info, "image::height", image_data->image_height);
+		g_file_info_set_attribute_int32 (image_data->file_data->info, "frame::width", image_data->image_width);
+		g_file_info_set_attribute_int32 (image_data->file_data->info, "frame::height", image_data->image_height);
+		size = g_strdup_printf (_("%d × %d"), image_data->image_width, image_data->image_height);
+		g_file_info_set_attribute_string (image_data->file_data->info, "general::dimensions", size);
+
+		/* save the pixbuf */
+
 		destination = get_image_file (self, image_data, self->priv->tmp_dir);
 		file_data = gth_file_data_new (destination, NULL);
 		_gdk_pixbuf_save_async (image_data->image,
@@ -2479,81 +2531,6 @@ copy_current_file (GthWebExporter *self)
 }
 
 
-static GdkPixbuf *
-pixbuf_scale (const GdkPixbuf *src,
-	      int              dest_width,
-	      int              dest_height,
-	      GdkInterpType    interp_type)
-{
-	GdkPixbuf *dest;
-
-	if (! gdk_pixbuf_get_has_alpha (src))
-		return gdk_pixbuf_scale_simple (src, dest_width, dest_height, interp_type);
-
-	g_return_val_if_fail (src != NULL, NULL);
-	g_return_val_if_fail (dest_width > 0, NULL);
-	g_return_val_if_fail (dest_height > 0, NULL);
-
-	dest = gdk_pixbuf_new (GDK_COLORSPACE_RGB, gdk_pixbuf_get_has_alpha (src), 8, dest_width, dest_height);
-	if (dest == NULL)
-		return NULL;
-
-	gdk_pixbuf_composite_color (src,
-				    dest,
-				    0, 0, dest_width, dest_height, 0, 0,
-				    (double) dest_width / gdk_pixbuf_get_width (src),
-				    (double) dest_height / gdk_pixbuf_get_height (src),
-				    interp_type,
-				    255,
-				    0, 0,
-				    200,
-				    0xFFFFFF,
-				    0xFFFFFF);
-
-	return dest;
-}
-
-
-static GdkPixbuf *
-create_squared_thumbnail (GdkPixbuf     *p,
-			  int            size,
-			  GdkInterpType  interp_type)
-{
-	int        w, h, tw, th;
-	GdkPixbuf *p1;
-	int        x, y;
-	GdkPixbuf *p2;
-
-	w = gdk_pixbuf_get_width (p);
-	h = gdk_pixbuf_get_height (p);
-
-	if ((w < size) && (h < size))
-		return gdk_pixbuf_copy (p);
-
-	if (w > h) {
-		th = size;
-		tw = (int) (((double) w / h) * th);
-	}
-	else {
-		tw = size;
-		th = (int) (((double) h / w) * tw);
-	}
-
-	p1 = pixbuf_scale (p, tw, th, interp_type);
-
-	if ((tw == size) && (th == size))
-		return p1;
-
-	x = (tw - size) / 2;
-	y = (th - size) / 2;
-	p2 = gdk_pixbuf_new_subpixbuf (p1, x, y, size, size);
-
-	g_object_unref (p1);
-
-	return p2;
-}
-
-
 static int
 image_data_cmp (gconstpointer a,
 		gconstpointer b,
@@ -2602,7 +2579,7 @@ image_loader_ready_cb (GObject      *source_object,
 		{
 			GdkPixbuf *scaled;
 
-			scaled = pixbuf_scale (pixbuf, w, h, GDK_INTERP_BILINEAR);
+			scaled = _gdk_pixbuf_scale_composite (pixbuf, w, h, GDK_INTERP_BILINEAR);
 			g_object_unref (idata->image);
 			idata->image = scaled;
 		}
@@ -2626,7 +2603,7 @@ image_loader_ready_cb (GObject      *source_object,
 					     FALSE))
 		{
 			GdkPixbuf *scaled;
-			scaled = pixbuf_scale (pixbuf, w, h, GDK_INTERP_BILINEAR);
+			scaled = _gdk_pixbuf_scale_composite (pixbuf, w, h, GDK_INTERP_BILINEAR);
 			g_object_unref (idata->preview);
 			idata->preview = scaled;
 		}
@@ -2655,7 +2632,7 @@ image_loader_ready_cb (GObject      *source_object,
 		if (self->priv->squared_thumbnails) {
 			GdkPixbuf *squared;
 
-			squared = create_squared_thumbnail (idata->thumb, self->priv->thumb_width, GDK_INTERP_BILINEAR);
+			squared = _gdk_pixbuf_scale_squared (idata->thumb, self->priv->thumb_width, GDK_INTERP_BILINEAR);
 			g_object_unref (idata->thumb);
 			idata->thumb = squared;
 		}
@@ -2666,34 +2643,9 @@ image_loader_ready_cb (GObject      *source_object,
 		{
 			GdkPixbuf *scaled;
 
-			scaled = pixbuf_scale (pixbuf, w, h, GDK_INTERP_BILINEAR);
+			scaled = _gdk_pixbuf_scale_composite (pixbuf, w, h, GDK_INTERP_BILINEAR);
 			g_object_unref (idata->thumb);
 			idata->thumb = scaled;
-
-			if (self->priv->squared_thumbnails) {
-				GdkPixbuf *squared;
-				int        src_x;
-				int        src_y;
-
-				src_x = (self->priv->thumb_width - gdk_pixbuf_get_width (idata->thumb)) / 2;
-				src_y = (self->priv->thumb_height - gdk_pixbuf_get_height (idata->thumb)) / 2;
-
-				g_print ("[%d, %d] => (%d, %d)[%d, %d]",
-						gdk_pixbuf_get_width (idata->thumb),
-						gdk_pixbuf_get_height (idata->thumb),
-						src_x,
-						src_y,
-						self->priv->thumb_width,
-						self->priv->thumb_height);
-
-				squared = gdk_pixbuf_new_subpixbuf (idata->thumb,
-								    src_x,
-								    src_y,
-								    self->priv->thumb_width,
-								    self->priv->thumb_height);
-				g_object_unref (idata->thumb);
-				idata->thumb = squared;
-			}
 		}
 	}
 
