@@ -16,10 +16,10 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -103,6 +103,26 @@ g_object_list_get_type (void)
 }
 
 
+#if NEED_G_TYPE_ERROR
+
+
+GType
+g_error_get_type (void)
+{
+	static GType type = 0;
+
+	if (type == 0)
+		type = g_boxed_type_register_static ("GError",
+						     (GBoxedCopyFunc) g_error_copy,
+						     (GBoxedFreeFunc) g_error_free);
+
+	return type;
+}
+
+
+#endif
+
+
 GEnumValue *
 _g_enum_type_get_value (GType enum_type,
 			int   value)
@@ -535,6 +555,9 @@ _g_time_val_strftime (GTimeVal   *time_,
 	struct tm *tm;
 
 	if ((format == NULL) || (*format == '\0'))
+		format = "%Y-%m-%d--%H.%M.%S";
+
+	if (strcmp (format, "%q") == 0)
 		format = "%Y-%m-%d";
 
 	secs = time_->tv_sec;
@@ -1329,145 +1352,6 @@ get_home_uri (void)
 }
 
 
-const char *
-get_desktop_uri (void)
-{
-	static char *desktop_uri = NULL;
-
-	if (desktop_uri == NULL) {
-		char *path;
-		char *uri;
-
-		path = xdg_user_dir_lookup ("DESKTOP");
-		uri = g_uri_escape_string (path, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, TRUE);
-
-		desktop_uri = g_strconcat ("file://", uri, NULL);
-
-		g_free (uri);
-		g_free (path);
-	}
-
-	return desktop_uri;
-}
-
-
-char *
-xdg_user_dir_lookup (const char *type)
-{
-	/* This function is used by gthumb to determine the default "PICTURES"
-	   directory for imports. The actually directory name is localized.
-	   Bug 425365. */
-
-	FILE *file;
-	char *home_dir, *config_home, *config_file;
-	char buffer[512];
-	char *user_dir;
-	char *p, *d;
-	int len;
-	int relative;
-
-	home_dir = getenv ("HOME");
-	if (home_dir == NULL)
-		return strdup ("/tmp");
-
-	config_home = getenv ("XDG_CONFIG_HOME");
-	if (config_home == NULL || config_home[0] == 0) {
-		config_file = malloc (strlen (home_dir) + strlen ("/.config/user-dirs.dirs") + 1);
-		strcpy (config_file, home_dir);
-		strcat (config_file, "/.config/user-dirs.dirs");
-	}
-	else {
-		config_file = malloc (strlen (config_home) + strlen ("/user-dirs.dirs") + 1);
-		strcpy (config_file, config_home);
-		strcat (config_file, "/user-dirs.dirs");
-	}
-
-	file = fopen (config_file, "r");
-	free (config_file);
-	if (file == NULL)
-		goto error;
-
-	user_dir = NULL;
-
-	while (fgets (buffer, sizeof (buffer), file)) {
-		/* Remove newline at end */
-		len = strlen (buffer);
-		if (len > 0 && buffer[len-1] == '\n')
-		buffer[len-1] = 0;
-
-		p = buffer;
-		while (*p == ' ' || *p == '\t')
-			p++;
-
-		if (strncmp (p, "XDG_", 4) != 0)
-			continue;
-		p += 4;
-
-		if (strncmp (p, type, strlen (type)) != 0)
-			continue;
-		p += strlen (type);
-
-		if (strncmp (p, "_DIR", 4) != 0)
-		continue;
-		p += 4;
-
-		while (*p == ' ' || *p == '\t')
-			p++;
-
-		if (*p != '=')
-			continue;
-		p++;
-
-		while (*p == ' ' || *p == '\t')
-			p++;
-
-		if (*p != '"')
-			continue;
-		p++;
-
-		relative = 0;
-		if (strncmp (p, "$HOME/", 6) == 0) {
-			p += 6;
-			relative = 1;
-		} else if (*p != '/')
-			continue;
-
-		if (relative) {
-			user_dir = malloc (strlen (home_dir) + 1 + strlen (p) + 1);
-			strcpy (user_dir, home_dir);
-			strcat (user_dir, "/");
-		} else {
-			user_dir = malloc (strlen (p) + 1);
-			*user_dir = 0;
-		}
-
-		d = user_dir + strlen (user_dir);
-		while (*p && *p != '"')	{
-			if ((*p == '\\') && (*(p+1) != 0))
-				p++;
-			*d++ = *p++;
-		}
-		*d = 0;
-	}
-
-	fclose (file);
-
-	if (user_dir)
-		return user_dir;
-
-error:
-	 /* Special case desktop for historical compatibility */
-	if (strcmp (type, "DESKTOP") == 0) {
-		user_dir = malloc (strlen (home_dir) + strlen ("/Desktop") + 1);
-		strcpy (user_dir, home_dir);
-		strcat (user_dir, "/Desktop");
-		return user_dir;
-	}
-	else
-		return strdup (home_dir);
-}
-
-
 int
 uricmp (const char *uri1,
 	const char *uri2)
@@ -2261,7 +2145,26 @@ int
 _g_file_cmp_uris (GFile *a,
 		  GFile *b)
 {
-	return g_file_equal (a, b) ? 0 : 1;
+	char *uri_a;
+	char *uri_b;
+	int   result;
+
+	uri_a = g_file_get_uri (a);
+	uri_b = g_file_get_uri (b);
+	result = g_strcmp0 (uri_a, uri_b);
+
+	g_free (uri_b);
+	g_free (uri_a);
+
+	return result;
+}
+
+
+gboolean
+_g_file_equal_uris (GFile *a,
+		    GFile *b)
+{
+	return _g_file_cmp_uris (a, b) == 0;
 }
 
 
