@@ -29,6 +29,8 @@
 
 G_BEGIN_DECLS
 
+#define SQR(x) ((x) * (x))
+
 #define GFILE_NAME_TYPE_ATTRIBUTES "standard::name,standard::type"
 #define GFILE_DISPLAY_ATTRIBUTES "standard::display-name,standard::icon"
 #define GFILE_BASIC_ATTRIBUTES GFILE_DISPLAY_ATTRIBUTES ",standard::name,standard::type"
@@ -60,6 +62,10 @@ G_BEGIN_DECLS
 #define ID_LENGTH 8
 #define G_TYPE_OBJECT_LIST (g_object_list_get_type ())
 #define G_TYPE_STRING_LIST (g_string_list_get_type ())
+#ifndef G_TYPE_ERROR
+#define NEED_G_TYPE_ERROR 1
+#define G_TYPE_ERROR (g_error_get_type ())
+#endif
 
 /* signals */
 
@@ -81,6 +87,7 @@ void          _g_clear_object                (gpointer     object_p);
 GList *       _g_object_list_ref             (GList       *list);
 void          _g_object_list_unref           (GList       *list);
 GType         g_object_list_get_type         (void);
+GType         g_error_get_type               (void);
 GEnumValue *  _g_enum_type_get_value         (GType        enum_type,
 					      int          value);
 GEnumValue *  _g_enum_type_get_value_by_nick (GType        enum_type,
@@ -204,8 +211,6 @@ void            free_regexps                     (GRegex     **regexps);
 /* URI utils */
 
 const char *    get_home_uri                     (void);
-const char *    get_desktop_uri                  (void);
-char *          xdg_user_dir_lookup              (const char *type);
 int             uricmp                           (const char *uri1,
 						  const char *uri2);
 gboolean        same_uri                         (const char *uri1,
