@@ -23,26 +23,26 @@
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include "gconf-utils.h"
 #include "glib-utils.h"
 #include "gth-cell-renderer-thumbnail.h"
 #include "gth-dumb-notebook.h"
 #include "gth-empty-list.h"
 #include "gth-file-list.h"
+#include "gth-file-selection.h"
 #include "gth-file-store.h"
 #include "gth-icon-cache.h"
-#include "gth-icon-view.h"
+#include "gth-preferences.h"
 #include "gth-thumb-loader.h"
 #include "gtk-utils.h"
 
 
-#define DEFAULT_THUMBNAIL_SIZE 112
 #define FLASH_THUMBNAIL_QUEUE_TIMEOUT 75
 #define UPDATE_THUMBNAILS_AFTER_SCROLL_TIMEOUT 125
 #define RESTART_LOADING_THUMBS_DELAY 1500
 #define N_VIEWAHEAD 50
 #define N_CREATEAHEAD 50000
 #define EMPTY (N_("(Empty)"))
-#define THUMBNAIL_BORDER (8 * 2)
 #define CHECK_JOBS_INTERVAL 50
 
 
@@ -98,17 +98,12 @@ typedef struct {
 } ThumbnailJob;
 
 
-enum {
-	FILE_POPUP,
-	LAST_SIGNAL
-};
-
-
 enum {
 	GTH_FILE_LIST_PANE_VIEW,
 	GTH_FILE_LIST_PANE_MESSAGE
 };
 
+
 struct _GthFileListPrivateData
 {
 	GthFileListType  type;
@@ -572,7 +567,7 @@ _gth_file_list_set_type (GthFileList     *file_list,
 	 * a time. */
 
 	g_object_set (file_list->priv->thumbnail_renderer,
-		      "fixed_size", (file_list->priv->type == GTH_FILE_LIST_TYPE_H_SIDEBAR),
+		      "fixed_size", TRUE /* (file_list->priv->type == GTH_FILE_LIST_TYPE_H_SIDEBAR) */,
 		      NULL);
 
 	_gth_file_list_update_orientation (file_list);
@@ -587,14 +582,28 @@ file_store_visibility_changed_cb (GthFileStore *file_store,
 }
 
 
+static void
+file_store_rows_reordered_cb (GtkTreeModel *tree_model,
+			      GtkTreePath  *path,
+			      GtkTreeIter  *iter,
+			      gpointer      new_order,
+			      gpointer      user_data)
+{
+	GthFileList *file_list = user_data;
+	file_list->priv->visibility_changed = TRUE;
+}
+
+
 static void
 gth_file_list_construct (GthFileList     *file_list,
+			 GtkWidget       *file_view,
 			 GthFileListType  list_type,
 			 gboolean         enable_drag_drop)
 {
 	GtkWidget       *viewport;
 	GtkCellRenderer *renderer;
 	GthFileStore    *model;
+	GtkCellLayout   *cell_layout;
 
 	file_list->priv->thumb_loader = gth_thumb_loader_new (file_list->priv->thumb_size);
 	file_list->priv->icon_cache = gth_icon_cache_new (gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (file_list))), file_list->priv->thumb_size / 2);
@@ -630,14 +639,19 @@ gth_file_list_construct (GthFileList     *file_list,
 			  G_CALLBACK (vadj_changed_cb),
 			  file_list);
 
+	file_list->priv->view = file_view;
 	model = gth_file_store_new ();
-	file_list->priv->view = gth_icon_view_new_with_model (GTK_TREE_MODEL (model));
+	gth_file_view_set_model (GTH_FILE_VIEW (file_list->priv->view), GTK_TREE_MODEL (model));
 	g_object_unref (model);
 
 	g_signal_connect (model,
 			  "visibility-changed",
 			  G_CALLBACK (file_store_visibility_changed_cb),
 			  file_list);
+	g_signal_connect (model,
+			  "rows-reordered",
+			  G_CALLBACK (file_store_rows_reordered_cb),
+			  file_list);
 
 	if (enable_drag_drop) {
 		GtkTargetList  *target_list;
@@ -649,10 +663,10 @@ gth_file_list_construct (GthFileList     *file_list,
 		gtk_target_list_add_text_targets (target_list, 0);
 		targets = gtk_target_table_new_from_list (target_list, &n_targets);
 		gth_file_view_enable_drag_source (GTH_FILE_VIEW (file_list->priv->view),
-						  GDK_BUTTON1_MASK,
+						  GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
 						  targets,
 						  n_targets,
-						  GDK_ACTION_MOVE | GDK_ACTION_COPY);
+						  GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK);
 
 		gtk_target_list_unref (target_list);
 		gtk_target_table_free (targets, n_targets);
@@ -666,39 +680,39 @@ gth_file_list_construct (GthFileList     *file_list,
 	/* checkbox */
 
 	file_list->priv->checkbox_renderer = renderer = gtk_cell_renderer_toggle_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (file_list->priv->view), renderer, FALSE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (file_list->priv->view),
-					renderer,
-					"active", GTH_FILE_STORE_CHECKED_COLUMN,
-					NULL);
 	g_signal_connect (file_list->priv->checkbox_renderer,
 			  "toggled",
 			  G_CALLBACK (checkbox_toggled_cb),
 			  file_list);
 
+	cell_layout = gth_file_view_add_renderer (GTH_FILE_VIEW (file_list->priv->view),
+						  GTH_FILE_VIEW_RENDERER_CHECKBOX,
+						  file_list->priv->checkbox_renderer);
+	gtk_cell_layout_set_attributes (cell_layout,
+					renderer,
+					"active", GTH_FILE_STORE_CHECKED_COLUMN,
+					NULL);
+
 	/* thumbnail */
 
 	file_list->priv->thumbnail_renderer = renderer = gth_cell_renderer_thumbnail_new ();
-	g_object_set (renderer,
-		      "size", file_list->priv->thumb_size,
-		      "yalign", 1.0,
-		      NULL);
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (file_list->priv->view), renderer, FALSE);
 
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (file_list->priv->view),
+	cell_layout = gth_file_view_add_renderer (GTH_FILE_VIEW (file_list->priv->view),
+						  GTH_FILE_VIEW_RENDERER_THUMBNAIL,
+						  file_list->priv->thumbnail_renderer);
+	gtk_cell_layout_set_attributes (cell_layout,
 					renderer,
 					"thumbnail", GTH_FILE_STORE_THUMBNAIL_COLUMN,
 					"is_icon", GTH_FILE_STORE_IS_ICON_COLUMN,
 					"file", GTH_FILE_STORE_FILE_DATA_COLUMN,
 					NULL);
-
 	if (file_list->priv->type == GTH_FILE_LIST_TYPE_BROWSER)
-		gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (file_list->priv->view),
+		gtk_cell_layout_add_attribute (cell_layout,
 					       renderer,
 					       "selected",
 					       GTH_FILE_STORE_CHECKED_COLUMN);
 	else if (file_list->priv->type == GTH_FILE_LIST_TYPE_SELECTOR)
-		gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (file_list->priv->view),
+		gtk_cell_layout_add_attribute (cell_layout,
 					       renderer,
 					       "checked",
 					       GTH_FILE_STORE_CHECKED_COLUMN);
@@ -706,20 +720,22 @@ gth_file_list_construct (GthFileList     *file_list,
 	/* text */
 
 	file_list->priv->text_renderer = renderer = gtk_cell_renderer_text_new ();
-	g_object_set (G_OBJECT (renderer),
-		      /*"ellipsize", PANGO_ELLIPSIZE_NONE,*/
-		      "alignment", PANGO_ALIGN_CENTER,
-		      "width", file_list->priv->thumb_size + THUMBNAIL_BORDER,
-		      "wrap-mode", PANGO_WRAP_WORD_CHAR,
-		      "wrap-width", file_list->priv->thumb_size + THUMBNAIL_BORDER,
-		      NULL);
 
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (file_list->priv->view), renderer, FALSE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (file_list->priv->view),
+	cell_layout = gth_file_view_add_renderer (GTH_FILE_VIEW (file_list->priv->view),
+						  GTH_FILE_VIEW_RENDERER_TEXT,
+						  file_list->priv->text_renderer);
+	gtk_cell_layout_set_attributes (cell_layout,
 					renderer,
 					"text", GTH_FILE_STORE_METADATA_COLUMN,
+					"visible", GTH_FILE_STORE_METADATA_VISIBLE_COLUMN,
 					NULL);
 
+	gth_file_view_update_attributes (GTH_FILE_VIEW (file_list->priv->view),
+					 file_list->priv->checkbox_renderer,
+					 file_list->priv->thumbnail_renderer,
+					 file_list->priv->text_renderer,
+					 file_list->priv->thumb_size);
+
 	_gth_file_list_set_type (file_list, list_type);
 
 	/* pack the widgets together */
@@ -772,13 +788,14 @@ gth_file_list_get_type (void)
 
 
 GtkWidget*
-gth_file_list_new (GthFileListType list_type,
-		   gboolean        enable_drag_drop)
+gth_file_list_new (GtkWidget       *file_view,
+		   GthFileListType  list_type,
+		   gboolean         enable_drag_drop)
 {
 	GtkWidget *widget;
 
 	widget = GTK_WIDGET (g_object_new (GTH_TYPE_FILE_LIST, NULL));
-	gth_file_list_construct (GTH_FILE_LIST (widget), list_type, enable_drag_drop);
+	gth_file_list_construct (GTH_FILE_LIST (widget), file_view, list_type, enable_drag_drop);
 
 	return widget;
 }
@@ -927,6 +944,9 @@ _gth_file_list_update_pane (GthFileList *file_list)
 }
 
 
+#define MAX_TEXT_LENGTH 70
+
+
 static GString *
 _gth_file_list_get_metadata (GthFileList *file_list,
 			     GthFileData *file_data)
@@ -934,16 +954,28 @@ _gth_file_list_get_metadata (GthFileList *file_list,
 	GString *metadata;
 	int      i;
 
-	metadata = g_string_new ("");
+	metadata = g_string_new (NULL);
 	for (i = 0; file_list->priv->caption_attributes_v[i] != NULL; i++) {
 		char *value;
 
 		value = gth_file_data_get_attribute_as_string (file_data, file_list->priv->caption_attributes_v[i]);
-		if (value == NULL)
-			value = g_strdup ("-");
-		if (metadata->len > 0)
-			g_string_append (metadata, "\n");
-		g_string_append (metadata, value);
+		if (value != NULL) {
+			if (metadata->len > 0)
+				g_string_append (metadata, "\n");
+			if (gth_file_view_truncate_metadata (GTH_FILE_VIEW (file_list->priv->view))
+			    && g_utf8_strlen (value, -1) > MAX_TEXT_LENGTH)
+			{
+				char *tmp;
+
+				tmp = g_strdup (value);
+				g_utf8_strncpy (tmp, value, MAX_TEXT_LENGTH);
+				g_free (value);
+				value = g_strdup_printf ("%s […]", tmp);
+
+				g_free (tmp);
+			}
+			g_string_append (metadata, value);
+		}
 
 		g_free (value);
 	}
@@ -958,11 +990,15 @@ gfl_add_files (GthFileList *file_list,
 {
 	GthFileStore *file_store;
 	GList        *scan;
+	char         *cache_base_uri;
 
 	file_store = (GthFileStore*) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view));
 
+	cache_base_uri = g_strconcat (get_home_uri (), "/.thumbnails", NULL);
 	for (scan = files; scan; scan = scan->next) {
 		GthFileData *file_data = scan->data;
+		char        *uri;
+		ThumbData   *thumb_data;
 		GIcon       *icon;
 		GdkPixbuf   *pixbuf = NULL;
 		GString     *metadata;
@@ -973,9 +1009,16 @@ gfl_add_files (GthFileList *file_list,
 		if (g_hash_table_lookup (file_list->priv->thumb_data, file_data->file) != NULL)
 			continue;
 
+		uri = g_file_get_uri (file_data->file);
+
+		thumb_data = thumb_data_new ();
+		/* files in the .thumbnails directory are already thumbnails,
+		 * set them as created. */
+		thumb_data->thumb_created = _g_uri_parent_of_uri (cache_base_uri, uri);
+
 		g_hash_table_insert (file_list->priv->thumb_data,
 				     g_object_ref (file_data->file),
-				     thumb_data_new ());
+				     thumb_data);
 
 		icon = g_file_info_get_icon (file_data->info);
 		pixbuf = gth_icon_cache_get_pixbuf (file_list->priv->icon_cache, icon);
@@ -990,7 +1033,10 @@ gfl_add_files (GthFileList *file_list,
 		g_string_free (metadata, TRUE);
 		if (pixbuf != NULL)
 			g_object_unref (pixbuf);
+
+		g_free (uri);
 	}
+	g_free (cache_base_uri);
 
 	gth_file_store_exec_add (file_store);
 	_gth_file_list_update_pane (file_list);
@@ -1017,6 +1063,7 @@ gfl_delete_files (GthFileList *file_list,
 	GList        *scan;
 
 	file_store = (GthFileStore*) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view));
+
 	for (scan = files; scan; scan = scan->next) {
 		GFile       *file = scan->data;
 		GtkTreeIter  iter;
@@ -1149,6 +1196,8 @@ static void
 gfl_set_files (GthFileList *file_list,
 	       GList       *files)
 {
+	gth_thumb_loader_set_save_thumbnails (file_list->priv->thumb_loader, eel_gconf_get_boolean (PREF_SAVE_THUMBNAILS, TRUE));
+	gth_thumb_loader_set_max_file_size (file_list->priv->thumb_loader, eel_gconf_get_integer (PREF_THUMBNAIL_LIMIT, 0));
 	gth_file_selection_unselect_all (GTH_FILE_SELECTION (file_list->priv->view));
 
 	gth_file_store_clear ((GthFileStore*) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view)));
@@ -1263,6 +1312,9 @@ gfl_enable_thumbs (GthFileList *file_list,
 	GthFileStore *file_store;
 	GtkTreeIter   iter;
 
+	gth_thumb_loader_set_save_thumbnails (file_list->priv->thumb_loader, eel_gconf_get_boolean (PREF_SAVE_THUMBNAILS, TRUE));
+	gth_thumb_loader_set_max_file_size (file_list->priv->thumb_loader, eel_gconf_get_integer (PREF_THUMBNAIL_LIMIT, 0));
+
 	file_list->priv->load_thumbs = enable;
 
 	file_store = (GthFileStore*) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view));
@@ -1323,18 +1375,19 @@ gth_file_list_set_thumb_size (GthFileList *file_list,
 			      int          size)
 {
 	file_list->priv->thumb_size = size;
+
 	gth_thumb_loader_set_requested_size (file_list->priv->thumb_loader, size);
+	gth_thumb_loader_set_save_thumbnails (file_list->priv->thumb_loader, eel_gconf_get_boolean (PREF_SAVE_THUMBNAILS, TRUE));
+	gth_thumb_loader_set_max_file_size (file_list->priv->thumb_loader, eel_gconf_get_integer (PREF_THUMBNAIL_LIMIT, 0));
 
 	gth_icon_cache_free (file_list->priv->icon_cache);
 	file_list->priv->icon_cache = gth_icon_cache_new (gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (file_list))), size / 2);
 
-	g_object_set (file_list->priv->thumbnail_renderer,
-		      "size", file_list->priv->thumb_size,
-		      NULL);
-	g_object_set (file_list->priv->text_renderer,
-		      "width", file_list->priv->thumb_size + THUMBNAIL_BORDER,
-		      "wrap-width", file_list->priv->thumb_size + THUMBNAIL_BORDER,
-		      NULL);
+	gth_file_view_update_attributes (GTH_FILE_VIEW (file_list->priv->view),
+					 file_list->priv->checkbox_renderer,
+					 file_list->priv->thumbnail_renderer,
+					 file_list->priv->text_renderer,
+					 file_list->priv->thumb_size);
 
 	_gth_file_list_update_orientation (file_list);
 }
@@ -1822,10 +1875,9 @@ _gth_file_list_update_next_thumb (GthFileList *file_list)
 				file_data = scan->data;
 				thumb_data = g_hash_table_lookup (file_list->priv->thumb_data, file_data->file);
 
-				/* disabled to avoid thumbnail jumps (bug #603642)
 				if (pos >= first_pos - N_VIEWAHEAD)
 					requested_action_performed = thumb_data->thumb_loaded;
-				else*/
+				else
 					requested_action_performed = thumb_data->thumb_created;
 
 				if (! requested_action_performed && can_create_file_thumbnail (file_data, thumb_data, &current_time, &young_file_found)) {
@@ -1853,11 +1905,7 @@ _gth_file_list_update_next_thumb (GthFileList *file_list)
 	job->cancellable = g_cancellable_new ();
 	job->file_data = file_data; /* already ref-ed above */
 	job->pos = new_pos;
-
-	/* disabled to avoid thumbnail jumps (bug #603642)
-	 job->update_in_view = (new_pos >= (first_pos - N_VIEWAHEAD)) && (new_pos <= (last_pos + N_VIEWAHEAD)); */
-
-	job->update_in_view = (new_pos >= first_pos) && (job->pos <= last_pos + N_VIEWAHEAD);
+	job->update_in_view = (new_pos >= (first_pos - N_VIEWAHEAD)) && (new_pos <= (last_pos + N_VIEWAHEAD));
 
 #if 0
 	g_print ("%d in [%d, %d] => %d\n",
