@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -32,7 +31,7 @@
 
 #define GTH_MONITOR_N_EVENTS 3
 #define MONITOR_UPDATE_DELAY 500
-#undef DEBUG_MONITOR
+#undef  DEBUG_MONITOR
 
 struct _GthFileSourceVfsPrivate
 {
@@ -42,10 +41,12 @@ struct _GthFileSourceVfsPrivate
 	ForEachChildCallback  for_each_file_func;
 	ReadyCallback         ready_func;
 	gpointer              user_data;
+	GHashTable           *hidden_files;
 	GHashTable           *monitors;
 	GList                *monitor_queue[GTH_MONITOR_N_EVENTS];
 	guint                 monitor_update_id;
 	GVolumeMonitor       *mount_monitor;
+	gboolean              check_hidden_files;
 };
 
 
@@ -93,37 +94,48 @@ gth_file_source_vfs_get_entry_points (GthFileSource *file_source)
 	mounts = g_volume_monitor_get_mounts (g_volume_monitor_get ());
 	for (scan = mounts; scan; scan = scan->next) {
 		GMount  *mount = scan->data;
-		GVolume *volume;
+		GIcon   *icon;
+		char    *name;
+		GDrive  *drive;
 
 		if (g_mount_is_shadowed (mount))
 			continue;
 
 		file = g_mount_get_root (mount);
+
+		if (gth_file_data_list_find_file (list, file) != NULL) {
+			g_object_unref (file);
+			continue;
+		}
+
 		info = g_file_query_info (file, GFILE_BASIC_ATTRIBUTES ",access::*", G_FILE_QUERY_INFO_NONE, NULL, NULL);
 
-		volume = g_mount_get_volume (mount);
-		if (volume != NULL) {
-			char  *name;
-			GIcon *icon;
+		icon = g_mount_get_icon (mount);
+		name = g_mount_get_name (mount);
 
-			name = g_volume_get_name (volume);
-			g_file_info_set_display_name (info, name);
+		drive = g_mount_get_drive (mount);
+		if (drive != NULL) {
+			char *drive_name;
+			char *tmp;
 
-			icon = g_mount_get_icon (mount);
-			g_file_info_set_icon (info, icon);
-
-			g_object_unref (icon);
+			drive_name = g_drive_get_name (drive);
+			tmp = g_strconcat (drive_name, ": ", name, NULL);
 			g_free (name);
-			g_object_unref (volume);
+			g_object_unref (drive);
+			name = tmp;
+
+			g_free (drive_name);
 		}
 
+		g_file_info_set_icon (info, icon);
+		g_file_info_set_display_name (info, name);
+
 		list = g_list_append (list, gth_file_data_new (file, info));
 		g_object_unref (info);
 		g_object_unref (file);
 	}
 
-	g_list_foreach (mounts, (GFunc) g_object_unref, NULL);
-	g_list_free (mounts);
+	_g_object_list_unref (mounts);
 
 	return list;
 }
@@ -187,6 +199,9 @@ fec__for_each_file_func (GFile       *file,
 		         gpointer     user_data)
 {
 	GthFileSourceVfs *file_source_vfs = user_data;
+
+	if (g_hash_table_lookup (file_source_vfs->priv->hidden_files, g_file_info_get_name (info)) != NULL)
+		g_file_info_set_is_hidden (info, TRUE);
 	file_source_vfs->priv->for_each_file_func (file, info, file_source_vfs->priv->user_data);
 }
 
@@ -198,8 +213,35 @@ fec__start_dir_func (GFile       *directory,
 		     gpointer     user_data)
 {
 	GthFileSourceVfs *file_source_vfs = user_data;
+	DirOp             op;
 
-	return file_source_vfs->priv->start_dir_func (directory, info, error, file_source_vfs->priv->user_data);
+	op = file_source_vfs->priv->start_dir_func (directory, info, error, file_source_vfs->priv->user_data);
+
+	if ((op == DIR_OP_CONTINUE) && file_source_vfs->priv->check_hidden_files) {
+		GFile            *dot_hidden_file;
+		GFileInputStream *input_stream;
+
+		g_hash_table_remove_all (file_source_vfs->priv->hidden_files);
+
+		dot_hidden_file = g_file_get_child (directory, ".hidden");
+		input_stream = g_file_read (dot_hidden_file, NULL, NULL);
+		if (input_stream != NULL) {
+			GDataInputStream *data_stream;
+			char             *line;
+
+			data_stream = g_data_input_stream_new (G_INPUT_STREAM (input_stream));
+			while ((line = g_data_input_stream_read_line (data_stream, NULL, NULL, NULL)) != NULL)
+				g_hash_table_insert (file_source_vfs->priv->hidden_files,
+						     line,
+						     GINT_TO_POINTER (1));
+
+			g_object_unref (input_stream);
+		}
+
+		g_object_unref (dot_hidden_file);
+	}
+
+	return op;
 }
 
 
@@ -218,11 +260,13 @@ gth_file_source_vfs_for_each_child (GthFileSource        *file_source,
 
 	gth_file_source_set_active (file_source, TRUE);
 	g_cancellable_reset (gth_file_source_get_cancellable (file_source));
+	g_hash_table_remove_all (file_source_vfs->priv->hidden_files);
 
 	file_source_vfs->priv->start_dir_func = start_dir_func;
 	file_source_vfs->priv->for_each_file_func = for_each_file_func;
 	file_source_vfs->priv->ready_func = ready_func;
 	file_source_vfs->priv->user_data = user_data;
+	file_source_vfs->priv->check_hidden_files = _g_file_attributes_matches_any (attributes, G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN);
 
 	gio_folder = gth_file_source_to_gio_file (file_source, parent);
 	g_directory_foreach_child (gio_folder,
@@ -283,6 +327,7 @@ gth_file_source_vfs_copy (GthFileSource    *file_source,
 			     destination->file,
 			     move,
 			     G_FILE_COPY_ALL_METADATA,
+			     GTH_OVERWRITE_RESPONSE_UNSPECIFIED,
 			     G_PRIORITY_DEFAULT,
 			     gth_file_source_get_cancellable (file_source),
 			     progress_callback,
@@ -295,7 +340,8 @@ gth_file_source_vfs_copy (GthFileSource    *file_source,
 
 
 static gboolean
-gth_file_source_vfs_can_cut (GthFileSource *file_source)
+gth_file_source_vfs_can_cut (GthFileSource *file_source,
+			     GFile         *file)
 {
 	return TRUE;
 }
@@ -552,6 +598,7 @@ gth_file_source_vfs_finalize (GObject *object)
 			file_source_vfs->priv->monitor_update_id = 0;
 		}
 
+		g_hash_table_destroy (file_source_vfs->priv->hidden_files);
 		g_hash_table_destroy (file_source_vfs->priv->monitors);
 
 		for (i = 0; i < GTH_MONITOR_N_EVENTS; i++) {
@@ -597,6 +644,7 @@ gth_file_source_vfs_init (GthFileSourceVfs *file_source)
 
 	file_source->priv = g_new0 (GthFileSourceVfsPrivate, 1);
 	gth_file_source_add_scheme (GTH_FILE_SOURCE (file_source), "vfs+");
+	file_source->priv->hidden_files = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	file_source->priv->monitors = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, g_object_unref, g_object_unref);
 	for (i = 0; i < GTH_MONITOR_N_EVENTS; i++)
 		file_source->priv->monitor_queue[i] = NULL;
