@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <math.h>
 #include "glib-utils.h"
-#include "gth-cursors.h"
 #include "gth-image-selector.h"
 #include "gth-marshal.h"
 
@@ -592,9 +591,6 @@ paint_background (GthImageSelector *self,
 		int           n_rects;
 		int           i;
 
-		cairo_save (cr);
-		gdk_cairo_region (cr, event->region);
-		cairo_clip (cr);
 		gdk_cairo_rectangle (cr, &self->priv->viewer->image_area);
 		cairo_clip (cr);
 		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.5);
@@ -606,7 +602,6 @@ paint_background (GthImageSelector *self,
 				cairo_rectangle (cr, paint_area.x, paint_area.y, paint_area.width, paint_area.height);
 		}
 		cairo_fill (cr);
-		cairo_restore (cr);
 
 		g_free (rects);
 	}
@@ -641,8 +636,6 @@ paint_selection (GthImageSelector *self,
 	cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
 #endif
 
-
-	gdk_cairo_region (cr, event->region);
 	gdk_cairo_rectangle (cr, &selection_area);
 	cairo_clip (cr);
 
@@ -659,12 +652,16 @@ paint_selection (GthImageSelector *self,
 
 		if (self->priv->grid_type == GTH_GRID_THIRDS) {
 			x_delta = selection_area.width / 3;
-			y_delta = selection_area.height /3;
+			y_delta = selection_area.height / 3;
 		}
 		else if (self->priv->grid_type == GTH_GRID_GOLDEN) {
 			x_delta = selection_area.width * GOLDER_RATIO_FACTOR;
 			y_delta = selection_area.height * GOLDER_RATIO_FACTOR;
 		}
+		else if (self->priv->grid_type == GTH_GRID_CENTER) {
+			x_delta = selection_area.width / 2;
+			y_delta = selection_area.height / 2;
+		}
 
 		grid_x1 = grid_x0 + x_delta;
 		grid_x2 = grid_x3 - x_delta;
@@ -674,21 +671,25 @@ paint_selection (GthImageSelector *self,
 		cairo_move_to (cr, grid_x1 + 0.5, grid_y0 + 0.5);
 		cairo_line_to (cr, grid_x1 + 0.5, grid_y3 + 0.5);
 
-		cairo_move_to (cr, grid_x2 + 0.5, grid_y0 + 0.5);
-		cairo_line_to (cr, grid_x2 + 0.5, grid_y3 + 0.5);
+		if (x_delta < selection_area.width / 2) {
+			cairo_move_to (cr, grid_x2 + 0.5, grid_y0 + 0.5);
+			cairo_line_to (cr, grid_x2 + 0.5, grid_y3 + 0.5);
+		}
 
 		cairo_move_to (cr, grid_x0 + 0.5, grid_y1 + 0.5);
 		cairo_line_to (cr, grid_x3 + 0.5, grid_y1 + 0.5);
 
-		cairo_move_to (cr, grid_x0 + 0.5, grid_y2 + 0.5);
-		cairo_line_to (cr, grid_x3 + 0.5, grid_y2 + 0.5);
+		if (y_delta < selection_area.height / 2) {
+			cairo_move_to (cr, grid_x0 + 0.5, grid_y2 + 0.5);
+			cairo_line_to (cr, grid_x3 + 0.5, grid_y2 + 0.5);
+		}
 	}
 
 	cairo_rectangle (cr,
 			 selection_area.x + 0.5,
 			 selection_area.y + 0.5,
-			 selection_area.width,
-			 selection_area.height);
+			 selection_area.width - 1,
+			 selection_area.height - 1);
 	cairo_stroke (cr);
 
 	cairo_restore (cr);
@@ -794,6 +795,7 @@ gth_image_selector_button_release (GthImageViewerTool *base,
 	update_cursor (self,
 		       event->x + self->priv->viewer->x_offset,
 		       event->y + self->priv->viewer->y_offset);
+	gtk_widget_queue_draw (GTK_WIDGET (self->priv->viewer));
 
 	return FALSE;
 }
@@ -966,6 +968,7 @@ gth_image_selector_button_press (GthImageViewerTool *base,
 		self->priv->viewer->pressed = TRUE;
 		self->priv->viewer->dragging = TRUE;
 		self->priv->drag_start_selection_area = self->priv->selection_area;
+		gtk_widget_queue_draw (GTK_WIDGET (self->priv->viewer));
 		retval = TRUE;
 	}
 
