@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GTH_IMAGE_SELECTOR_H
@@ -49,7 +48,8 @@ typedef enum {
 typedef enum {
 	GTH_GRID_NONE = 0,
 	GTH_GRID_THIRDS,
-	GTH_GRID_GOLDEN
+	GTH_GRID_GOLDEN,
+	GTH_GRID_CENTER
 } GthGridType;
 
 struct _GthImageSelector
@@ -102,6 +102,9 @@ gboolean              gth_image_selector_get_mask_visible     (GthImageSelector
 void                  gth_image_selector_set_grid_type        (GthImageSelector *selector,
                                                                GthGridType       grid_type);
 GthGridType           gth_image_selector_get_grid_type        (GthImageSelector *selector);
+void                  gth_image_selector_bind_dimensions      (GthImageSelector *selector,
+							       gboolean          bind,
+							       int               factor);
 
 G_END_DECLS
 
