@@ -25,7 +25,6 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include "gth-cursors.h"
 #include "gth-enum-types.h"
 #include "gth-image-dragger.h"
 #include "gth-image-viewer.h"
@@ -46,6 +45,7 @@
 			       * delay use this delay instead. */
 #define STEP_INCREMENT  20.0  /* Scroll increment. */
 
+
 enum {
 	CLICKED,
 	ZOOM_IN,
@@ -114,6 +114,7 @@ struct _GthImageViewerPrivate {
 	gboolean                skip_size_change;
 
 	gboolean                reset_scrollbars;
+	guint                   update_adjustment_values;
 
 	GList                  *painters;
 };
@@ -356,7 +357,7 @@ gth_image_viewer_realize (GtkWidget *widget)
 	gtk_widget_set_style (widget, style);
 
 	self->priv->cursor = gdk_cursor_new (GDK_LEFT_PTR);
-	self->priv->cursor_void = gth_cursor_get (window, GTH_CURSOR_VOID);
+	self->priv->cursor_void = gdk_cursor_new_for_display (gtk_widget_get_display (widget), GDK_BLANK_CURSOR);
 	if (self->priv->cursor_visible)
 		gdk_window_set_cursor (window, self->priv->cursor);
 	else
@@ -398,6 +399,11 @@ gth_image_viewer_unrealize (GtkWidget *widget)
 		self->priv->cursor_void = NULL;
 	}
 
+	if (self->priv->update_adjustment_values != 0) {
+		g_source_remove (self->priv->update_adjustment_values);
+		self->priv->update_adjustment_values = 0;
+	}
+
 	gth_image_viewer_tool_unrealize (self->priv->tool);
 
 	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
@@ -564,15 +570,15 @@ gth_image_viewer_size_allocate (GtkWidget     *widget,
 
 		if (width > gdk_width)
 			self->x_offset = CLAMP (self->x_offset,
-						  0,
-						  width - gdk_width);
+						0,
+						width - gdk_width);
 		else
 			self->x_offset = 0;
 
 		if (height > gdk_height)
 			self->y_offset = CLAMP (self->y_offset,
-						  0,
-						  height - gdk_height);
+						0,
+						height - gdk_height);
 		else
 			self->y_offset = 0;
 
@@ -583,40 +589,48 @@ gth_image_viewer_size_allocate (GtkWidget     *widget,
 
 		/* Change adjustment values. */
 
-		gtk_adjustment_set_lower (self->hadj, 0.0);
-		gtk_adjustment_set_upper (self->hadj, width);
-		gtk_adjustment_set_value (self->hadj, self->x_offset);
-		gtk_adjustment_set_step_increment (self->hadj, STEP_INCREMENT);
-		gtk_adjustment_set_page_increment (self->hadj, gdk_width / 2);
-		gtk_adjustment_set_page_size (self->hadj, gdk_width);
-
-		gtk_adjustment_set_lower (self->vadj, 0.0);
-		gtk_adjustment_set_upper (self->vadj, height);
-		gtk_adjustment_set_value (self->vadj, self->y_offset);
-		gtk_adjustment_set_step_increment (self->vadj, STEP_INCREMENT);
-		gtk_adjustment_set_page_increment (self->vadj, gdk_height / 2);
-		gtk_adjustment_set_page_size (self->vadj, gdk_height);
+		gtk_adjustment_configure (self->hadj,
+					  self->x_offset,
+					  0.0,
+					  width,
+					  STEP_INCREMENT,
+					  gdk_width / 2,
+					  gdk_width);
+		gtk_adjustment_configure (self->vadj,
+					  self->y_offset,
+					  0.0,
+					  height,
+					  STEP_INCREMENT,
+					  gdk_height / 2,
+					  gdk_height);
 	}
 	else {
-		gtk_adjustment_set_lower (self->hadj, 0.0);
-		gtk_adjustment_set_upper (self->hadj, 1.0);
-		gtk_adjustment_set_value (self->hadj, 0.0);
-		gtk_adjustment_set_page_size (self->hadj, 1.0);
-
-		gtk_adjustment_set_lower (self->vadj, 0.0);
-		gtk_adjustment_set_upper (self->vadj, 1.0);
-		gtk_adjustment_set_value (self->vadj, 0.0);
-		gtk_adjustment_set_page_size (self->vadj, 1.0);
+		gtk_adjustment_configure (self->hadj,
+					  0.0,
+					  0.0,
+					  1.0,
+					  0.1,
+					  1.0,
+					  1.0);
+		gtk_adjustment_configure (self->vadj,
+					  0.0,
+					  0.0,
+					  1.0,
+					  0.1,
+					  1.0,
+					  1.0);
 	}
 
 	_gth_image_viewer_update_image_area (self);
 
+	/* FIXME
 	g_signal_handlers_block_by_data (G_OBJECT (self->hadj), self);
 	g_signal_handlers_block_by_data (G_OBJECT (self->vadj), self);
 	gtk_adjustment_changed (self->hadj);
 	gtk_adjustment_changed (self->vadj);
 	g_signal_handlers_unblock_by_data (G_OBJECT (self->hadj), self);
 	g_signal_handlers_unblock_by_data (G_OBJECT (self->vadj), self);
+	*/
 
 	/**/
 
@@ -734,7 +748,6 @@ gth_image_viewer_expose (GtkWidget      *widget,
 	self = GTH_IMAGE_VIEWER (widget);
 
 	cr = gdk_cairo_create (gtk_widget_get_window (widget));
-
 	cairo_set_line_width (cr, 0.5);
 	cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
 
@@ -988,6 +1001,19 @@ scroll_to (GthImageViewer *self,
 	self->x_offset = *x_offset;
 	self->y_offset = *y_offset;
 
+	if (self->priv->painters != NULL) {
+		GdkRectangle  area;
+
+		area.x = 0;
+		area.y = 0;
+		area.width = allocation.width;
+		area.height = allocation.height;
+		gdk_window_invalidate_rect (drawable, &area, TRUE);
+		gdk_window_process_updates (drawable, TRUE);
+
+		return;
+	}
+
 	/* move without invalidating the frame */
 
 	{
@@ -1573,13 +1599,14 @@ gth_image_viewer_instance_init (GthImageViewer *self)
 	self->priv->cursor_void = NULL;
 
 	self->priv->reset_scrollbars = TRUE;
+	self->priv->update_adjustment_values = 0;
 
 	self->priv->tool = gth_image_dragger_new (self);
 
 	/* Create the widget. */
 
-	self->hadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 1.0, 0.0, 1.0, 1.0, 1.0));
-	self->vadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 1.0, 0.0, 1.0, 1.0, 1.0));
+	self->hadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 1.0, 0.0, 0.1, 1.0, 1.0));
+	self->vadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 1.0, 0.0, 0.1, 1.0, 1.0));
 
 	g_object_ref (self->hadj);
 	g_object_ref_sink (self->hadj);
@@ -1809,10 +1836,24 @@ gth_image_viewer_add_painter (GthImageViewer          *self,
 			      GthImageViewerPaintFunc  func,
 			      gpointer                 user_data)
 {
+	GList       *link;
+	GList       *scan;
 	PainterData *painter_data;
 
 	g_return_if_fail (self != NULL);
 
+	link = NULL;
+	for (scan = self->priv->painters; scan; scan = scan->next) {
+		PainterData *painter_data = scan->data;
+		if ((painter_data->func == func) && (painter_data->user_data == user_data)) {
+			link = scan;
+			break;
+		}
+	}
+
+	if (link != NULL)
+		return;
+
 	painter_data = g_new0 (PainterData, 1);
 	painter_data->func = func;
 	painter_data->user_data = user_data;
@@ -1820,6 +1861,32 @@ gth_image_viewer_add_painter (GthImageViewer          *self,
 }
 
 
+void
+gth_image_viewer_remove_painter (GthImageViewer          *self,
+				 GthImageViewerPaintFunc  func,
+				 gpointer                 user_data)
+{
+	GList *link;
+	GList *scan;
+
+	link = NULL;
+	for (scan = self->priv->painters; scan; scan = scan->next) {
+		PainterData *painter_data = scan->data;
+		if ((painter_data->func == func) && (painter_data->user_data == user_data)) {
+			link = scan;
+			break;
+		}
+	}
+
+	if (link == NULL)
+		return;
+
+	self->priv->painters = g_list_remove_link (self->priv->painters, link);
+	painter_data_free (link->data);
+	g_list_free (link);
+}
+
+
 int
 gth_image_viewer_get_image_width (GthImageViewer *self)
 {
@@ -2212,6 +2279,25 @@ gth_image_viewer_set_tool (GthImageViewer     *self,
 }
 
 
+static gboolean
+update_adjustment_values_cb (gpointer user_data)
+{
+	GthImageViewer *self = user_data;
+
+	g_source_remove (self->priv->update_adjustment_values);
+	self->priv->update_adjustment_values = 0;
+
+	g_signal_handlers_block_by_data (G_OBJECT (self->hadj), self);
+	g_signal_handlers_block_by_data (G_OBJECT (self->vadj), self);
+	gtk_adjustment_set_value (self->hadj, self->x_offset);
+	gtk_adjustment_set_value (self->vadj, self->y_offset);
+	g_signal_handlers_unblock_by_data (G_OBJECT (self->hadj), self);
+	g_signal_handlers_unblock_by_data (G_OBJECT (self->vadj), self);
+
+	return FALSE;
+}
+
+
 void
 gth_image_viewer_scroll_to (GthImageViewer *self,
 			    int             x_offset,
@@ -2224,12 +2310,9 @@ gth_image_viewer_scroll_to (GthImageViewer *self,
 
 	scroll_to (self, &x_offset, &y_offset);
 
-	g_signal_handlers_block_by_data (G_OBJECT (self->hadj), self);
-	g_signal_handlers_block_by_data (G_OBJECT (self->vadj), self);
-	gtk_adjustment_set_value (self->hadj, self->x_offset);
-	gtk_adjustment_set_value (self->vadj, self->y_offset);
-	g_signal_handlers_unblock_by_data (G_OBJECT (self->hadj), self);
-	g_signal_handlers_unblock_by_data (G_OBJECT (self->vadj), self);
+	if (self->priv->update_adjustment_values != 0)
+		g_source_remove (self->priv->update_adjustment_values);
+	self->priv->update_adjustment_values = g_timeout_add (10, update_adjustment_values_cb, self);
 }
 
 
@@ -2474,8 +2557,7 @@ gth_image_viewer_paint_region (GthImageViewer *self,
 	int           i;
 
 	cairo_save (cr);
-	gdk_cairo_region (cr, region);
-	cairo_clip (cr);
+
 	gdk_cairo_rectangle (cr, pixbuf_area);
 	cairo_clip (cr);
 
@@ -2511,7 +2593,10 @@ gth_image_viewer_apply_painters (GthImageViewer *self,
 
 	for (scan = self->priv->painters; scan; scan = scan->next) {
 		PainterData *painter_data = scan->data;
+
+		cairo_save (cr);
 		painter_data->func (self, event, cr, painter_data->user_data);
+		cairo_restore (cr);
 	}
 }
 
