@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -38,6 +37,12 @@
 #include "pixbuf-io.h"
 #include "typedefs.h"
 
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#endif
+
+#include "eggsmclient.h"
+#include "eggdesktopfile.h"
 
 static GStaticMutex register_mutex = G_STATIC_MUTEX_INIT;
 
@@ -96,6 +101,7 @@ struct _GthMainPrivate
 	GList               *metadata_provider;
 	GPtrArray           *metadata_category;
 	GPtrArray           *metadata_info;
+	GHashTable          *metadata_info_hash;
 	gboolean             metadata_info_sorted;
 	GHashTable          *sort_types;
 	GHashTable          *loaders;
@@ -121,6 +127,7 @@ gth_main_finalize (GObject *object)
 	if (gth_main->priv != NULL) {
 		_g_object_list_unref (gth_main->priv->file_sources);
 
+		g_hash_table_unref (gth_main->priv->metadata_info_hash);
 		g_ptr_array_free (gth_main->priv->metadata_category, TRUE);
 		g_ptr_array_free (gth_main->priv->metadata_info, TRUE);
 		g_list_foreach (gth_main->priv->metadata_provider, (GFunc) g_object_unref, NULL);
@@ -175,6 +182,7 @@ gth_main_init (GthMain *main)
 	main->priv->loaders = g_hash_table_new (g_str_hash, (GEqualFunc) g_content_type_equals);
 	main->priv->metadata_category = g_ptr_array_new ();
 	main->priv->metadata_info = g_ptr_array_new ();
+	main->priv->metadata_info_hash = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
 	main->priv->metadata_info_sorted = FALSE;
 }
 
@@ -254,8 +262,14 @@ gth_main_initialize (void)
 		return;
 	Main = (GthMain*) g_object_new (GTH_TYPE_MAIN, NULL);
 
+#ifdef GDK_WINDOWING_X11
+	egg_set_desktop_file (GTHUMB_DATADIR "/applications/gthumb.desktop");
+#else
+	/* manually set name and icon */
 	g_set_application_name (_("gthumb"));
 	gtk_window_set_default_icon_name ("gthumb");
+#endif
+
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), GTHUMB_ICON_DIR);
 	gtk_about_dialog_set_url_hook (about_dialog_activate_link_cb, NULL, NULL);
 	gtk_about_dialog_set_email_hook (about_dialog_activate_email_cb, NULL, NULL);
@@ -414,6 +428,7 @@ gth_main_register_metadata_info (GthMetadataInfo *metadata_info)
 
 	info = gth_metadata_info_dup (metadata_info);
 	g_ptr_array_add (Main->priv->metadata_info, info);
+	g_hash_table_insert (Main->priv->metadata_info_hash, (gpointer) info->id, info);
 	Main->priv->metadata_info_sorted = FALSE;
 
 	g_static_mutex_unlock (&metadata_info_mutex);
@@ -430,8 +445,10 @@ gth_main_register_metadata_info_v (GthMetadataInfo metadata_info[])
 	g_static_mutex_lock (&metadata_info_mutex);
 
 	for (i = 0; metadata_info[i].id != NULL; i++)
-		if ((metadata_info[i].display_name == NULL) || (strstr (metadata_info[i].display_name, "0x") == NULL))
+		if ((metadata_info[i].display_name == NULL) || (strstr (metadata_info[i].display_name, "0x") == NULL)) {
 			g_ptr_array_add (Main->priv->metadata_info, &metadata_info[i]);
+			g_hash_table_insert (Main->priv->metadata_info_hash, (gpointer) (&metadata_info[i])->id, &metadata_info[i]);
+		}
 
 	g_static_mutex_unlock (&metadata_info_mutex);
 }
@@ -489,6 +506,8 @@ gth_main_get_metadata_attributes (const char *mask)
 	GList                  *scan;
 	char                  **values;
 
+	g_static_mutex_lock (&metadata_info_mutex);
+
 	if (! Main->priv->metadata_info_sorted) {
 		g_ptr_array_sort (Main->priv->metadata_info, metadata_info_sort_func);
 		Main->priv->metadata_info_sorted = TRUE;
@@ -505,6 +524,8 @@ gth_main_get_metadata_attributes (const char *mask)
 	}
 	list = g_list_reverse (list);
 
+	g_static_mutex_unlock (&metadata_info_mutex);
+
 	values = g_new (char *, n + 1);
 	for (i = 0, scan = list; scan; i++, scan = scan->next)
 		values[i] = g_strdup (scan->data);
@@ -582,27 +603,35 @@ gth_main_get_metadata_category (const char *id)
 GthMetadataInfo *
 gth_main_get_metadata_info (const char *id)
 {
-	int i;
+	GthMetadataInfo *info;
 
 	if (id == NULL)
 		return NULL;
 
-	for (i = 0; i < Main->priv->metadata_info->len; i++) {
-		GthMetadataInfo *info;
+	info = g_hash_table_lookup (Main->priv->metadata_info_hash, id);
 
-		info = g_ptr_array_index (Main->priv->metadata_info, i);
-		if (strcmp (info->id, id) == 0)
-			return info;
-	}
-
-	return NULL;
+	return info;
 }
 
 
-GPtrArray *
+GList *
 gth_main_get_all_metadata_info (void)
 {
-	return Main->priv->metadata_info;
+	GList *list = NULL;
+	int    i;
+
+	g_static_mutex_lock (&metadata_info_mutex);
+
+	for (i = 0; i < Main->priv->metadata_info->len; i++) {
+		GthMetadataInfo *metadata_info = g_ptr_array_index (Main->priv->metadata_info, i);
+
+		list = g_list_prepend (list, metadata_info);
+	}
+	list = g_list_reverse (list);
+
+	g_static_mutex_unlock (&metadata_info_mutex);
+
+	return list;
 }
 
 
@@ -1200,6 +1229,7 @@ gth_main_activate_extensions (void)
 						"catalogs",
 						"change_date",
 						"comments",
+						"contact_sheet",
 						"convert_format",
 						"desktop_background",
 						"edit_metadata",
@@ -1207,6 +1237,7 @@ gth_main_activate_extensions (void)
 						"facebook",
 						"file_manager",
 						"file_tools",
+						"find_duplicates",
 						"flicker",
 						"gstreamer_tools",
 						"gstreamer_utils",
@@ -1227,32 +1258,52 @@ gth_main_activate_extensions (void)
 						"resize_images",
 						"search",
 						"slideshow",
+						"webalbums",
 						NULL };
-	int         i;
-	GSList     *active_extensions;
-	GSList     *scan;
+	int                  i;
+	GError              *error = NULL;
+	GSList              *active_extensions;
+	GthExtensionManager *manager;
+	GSList              *scan;
 
 	if (Main->priv->extension_manager == NULL)
 		Main->priv->extension_manager = gth_extension_manager_new ();
 
 	for (i = 0; mandatory_extensions[i] != NULL; i++) {
-		GError *error = NULL;
-
 		if (! gth_extension_manager_activate (Main->priv->extension_manager, mandatory_extensions[i], &error)) {
 			g_warning ("Could not load the '%s' extension: %s", mandatory_extensions[i], error->message);
 			g_clear_error (&error);
 		}
 	}
 
-	active_extensions = eel_gconf_get_string_list (PREF_ACTIVE_EXTENSIONS);
-	if (active_extensions == NULL)
+	active_extensions = eel_gconf_get_string_list_with_error (PREF_ACTIVE_EXTENSIONS, &error);
+	if ((error != NULL)
+	    || ((active_extensions != NULL)
+	        && (active_extensions->next == NULL)
+	        && (g_strcmp0 (active_extensions->data, "default") == 0)))
+	{
+		g_clear_error (&error);
+		g_slist_foreach (active_extensions, (GFunc) g_free, NULL);
+		g_slist_free (active_extensions);
+
+		active_extensions = NULL;
 		for (i = 0; default_extensions[i] != NULL; i++)
 			active_extensions = g_slist_prepend (active_extensions, g_strdup (default_extensions[i]));
 
+		eel_gconf_set_string_list (PREF_ACTIVE_EXTENSIONS, active_extensions);
+	}
 	active_extensions = gth_extension_manager_order_extensions (Main->priv->extension_manager, active_extensions);
+
+	manager = gth_main_get_default_extension_manager ();
+
 	for (scan = active_extensions; scan; scan = scan->next) {
-		char   *name = scan->data;
-		GError *error = NULL;
+		char                    *name = scan->data;
+		GthExtensionDescription *description;
+		GError                  *error = NULL;
+
+		description = gth_extension_manager_get_description (manager, name);
+		if ((description != NULL) && (description->hidden || description->mandatory))
+			continue;
 
 		if (! gth_extension_manager_activate (Main->priv->extension_manager, name, &error)) {
 			g_warning ("Could not load the '%s' extension: %s", name, error->message);
@@ -1325,8 +1376,11 @@ attribute_list_reload_required (const char *old_attributes,
 		g_object_ref (provider);
 	}
 
-	/*g_print ("attributes to load: %s\n", new_attributes);
-	g_print ("attributes not available: \n");*/
+	/*
+	g_print ("old attributes: %s\n", old_attributes);
+	g_print ("new attributes: %s\n", new_attributes);
+	g_print ("attributes not available: \n");
+	*/
 
 	reload_required = FALSE;
 	for (i = 0; ! reload_required && (i < new_attributes_len); i++)
@@ -1335,9 +1389,11 @@ attribute_list_reload_required (const char *old_attributes,
 		    && (strcmp (new_attributes_v[i], "none") != 0))
 		{
 			reload_required = TRUE;
-			/*g_print ("\t%s\n", new_attributes_v[i]);*/
+			/* g_print ("\t%s\n", new_attributes_v[i]); */
 		}
 
+	/* g_print ("reload required: %d\n", reload_required); */
+
 	g_strfreev (new_attributes_v);
 	g_strfreev (old_attributes_v);
 
