@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GTH_PREF_H
@@ -36,6 +35,9 @@ G_BEGIN_DECLS
 #define PREF_STARTUP_LOCATION               "/apps/gthumb/general/startup_location"
 #define PREF_ACTIVE_EXTENSIONS              "/apps/gthumb/general/active_extensions"
 #define PREF_STORE_METADATA_IN_FILES        "/apps/gthumb/general/store_metadata_in_files"
+
+#define PREF_DATA_MIGRATION_CATALOGS_2_10   "/apps/gthumb/data_migration/catalogs_2_10"
+
 #define PREF_GENERAL_FILTER                 "/apps/gthumb/browser/general_filter"
 #define PREF_SHOW_HIDDEN_FILES              "/apps/gthumb/browser/show_hidden_files"
 #define PREF_SHOW_THUMBNAILS                "/apps/gthumb/browser/show_thumbnails"
@@ -81,7 +83,7 @@ G_BEGIN_DECLS
 /* default values */
 
 #define DEFAULT_GENERAL_FILTER "file::type::is_media"
-#define DEFAULT_THUMBNAIL_CAPTION "standard::display-name,gth::file::display-size"
+#define DEFAULT_THUMBNAIL_CAPTION "comment::note,comment::time"
 #define DEFAULT_UI_WINDOW_WIDTH 690
 #define DEFAULT_UI_WINDOW_HEIGHT 460
 #define DEFAULT_FAST_FILE_TYPE TRUE
