@@ -140,13 +140,13 @@ gth_thumb_loader_get_type (void)
 
 
 static GdkPixbufAnimation *
-load_thumbnail (GthFileData   *file_data,
-	        int            requested_size,
-	        int           *original_width,
-	        int           *original_height,
-	        gpointer       user_data,
-	        GCancellable  *cancellable,
-	        GError       **error)
+generate_thumbnail (GthFileData   *file_data,
+		    int            requested_size,
+		    int           *original_width,
+		    int           *original_height,
+		    gpointer       user_data,
+		    GCancellable  *cancellable,
+		    GError       **error)
 {
 	GthThumbLoader     *self = user_data;
 	GdkPixbuf          *pixbuf = NULL;
@@ -163,7 +163,16 @@ load_thumbnail (GthFileData   *file_data,
 	uri = g_file_get_uri (file_data->file);
 	pixbuf = gnome_desktop_thumbnail_factory_generate_no_script (self->priv->thumb_factory,
 								     uri,
-								     gth_file_data_get_mime_type (file_data));
+								     gth_file_data_get_mime_type (file_data),
+								     cancellable);
+
+	if (g_cancellable_is_cancelled (cancellable)) {
+		_g_object_unref (pixbuf);
+		if (error != NULL)
+			*error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_CANCELLED, "");
+		return NULL;
+	}
+
 	if (pixbuf == NULL) {
 		PixbufLoader thumbnailer;
 
@@ -179,13 +188,20 @@ load_thumbnail (GthFileData   *file_data,
 	}
 
 	if (pixbuf != NULL) {
-		g_clear_error (error);
+		if (original_width != NULL)
+			*original_width = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (pixbuf), "gnome-original-width"));
+		if (original_height != NULL)
+			*original_height = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (pixbuf), "gnome-original-height"));
+
+		if (error != NULL)
+			g_clear_error (error);
 		animation = gdk_pixbuf_non_anim_new (pixbuf);
 		g_object_unref (pixbuf);
 	}
 
 	if (animation == NULL)
-		*error = g_error_new_literal (GTH_ERROR, 0, "Cannot generate the thumbnail");
+		if (error != NULL)
+			*error = g_error_new_literal (GTH_ERROR, 0, "Cannot generate the thumbnail");
 
 	g_free (uri);
 
@@ -193,13 +209,39 @@ load_thumbnail (GthFileData   *file_data,
 }
 
 
+static GdkPixbufAnimation *
+load_cached_thumbnail (GthFileData   *file_data,
+		       int            requested_size,
+		       int           *original_width,
+		       int           *original_height,
+		       gpointer       user_data,
+		       GCancellable  *cancellable,
+		       GError       **error)
+{
+	GdkPixbufAnimation *animation = NULL;
+	char               *filename;
+	GdkPixbuf          *pixbuf;
+
+	filename = g_file_get_path (file_data->file);
+	pixbuf = gdk_pixbuf_new_from_file (filename, error);
+	if (pixbuf != NULL) {
+		animation = gdk_pixbuf_non_anim_new (pixbuf);
+		g_object_unref (pixbuf);
+	}
+
+	g_free (filename);
+
+	return animation;
+}
+
+
 static void
 gth_thumb_loader_construct (GthThumbLoader *self,
 			    int             requested_size)
 {
 	gth_thumb_loader_set_requested_size (self, requested_size);
-	self->priv->tloader = gth_image_loader_new (load_thumbnail, self);
-	self->priv->iloader = gth_image_loader_new (NULL, NULL);
+	self->priv->tloader = gth_image_loader_new (generate_thumbnail, self);
+	self->priv->iloader = gth_image_loader_new (load_cached_thumbnail, NULL);
 }
 
 
@@ -219,8 +261,8 @@ void
 gth_thumb_loader_set_loader_func (GthThumbLoader *self,
 			          PixbufLoader    loader_func)
 {
-	gth_image_loader_set_loader_func (self->priv->iloader,
-					  (loader_func != NULL) ? loader_func : load_thumbnail,
+	gth_image_loader_set_loader_func (self->priv->tloader,
+					  (loader_func != NULL) ? loader_func : generate_thumbnail,
 					  self);
 }
 
@@ -229,21 +271,23 @@ void
 gth_thumb_loader_set_requested_size (GthThumbLoader *self,
 				     int             size)
 {
-	if (self->priv->thumb_factory != NULL) {
-		g_object_unref (self->priv->thumb_factory);
-		self->priv->thumb_factory = NULL;
-	}
+	GnomeDesktopThumbnailSize thumb_size;
 
 	self->priv->requested_size = size;
 	if (self->priv->requested_size <= THUMBNAIL_NORMAL_SIZE) {
 		self->priv->cache_max_size = THUMBNAIL_NORMAL_SIZE;
-		self->priv->thumb_size = GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL;
+		thumb_size = GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL;
 	}
 	else {
 		self->priv->cache_max_size = THUMBNAIL_LARGE_SIZE;
-		self->priv->thumb_size = GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE;
+		thumb_size = GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE;
+	}
+
+	if ((self->priv->thumb_size != thumb_size) || (self->priv->thumb_factory == NULL)) {
+		self->priv->thumb_size = thumb_size;
+		_g_object_unref (self->priv->thumb_factory);
+		self->priv->thumb_factory = gnome_desktop_thumbnail_factory_new (self->priv->thumb_size);
 	}
-	self->priv->thumb_factory = gnome_desktop_thumbnail_factory_new (self->priv->thumb_size);
 }
 
 
@@ -436,6 +480,21 @@ cache_image_ready_cb (GObject      *source_object,
 }
 
 
+static gboolean
+is_a_cache_file (const char *uri)
+{
+	char     *cache_base_uri;
+	gboolean  result;
+
+	cache_base_uri = g_strconcat (get_home_uri (), "/.thumbnails", NULL);
+	result = _g_uri_parent_of_uri (cache_base_uri, uri);
+
+	g_free (cache_base_uri);
+
+	return result;
+}
+
+
 static gboolean
 _gth_thumb_loader_save_to_cache (GthThumbLoader *self,
 				 GthFileData    *file_data,
@@ -452,18 +511,14 @@ _gth_thumb_loader_save_to_cache (GthThumbLoader *self,
 	uri = g_file_get_uri (file_data->file);
 
 	if (g_file_is_native (file_data->file)) {
-		char *cache_base_uri;
 
 		/* Do not save thumbnails from the user's thumbnail directory,
 		   or an endless loop of thumbnailing may be triggered. */
 
-		cache_base_uri = g_strconcat (get_home_uri (), "/.thumbnails", NULL);
-		if (_g_uri_parent_of_uri (cache_base_uri, uri)) {
-			g_free (cache_base_uri);
+		if (is_a_cache_file (uri)) {
 			g_free (uri);
 			return FALSE;
 		}
-		g_free (cache_base_uri);
 	}
 
 	cache_path = gnome_desktop_thumbnail_path_for_uri (uri, self->priv->thumb_size);
@@ -742,6 +797,7 @@ gth_thumb_loader_load (GthThumbLoader      *self,
 {
 	GSimpleAsyncResult *simple;
 	char               *cache_path;
+	char               *uri;
 	LoadData           *load_data;
 
 	simple = g_simple_async_result_new (G_OBJECT (self),
@@ -750,11 +806,15 @@ gth_thumb_loader_load (GthThumbLoader      *self,
 					    gth_thumb_loader_load);
 
 	cache_path = NULL;
-	if (self->priv->use_cache) {
-		char   *uri;
-		time_t  mtime;
 
-		uri = g_file_get_uri (file_data->file);
+	uri = g_file_get_uri (file_data->file);
+
+	if (is_a_cache_file (uri)) {
+		cache_path = g_file_get_path (file_data->file);
+	}
+	else if (self->priv->use_cache) {
+		time_t mtime;
+
 		mtime = gth_file_data_get_mtime (file_data);
 
 		if (gnome_desktop_thumbnail_factory_has_valid_failed_thumbnail (self->priv->thumb_factory, uri, mtime)) {
@@ -771,10 +831,10 @@ gth_thumb_loader_load (GthThumbLoader      *self,
 		}
 
 		cache_path = gnome_desktop_thumbnail_factory_lookup (self->priv->thumb_factory, uri, mtime);
-
-		g_free (uri);
 	}
 
+	g_free (uri);
+
 	if ((cache_path == NULL)
 	    && (self->priv->max_file_size > 0)
 	    && (g_file_info_get_size (file_data->info) > self->priv->max_file_size))
