@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -60,7 +59,7 @@ create_button (const char *stock_id,
 	hbox = gtk_hbox_new (FALSE, 2);
 	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
 
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (button, TRUE);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
 
 	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
@@ -604,7 +603,6 @@ _gtk_error_dialog_from_gerror_show (GtkWindow   *parent,
 }
 
 
-
 void
 _gtk_error_dialog_run (GtkWindow        *parent,
 		       const gchar      *format,
@@ -635,6 +633,35 @@ _gtk_error_dialog_run (GtkWindow        *parent,
 }
 
 
+void
+_gtk_error_dialog_show (GtkWindow  *parent,
+			const char *title,
+			const char *format,
+			...)
+{
+	va_list    args;
+	char      *message;
+	GtkWidget *d;
+
+	va_start (args, format);
+	message = g_strdup_vprintf (format, args);
+	va_end (args);
+
+	d = _gtk_message_dialog_new (parent,
+				     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				     GTK_STOCK_DIALOG_ERROR,
+				     title,
+				     message,
+				     GTK_STOCK_OK, GTK_RESPONSE_OK,
+				     NULL);
+	g_signal_connect (d, "response", G_CALLBACK (error_dialog_response_cb), NULL);
+
+	gtk_window_present (GTK_WINDOW (d));
+
+	g_free (message);
+}
+
+
 void
 _gtk_info_dialog_run (GtkWindow        *parent,
 		      const gchar      *format,
@@ -677,6 +704,8 @@ get_themed_icon_pixbuf (GThemedIcon  *icon,
 
 	g_object_get (icon, "names", &icon_names, NULL);
 
+	if (icon_theme == NULL)
+		icon_theme = gtk_icon_theme_get_default ();
 	icon_info = gtk_icon_theme_choose_icon (icon_theme, (const char **)icon_names, size, 0);
 	if (icon_info == NULL)
 		icon_info = gtk_icon_theme_lookup_icon (icon_theme, "text-x-generic", size, GTK_ICON_LOOKUP_USE_BUILTIN);
@@ -998,15 +1027,15 @@ _gtk_widget_get_screen_size (GtkWidget *widget,
 			     int       *height)
 {
 	GdkScreen    *screen;
-	GdkRectangle  screen_geom;
+	GdkRectangle  geometry;
 
 	screen = gtk_widget_get_screen (widget);
 	gdk_screen_get_monitor_geometry (screen,
-					 gdk_screen_get_monitor_at_window (screen, widget->window),
-					 &screen_geom);
+					 gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (widget)),
+					 &geometry);
 
-	*width = screen_geom.width;
-	*height = screen_geom.height;
+	*width = geometry.width;
+	*height = geometry.height;
 }
 
 
@@ -1026,7 +1055,7 @@ _gtk_paned_get_position2 (GtkPaned *paned)
 	GtkRequisition  requisition;
 	int             size;
 
-	if (! GTK_WIDGET_VISIBLE (paned))
+	if (! gtk_widget_get_visible (GTK_WIDGET (paned)))
 		return 0;
 
 	pos = gtk_paned_get_position (paned);
@@ -1034,7 +1063,7 @@ _gtk_paned_get_position2 (GtkPaned *paned)
 		return 0;
 
 	w = gtk_widget_get_toplevel (GTK_WIDGET (paned));
-	if (! GTK_WIDGET_TOPLEVEL (w))
+	if (! gtk_widget_is_toplevel (w))
 		return 0;
 
 	gtk_window_get_size (GTK_WINDOW (w), &(requisition.width), &(requisition.height));
@@ -1054,17 +1083,19 @@ void
 _gtk_paned_set_position2 (GtkPaned *paned,
 			  int       pos)
 {
-	GtkWidget *top_level;
-	int        size;
+	GtkWidget     *top_level;
+	GtkAllocation  allocation;
+	int            size;
 
 	top_level = gtk_widget_get_toplevel (GTK_WIDGET (paned));
-	if (! GTK_WIDGET_TOPLEVEL (top_level))
+	if (! gtk_widget_is_toplevel (top_level))
 		return;
 
+	gtk_widget_get_allocation (top_level, &allocation);
 	if (gtk_orientable_get_orientation (GTK_ORIENTABLE (paned)) == GTK_ORIENTATION_HORIZONTAL)
-		size = top_level->allocation.width;
+		size = allocation.width;
 	else
-		size = top_level->allocation.height;
+		size = allocation.height;
 
 	if (pos > 0)
 		gtk_paned_set_position (paned, size - pos);
@@ -1097,3 +1128,143 @@ _g_launch_command (GtkWidget  *parent,
 
 	g_object_unref (app_info);
 }
+
+
+void
+_gtk_window_resize_to_fit_screen_height (GtkWidget *window,
+					 int        default_width)
+{
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (window);
+	if ((screen != NULL) && (gdk_screen_get_height (screen) < 768))
+		/* maximize on netbooks */
+		gtk_window_maximize (GTK_WINDOW (window));
+	else
+		/* This should fit on a XGA/WXGA (height 768) screen
+		 * with top and bottom panels */
+		gtk_window_set_default_size (GTK_WINDOW (window), default_width, 670);
+}
+
+
+void
+_gtk_info_bar_clear_action_area (GtkInfoBar *info_bar)
+{
+	_gtk_container_remove_children (GTK_CONTAINER (gtk_info_bar_get_action_area (info_bar)), NULL, NULL);
+}
+
+
+/* -- _gtk_menu_ask_drag_drop_action -- */
+
+
+typedef struct {
+	GMainLoop     *loop;
+	GdkDragAction  action;
+} DropActionData;
+
+
+static void
+ask_drag_drop_action_menu_deactivate_cb (GtkMenuShell *menushell,
+					 gpointer      user_data)
+{
+	DropActionData *drop_data = user_data;
+
+	if (g_main_loop_is_running (drop_data->loop))
+		g_main_loop_quit (drop_data->loop);
+}
+
+
+static void
+ask_drag_drop_action_item_activate_cb (GtkMenuItem *menuitem,
+                		       gpointer     user_data)
+{
+	DropActionData *drop_data = user_data;
+
+	drop_data->action = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menuitem), "drop-action"));
+	if (g_main_loop_is_running (drop_data->loop))
+		g_main_loop_quit (drop_data->loop);
+}
+
+
+static void
+_gtk_menu_ask_drag_drop_action_append_item (GtkWidget      *menu,
+					    const char     *label,
+					    GdkDragAction   actions,
+					    GdkDragAction   action,
+					    DropActionData *drop_data)
+{
+	GtkWidget *item;
+
+	item = gtk_menu_item_new_with_mnemonic (label);
+	g_object_set_data (G_OBJECT (item), "drop-action", GINT_TO_POINTER (action));
+	gtk_widget_set_sensitive (item, ((actions & action) != 0));
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	g_signal_connect (item,
+			  "activate",
+			  G_CALLBACK (ask_drag_drop_action_item_activate_cb),
+			  drop_data);
+}
+
+
+GdkDragAction
+_gtk_menu_ask_drag_drop_action (GtkWidget     *widget,
+				GdkDragAction  actions,
+				guint32        activate_time)
+{
+	DropActionData  drop_data;
+	GtkWidget      *menu;
+	GtkWidget      *item;
+
+	drop_data.action = 0;
+	drop_data.loop = g_main_loop_new (NULL, FALSE);
+
+	menu = gtk_menu_new ();
+	gtk_menu_set_screen (GTK_MENU (menu), gtk_widget_get_screen (widget));
+
+	_gtk_menu_ask_drag_drop_action_append_item (menu,
+						    _("_Copy Here"),
+						    actions,
+						    GDK_ACTION_COPY,
+						    &drop_data);
+	_gtk_menu_ask_drag_drop_action_append_item (menu,
+						    _("_Move Here"),
+						    actions,
+						    GDK_ACTION_MOVE,
+						    &drop_data);
+	_gtk_menu_ask_drag_drop_action_append_item (menu,
+						    _("_Link Here"),
+						    actions,
+						    GDK_ACTION_LINK,
+						    &drop_data);
+
+	item = gtk_separator_menu_item_new ();
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	item = gtk_menu_item_new_with_label (_("Cancel"));
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	g_signal_connect (menu,
+        		  "deactivate",
+                          G_CALLBACK (ask_drag_drop_action_menu_deactivate_cb),
+                          &drop_data);
+
+	gtk_menu_popup (GTK_MENU (menu),
+			NULL,
+			NULL,
+			NULL,
+			NULL,
+			0,
+			activate_time);
+	gtk_grab_add (menu);
+	g_main_loop_run (drop_data.loop);
+
+	gtk_grab_remove (menu);
+	gtk_widget_destroy (menu);
+	g_main_loop_unref (drop_data.loop);
+
+	return drop_data.action;
+}
