@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001-2009 Free Software Foundation, Inc.
+ *  Copyright (C) 2001-2010 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -36,6 +35,7 @@
 #include "gth-file-source-vfs.h"
 #include "gth-main.h"
 #include "gth-preferences.h"
+#include "main-migrate.h"
 
 
 enum {
@@ -90,6 +90,14 @@ static const GOptionEntry options[] = {
 };
 
 
+
+static void
+migrate_data (void)
+{
+	migrate_catalogs_from_2_10 ();
+}
+
+
 static void
 gth_save_state (EggSMClient *client,
 		GKeyFile    *state,
@@ -105,16 +113,22 @@ gth_save_state (EggSMClient *client,
 
 	i = 0;
 	for (scan = gth_window_get_window_list (); scan; scan = scan->next) {
-		GtkWidget *window = scan->data;
-		GFile     *location;
-		char      *key;
-		char      *uri;
+		GtkWidget   *window = scan->data;
+		GFile       *location;
+		char        *key;
+		char        *uri;
+		GthFileData *focused_file = NULL;
+
+		focused_file = gth_browser_get_current_file (GTH_BROWSER (window));
+		if (focused_file == NULL)
+			location = gth_browser_get_location (GTH_BROWSER (window));
+		else
+			location = focused_file->file;
 
-		location = gth_browser_get_location (GTH_BROWSER (window));
 		if (location == NULL)
 			continue;
 
-		key = g_strdup_printf ("location%d", i++);
+		key = g_strdup_printf ("location%d", ++i);
 		uri = g_file_get_uri (location);
 		g_key_file_set_string (state, "Session", key, uri);
 
@@ -127,13 +141,87 @@ gth_save_state (EggSMClient *client,
 }
 
 
+/* quit_requested handler for the master client */
+
+
+static GList *client_window = NULL;
+
+
+static void modified_file_saved_cb (GthBrowser  *browser,
+				    gboolean     cancelled,
+				    gpointer     user_data);
+
+
+static void
+check_whether_to_save (EggSMClient *client)
+{
+	for (/* void */; client_window; client_window = client_window->next) {
+		GtkWidget *window = client_window->data;
+
+		if (gth_browser_get_file_modified (GTH_BROWSER (window))) {
+			gth_browser_ask_whether_to_save (GTH_BROWSER (window),
+							 modified_file_saved_cb,
+							 client);
+			return;
+		}
+	}
+
+	egg_sm_client_will_quit (client, TRUE);
+}
+
+
+static void
+modified_file_saved_cb (GthBrowser *browser,
+			gboolean    cancelled,
+			gpointer    user_data)
+{
+	EggSMClient *client = user_data;
+
+	if (cancelled) {
+		egg_sm_client_will_quit (client, FALSE);
+	}
+	else {
+		client_window = client_window->next;
+		check_whether_to_save (client);
+	}
+}
+
+
+static void
+client_quit_requested_cb (EggSMClient *client,
+			  gpointer     data)
+{
+	client_window = gth_window_get_window_list ();
+	check_whether_to_save (client);
+}
+
+
+static void
+client_quit_cb (EggSMClient *client,
+		gpointer     data)
+{
+	gtk_main_quit ();
+}
+
+
 static void
 gth_session_manager_init (void)
 {
 	EggSMClient *client = NULL;
 
 	client = egg_sm_client_get ();
-	g_signal_connect (client, "save-state", G_CALLBACK (gth_save_state), NULL);
+	g_signal_connect (client,
+			  "save_state",
+			  G_CALLBACK (gth_save_state),
+			  NULL);
+	g_signal_connect (client,
+			  "quit_requested",
+			  G_CALLBACK (client_quit_requested_cb),
+			  NULL);
+	g_signal_connect (client,
+			  "quit",
+			  G_CALLBACK (client_quit_cb),
+			  NULL);
 }
 
 
@@ -141,23 +229,31 @@ static void
 gth_restore_session (EggSMClient *client)
 {
 	GKeyFile *state = NULL;
-	guint i;
+	guint     i;
 
 	state = egg_sm_client_get_state_file (client);
 
 	i = g_key_file_get_integer (state, "Session", "locations", NULL);
+	g_assert (i > 0);
 	for (; i > 0; i--) {
 		GtkWidget *window;
 		char      *key;
 		char      *location;
+		GFile     *file;
 
 		key = g_strdup_printf ("location%d", i);
 		location = g_key_file_get_string (state, "Session", key, NULL);
 		g_free (key);
 
-		window = gth_browser_new (location);
+		g_assert (location != NULL);
+
+		window = gth_browser_new (NULL);
 		gtk_widget_show (window);
 
+		file = g_file_new_for_uri (location);
+		gth_browser_load_location(GTH_BROWSER (window), file);
+
+		g_object_unref (file);
 		g_free (location);
 	}
 }
@@ -184,17 +280,19 @@ unique_app_message_received_cb (UniqueApp         *unique_app,
 	UniqueResponse  res;
 	char           *uri;
 	GFile          *location = NULL;
-	GtkWidget      *window;
+	GtkWidget      *window = NULL;
 
 	res = UNIQUE_RESPONSE_OK;
 
 	switch (command) {
 	case UNIQUE_OPEN:
 	case UNIQUE_NEW:
-		if (command == UNIQUE_NEW)
-			window = gth_browser_new (NULL);
-		else
+		if (command == UNIQUE_OPEN)
 			window = gth_window_get_current_window ();
+
+		if (window == NULL)
+			window = gth_browser_new (NULL);
+
 		show_window (GTK_WINDOW (window),
 			     unique_message_data_get_startup_id (message),
 			     unique_message_data_get_screen (message));
@@ -384,7 +482,7 @@ prepare_application (void)
 int
 main (int argc, char *argv[])
 {
-	GOptionContext *context = NULL;
+	GOptionContext *context;
 	GError         *error = NULL;
 
 	if (! g_thread_supported ())
@@ -416,19 +514,20 @@ main (int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
+	if (version) {
+		g_printf ("%s %s, Copyright © 2001-2010 Free Software Foundation, Inc.\n", PACKAGE_NAME, PACKAGE_VERSION);
+		return 0;
+	}
+
 #ifdef HAVE_CLUTTER
 	ClutterInitResult = gtk_clutter_init (NULL, NULL);
 #endif
 
-	if (version) {
-		g_printf ("%s %s, Copyright (C) 2001-2010 Free Software Foundation, Inc.\n", PACKAGE_NAME, PACKAGE_VERSION);
-		return 0;
-	}
-
 	/* other initializations */
 
 	gth_session_manager_init ();
 	gth_pref_initialize ();
+	migrate_data ();
 	gth_main_initialize ();
 	prepare_application ();
 
